@@ -3,7 +3,7 @@
  */
 
 /*
- * Copyright (C) 2006-2009 Sun Microsystems, Inc.
+ * Copyright (C) 2006-2010 Oracle Corporation
  *
  * This file is part of VirtualBox Open Source Edition (OSE), as
  * available from http://www.virtualbox.org. This file is free software;
@@ -21,10 +21,6 @@
  *
  * You may elect to license modified versions of this file under the
  * terms and conditions of either the GPL or the CDDL or both.
- *
- * Please contact Sun Microsystems, Inc., 4150 Network Circle, Santa
- * Clara, CA 95054 USA or visit http://www.sun.com if you need
- * additional information or have any questions.
  */
 
 #ifndef __VBoxHDD_Internal_h__
@@ -470,12 +466,11 @@ typedef struct VBOXHDDBACKEND
      * @param   pvBackendData   Opaque state data for this image.
      * @param   uOffset         The offset of the virtual disk to read from.
      * @param   cbRead          How many bytes to read.
-     * @param   paSeg           Pointer to the segment array.
-     * @param   cSeg            Number of segments.
-     * @param   pvUser          Opaque user data.
+     * @param   pIoCtx          I/O context associated with this request.
+     * @param   pcbActuallyRead Pointer to returned number of bytes read.
      */
     DECLR3CALLBACKMEMBER(int, pfnAsyncRead, (void *pvBackendData, uint64_t uOffset, size_t cbRead,
-                                             PPDMDATASEG paSeg, unsigned cSeg, void *pvUser));
+                                             PVDIOCTX pIoCtx, size_t *pcbActuallyRead));
 
     /**
      * Start an asynchronous write request.
@@ -484,12 +479,33 @@ typedef struct VBOXHDDBACKEND
      * @param   pvBackendData   Opaque state data for this image.
      * @param   uOffset         The offset of the virtual disk to write to.
      * @param   cbWrite         How many bytes to write.
-     * @param   paSeg           Pointer to the segment array.
-     * @param   cSeg            Number of segments.
-     * @param   pvUser          Oaque user data-
+     * @param   pIoCtx          I/O context associated with this request.
+     * @param   pcbWriteProcess Pointer to returned number of bytes that could
+     *                          be processed. In case the function returned
+     *                          VERR_VD_BLOCK_FREE this is the number of bytes
+     *                          that could be written in a full block write,
+     *                          when prefixed/postfixed by the appropriate
+     *                          amount of (previously read) padding data.
+     * @param   pcbPreRead      Pointer to the returned amount of data that must
+     *                          be prefixed to perform a full block write.
+     * @param   pcbPostRead     Pointer to the returned amount of data that must
+     *                          be postfixed to perform a full block write.
+     * @param   fWrite          Flags which affect write behavior. Combination
+     *                          of the VD_WRITE_* flags.
      */
     DECLR3CALLBACKMEMBER(int, pfnAsyncWrite, (void *pvBackendData, uint64_t uOffset, size_t cbWrite,
-                                              PPDMDATASEG paSeg, unsigned cSeg, void *pvUser));
+                                              PVDIOCTX pIoCtx,
+                                              size_t *pcbWriteProcess, size_t *pcbPreRead,
+                                              size_t *pcbPostRead, unsigned fWrite));
+
+    /**
+     * Flush data to disk.
+     *
+     * @returns VBox status code.
+     * @param   pvBackendData   Opaque state data for this image.
+     * @param   pIoCtx          I/O context associated with this request.
+     */
+    DECLR3CALLBACKMEMBER(int, pfnAsyncFlush, (void *pvBackendData, PVDIOCTX pIoCtx));
 
     /** Returns a human readable hard disk location string given a
      *  set of hard disk configuration keys. The returned string is an
@@ -513,10 +529,14 @@ typedef struct VBOXHDDBACKEND
      * @param   pvBackendData   Opaque state data for this image.
      * @param   uPercentStart   Starting value for progress percentage.
      * @param   uPercentSpan    Span for varying progress percentage.
+     * @param   pVDIfsDisk      Pointer to the per-disk VD interface list.
+     * @param   pVDIfsImage     Pointer to the per-image VD interface list.
      * @param   pVDIfsOperation Pointer to the per-operation VD interface list.
      */
     DECLR3CALLBACKMEMBER(int, pfnCompact, (void *pvBackendData,
                                            unsigned uPercentStart, unsigned uPercentSpan,
+                                           PVDINTERFACE pVDIfsDisk,
+                                           PVDINTERFACE pVDIfsImage,
                                            PVDINTERFACE pVDIfsOperation));
 
 } VBOXHDDBACKEND;
@@ -547,7 +567,7 @@ typedef VBOXHDDFORMATLOAD *PFNVBOXHDDFORMATLOAD;
 
 /** The prefix to identify Storage Plugins. */
 #define VBOX_HDDFORMAT_PLUGIN_PREFIX "VBoxHDD"
-/** The size of the prefix excluding the '\0' terminator. */
+/** The size of the prefix excluding the '\\0' terminator. */
 #define VBOX_HDDFORMAT_PLUGIN_PREFIX_LENGTH (sizeof(VBOX_HDDFORMAT_PLUGIN_PREFIX)-1)
 
 #endif
