@@ -3,7 +3,7 @@
  */
 
 /*
- * Copyright (C) 2006-2007 Sun Microsystems, Inc.
+ * Copyright (C) 2006-2007 Oracle Corporation
  *
  * This file is part of VirtualBox Open Source Edition (OSE), as
  * available from http://www.virtualbox.org. This file is free software;
@@ -21,10 +21,6 @@
  *
  * You may elect to license modified versions of this file under the
  * terms and conditions of either the GPL or the CDDL or both.
- *
- * Please contact Sun Microsystems, Inc., 4150 Network Circle, Santa
- * Clara, CA 95054 USA or visit http://www.sun.com if you need
- * additional information or have any questions.
  */
 
 #ifndef ___VBox_log_h
@@ -65,6 +61,8 @@ typedef enum LOGGROUP
     LOG_GROUP_DEFAULT = RTLOGGROUP_FIRST_USER,
     /** CFGM group. */
     LOG_GROUP_CFGM,
+    /** Core Dumper group. */
+    LOG_GROUP_CORE_DUMPER,
     /** CPUM group. */
     LOG_GROUP_CPUM,
     /** CSAM group. */
@@ -157,6 +155,8 @@ typedef enum LOGGROUP
     LOG_GROUP_DRV_BLOCK,
     /** Char driver group. */
     LOG_GROUP_DRV_CHAR,
+    /** Disk integrity driver group. */
+    LOG_GROUP_DRV_DISK_INTEGRITY,
     /** Floppy media driver group. */
     LOG_GROUP_DRV_FLOPPY,
     /** Host Base block driver group. */
@@ -269,6 +269,8 @@ typedef enum LOGGROUP
     LOG_GROUP_PGM_PHYS_ACCESS,
     /** PGM shadow page pool group. */
     LOG_GROUP_PGM_POOL,
+    /** PGM shared paging group. */
+    LOG_GROUP_PGM_SHARED,
     /** REM group. */
     LOG_GROUP_REM,
     /** REM disassembly handler group. */
@@ -311,6 +313,10 @@ typedef enum LOGGROUP
     LOG_GROUP_USB_DRV,
     /** USBFilter group. */
     LOG_GROUP_USB_FILTER,
+    /** USB keyboard device group. */
+    LOG_GROUP_USB_KBD,
+    /** MSD USB device group. */
+    LOG_GROUP_USB_MSD,
     /** Generic virtual disk layer. */
     LOG_GROUP_VD,
     /** iSCSI virtual disk backend. */
@@ -329,6 +335,8 @@ typedef enum LOGGROUP
     LOG_GROUP_VMM,
     /** VRDP group */
     LOG_GROUP_VRDP,
+    /** VSCSI group */
+    LOG_GROUP_VSCSI,
     /** Webservice group. */
     LOG_GROUP_WEBSERVICE
     /* !!!ALPHABETICALLY!!! */
@@ -353,6 +361,7 @@ typedef enum LOGGROUP
     RT_LOGGROUP_NAMES, \
     "DEFAULT",      \
     "CFGM",         \
+    "COREDUMPER",   \
     "CPUM",         \
     "CSAM",         \
     "DBGC",         \
@@ -399,6 +408,7 @@ typedef enum LOGGROUP
     "DRV_ACPI",     \
     "DRV_BLOCK",    \
     "DRV_CHAR",     \
+    "DRV_DISK_INTEGRITY", \
     "DRV_FLOPPY",   \
     "DRV_HOST_BASE", \
     "DRV_HOST_DVD", \
@@ -455,6 +465,7 @@ typedef enum LOGGROUP
     "PGM_PHYS",     \
     "PGM_PHYS_ACCESS",\
     "PGM_POOL",     \
+    "PGM_SHARED",   \
     "REM",          \
     "REM_DISAS",    \
     "REM_HANDLER",  \
@@ -476,6 +487,8 @@ typedef enum LOGGROUP
     "TRPM",         \
     "USB_DRV",      \
     "USB_FILTER",   \
+    "USB_KBD",      \
+    "USB_MSD",      \
     "VD",           \
     "VD_ISCSI",     \
     "VD_RAW",       \
@@ -485,6 +498,7 @@ typedef enum LOGGROUP
     "VM",           \
     "VMM",          \
     "VRDP",         \
+    "VSCSI",        \
     "WEBSERVICE",   \
 }
 
