@@ -3,7 +3,7 @@
  */
 
 /*
- * Copyright (C) 2006-2007 Sun Microsystems, Inc.
+ * Copyright (C) 2006-2007 Oracle Corporation
  *
  * This file is part of VirtualBox Open Source Edition (OSE), as
  * available from http://www.virtualbox.org. This file is free software;
@@ -21,10 +21,6 @@
  *
  * You may elect to license modified versions of this file under the
  * terms and conditions of either the GPL or the CDDL or both.
- *
- * Please contact Sun Microsystems, Inc., 4150 Network Circle, Santa
- * Clara, CA 95054 USA or visit http://www.sun.com if you need
- * additional information or have any questions.
  */
 
 #ifndef ___VBox_ssm_h
@@ -509,6 +505,141 @@ typedef FNSSMDEVLOADDONE *PFNSSMDEVLOADDONE;
 /** @} */
 
 
+/** The PDM USB device callback variants.
+ * @{
+ */
+
+/**
+ * Prepare state live save operation.
+ *
+ * @returns VBox status code.
+ * @param   pUsbIns         The USB device instance of the USB device which
+ *                          registered the data unit.
+ * @param   pSSM            SSM operation handle.
+ * @thread  Any.
+ */
+typedef DECLCALLBACK(int) FNSSMUSBLIVEPREP(PPDMUSBINS pUsbIns, PSSMHANDLE pSSM);
+/** Pointer to a FNSSMUSBLIVEPREP() function. */
+typedef FNSSMUSBLIVEPREP *PFNSSMUSBLIVEPREP;
+
+/**
+ * Execute state live save operation.
+ *
+ * This will be called repeatedly until all units vote that the live phase has
+ * been concluded.
+ *
+ * @returns VBox status code.
+ * @param   pUsbIns         The USB device instance of the USB device which
+ *                          registered the data unit.
+ * @param   pSSM            SSM operation handle.
+ * @param   uPass           The pass.
+ * @thread  Any.
+ */
+typedef DECLCALLBACK(int) FNSSMUSBLIVEEXEC(PPDMUSBINS pUsbIns, PSSMHANDLE pSSM, uint32_t uPass);
+/** Pointer to a FNSSMUSBLIVEEXEC() function. */
+typedef FNSSMUSBLIVEEXEC *PFNSSMUSBLIVEEXEC;
+
+/**
+ * Vote on whether the live part of the saving has been concluded.
+ *
+ * The vote stops once a unit has vetoed the decision, so don't rely upon this
+ * being called every time.
+ *
+ * @returns VBox status code.
+ * @retval  VINF_SUCCESS if done.
+ * @retval  VINF_SSM_VOTE_FOR_ANOTHER_PASS if another pass is needed.
+ * @retval  VINF_SSM_VOTE_DONE_DONT_CALL_AGAIN if the live saving of the unit is
+ *          done and there is not need calling it again before the final pass.
+ * @retval  VERR_SSM_VOTE_FOR_GIVING_UP if its time to give up.
+ *
+ * @param   pUsbIns         The USB device instance of the USB device which
+ *                          registered the data unit.
+ * @param   pSSM            SSM operation handle.
+ * @param   uPass           The data pass.
+ * @thread  Any.
+ */
+typedef DECLCALLBACK(int) FNSSMUSBLIVEVOTE(PPDMUSBINS pUsbIns, PSSMHANDLE pSSM, uint32_t uPass);
+/** Pointer to a FNSSMUSBLIVEVOTE() function. */
+typedef FNSSMUSBLIVEVOTE *PFNSSMUSBLIVEVOTE;
+
+/**
+ * Prepare state save operation.
+ *
+ * @returns VBox status code.
+ * @param   pUsbIns         The USB device instance of the USB device which
+ *                          registered the data unit.
+ * @param   pSSM            SSM operation handle.
+ */
+typedef DECLCALLBACK(int) FNSSMUSBSAVEPREP(PPDMUSBINS pUsbIns, PSSMHANDLE pSSM);
+/** Pointer to a FNSSMUSBSAVEPREP() function. */
+typedef FNSSMUSBSAVEPREP *PFNSSMUSBSAVEPREP;
+
+/**
+ * Execute state save operation.
+ *
+ * @returns VBox status code.
+ * @param   pUsbIns         The USB device instance of the USB device which
+ *                          registered the data unit.
+ * @param   pSSM            SSM operation handle.
+ */
+typedef DECLCALLBACK(int) FNSSMUSBSAVEEXEC(PPDMUSBINS pUsbIns, PSSMHANDLE pSSM);
+/** Pointer to a FNSSMUSBSAVEEXEC() function. */
+typedef FNSSMUSBSAVEEXEC *PFNSSMUSBSAVEEXEC;
+
+/**
+ * Done state save operation.
+ *
+ * @returns VBox status code.
+ * @param   pUsbIns         The USB device instance of the USB device which
+ *                          registered the data unit.
+ * @param   pSSM            SSM operation handle.
+ */
+typedef DECLCALLBACK(int) FNSSMUSBSAVEDONE(PPDMUSBINS pUsbIns, PSSMHANDLE pSSM);
+/** Pointer to a FNSSMUSBSAVEDONE() function. */
+typedef FNSSMUSBSAVEDONE *PFNSSMUSBSAVEDONE;
+
+/**
+ * Prepare state load operation.
+ *
+ * @returns VBox status code.
+ * @param   pUsbIns         The USB device instance of the USB device which
+ *                          registered the data unit.
+ * @param   pSSM            SSM operation handle.
+ */
+typedef DECLCALLBACK(int) FNSSMUSBLOADPREP(PPDMUSBINS pUsbIns, PSSMHANDLE pSSM);
+/** Pointer to a FNSSMUSBLOADPREP() function. */
+typedef FNSSMUSBLOADPREP *PFNSSMUSBLOADPREP;
+
+/**
+ * Execute state load operation.
+ *
+ * @returns VBox status code.
+ * @param   pUsbIns         The USB device instance of the USB device which
+ *                          registered the data unit.
+ * @param   pSSM            SSM operation handle.
+ * @param   uVersion        Data layout version.
+ * @param   uPass           The pass. This is always SSM_PASS_FINAL for units
+ *                          that doesn't specify a pfnSaveLive callback.
+ */
+typedef DECLCALLBACK(int) FNSSMUSBLOADEXEC(PPDMUSBINS pUsbIns, PSSMHANDLE pSSM, uint32_t uVersion, uint32_t uPass);
+/** Pointer to a FNSSMUSBLOADEXEC() function. */
+typedef FNSSMUSBLOADEXEC *PFNSSMUSBLOADEXEC;
+
+/**
+ * Done state load operation.
+ *
+ * @returns VBox load code.
+ * @param   pUsbIns         The USB device instance of the USB device which
+ *                          registered the data unit.
+ * @param   pSSM            SSM operation handle.
+ */
+typedef DECLCALLBACK(int) FNSSMUSBLOADDONE(PPDMUSBINS pUsbIns, PSSMHANDLE pSSM);
+/** Pointer to a FNSSMUSBLOADDONE() function. */
+typedef FNSSMUSBLOADDONE *PFNSSMUSBLOADDONE;
+
+/** @} */
+
+
 /** The PDM Driver callback variants.
  * @{
  */
@@ -517,7 +648,7 @@ typedef FNSSMDEVLOADDONE *PFNSSMDEVLOADDONE;
  * Prepare state live save operation.
  *
  * @returns VBox status code.
- * @param   pDrvIns         Driver instance of the device which registered the
+ * @param   pDrvIns         Driver instance of the driver which registered the
  *                          data unit.
  * @param   pSSM            SSM operation handle.
  * @thread  Any.
@@ -533,7 +664,7 @@ typedef FNSSMDRVLIVEPREP *PFNSSMDRVLIVEPREP;
  * been concluded.
  *
  * @returns VBox status code.
- * @param   pDrvIns         Driver instance of the device which registered the
+ * @param   pDrvIns         Driver instance of the driver which registered the
  *                          data unit.
  * @param   pSSM            SSM operation handle.
  * @param   uPass           The data pass.
@@ -556,7 +687,7 @@ typedef FNSSMDRVLIVEEXEC *PFNSSMDRVLIVEEXEC;
  *          done and there is not need calling it again before the final pass.
  * @retval  VERR_SSM_VOTE_FOR_GIVING_UP if its time to give up.
  *
- * @param   pDrvIns         Driver instance of the device which registered the
+ * @param   pDrvIns         Driver instance of the driver which registered the
  *                          data unit.
  * @param   pSSM            SSM operation handle.
  * @param   uPass           The data pass.
