@@ -3,7 +3,7 @@
  */
 
 /*
- * Copyright (C) 2006-2007 Sun Microsystems, Inc.
+ * Copyright (C) 2006-2007 Oracle Corporation
  *
  * This file is part of VirtualBox Open Source Edition (OSE), as
  * available from http://www.virtualbox.org. This file is free software;
@@ -21,10 +21,6 @@
  *
  * You may elect to license modified versions of this file under the
  * terms and conditions of either the GPL or the CDDL or both.
- *
- * Please contact Sun Microsystems, Inc., 4150 Network Circle, Santa
- * Clara, CA 95054 USA or visit http://www.sun.com if you need
- * additional information or have any questions.
  */
 
 #ifndef ___iprt_nocrt_amd64_math_h
@@ -33,7 +29,7 @@
 #include <iprt/asm.h>
 
 
-#if RT_INLINE_ASM_GNU_STYLE
+#if RT_INLINE_ASM_GNU_STYLE && defined(__SSE__)
 
 DECLINLINE(long double) inline_atan2l(long double lrd1, long double lrd2)
 {
