@@ -378,6 +378,14 @@ RTDECL(int) RTStrValidateEncodingEx(const char *psz, size_t cch, uint32_t fFlags
  */
 RTDECL(bool) RTStrIsValidEncoding(const char *psz);
 
+/**
+ * Purge all bad UTF-8 encoding in the string, replacing it with '?'.
+ *
+ * @returns The number of bad characters (0 if nothing was done).
+ * @param   psz         The string to purge.
+ */
+RTDECL(size_t) RTStrPurgeEncoding(char *psz);
+
 /**
  * Gets the number of code points the string is made up of, excluding
  * the terminator.
@@ -516,7 +524,8 @@ RTDECL(RTUNICP) RTStrGetCpInternal(const char *psz);
  *
  * @returns iprt status code
  * @returns VERR_INVALID_UTF8_ENCODING if the encoding is invalid.
- * @param   ppsz        The string.
+ * @param   ppsz        The string cursor.
+ *                      This is advanced one character forward on failure.
  * @param   pCp         Where to store the unicode code point.
  *                      Stores RTUNICP_INVALID if the encoding is invalid.
  */
@@ -582,6 +591,7 @@ DECLINLINE(RTUNICP) RTStrGetCp(const char *psz)
  * @returns iprt status code.
  * @param   ppsz        Pointer to the string pointer. This will be updated to
  *                      point to the char following the current code point.
+ *                      This is advanced one character forward on failure.
  * @param   pCp         Where to store the code point.
  *                      RTUNICP_INVALID is stored here on failure.
  *
@@ -1194,6 +1204,54 @@ RTDECL(size_t) RTStrNLen(const char *pszString, size_t cchMax);
  */
 RTDECL(int) RTStrNLenEx(const char *pszString, size_t cchMax, size_t *pcch);
 
+RT_C_DECLS_END
+
+/** The maximum size argument of a memchr call. */
+#define RTSTR_MEMCHR_MAX            (~(size_t)0x10000)
+
+/**
+ * Find the zero terminator in a string with a limited length.
+ *
+ * @returns Pointer to the zero terminator.
+ * @returns NULL if the zero terminator was not found.
+ *
+ * @param   pszString   The string.
+ * @param   cchMax      The max string length.  RTSTR_MAX is fine.
+ */
+#if defined(__cplusplus) && !defined(DOXYGEN_RUNNING)
+DECLINLINE(char const *) RTStrEnd(char const *pszString, size_t cchMax)
+{
+    /* Avoid potential issues with memchr seen in glibc. */
+    if (cchMax > RTSTR_MEMCHR_MAX)
+    {
+        char const *pszRet = (char const *)memchr(pszString, '\0', RTSTR_MEMCHR_MAX);
+        if (RT_LIKELY(pszRet))
+            return pszRet;
+        pszString += RTSTR_MEMCHR_MAX;
+        cchMax    -= RTSTR_MEMCHR_MAX;
+    }
+    return (char const *)memchr(pszString, '\0', cchMax);
+}
+
+DECLINLINE(char *) RTStrEnd(char *pszString, size_t cchMax)
+#else
+DECLINLINE(char *) RTStrEnd(const char *pszString, size_t cchMax)
+#endif
+{
+    /* Avoid potential issues with memchr seen in glibc. */
+    if (cchMax > RTSTR_MEMCHR_MAX)
+    {
+        char *pszRet = (char *)memchr(pszString, '\0', RTSTR_MEMCHR_MAX);
+        if (RT_LIKELY(pszRet))
+            return pszRet;
+        pszString += RTSTR_MEMCHR_MAX;
+        cchMax    -= RTSTR_MEMCHR_MAX;
+    }
+    return (char *)memchr(pszString, '\0', cchMax);
+}
+
+RT_C_DECLS_BEGIN
+
 /**
  * Matches a simple string pattern.
  *
