@@ -1,10 +1,10 @@
-/* $Revision: 57546 $ */
+/* $Revision: 60692 $ */
 /** @file
  * VBoxGuestLibR0 - Physical memory heap.
  */
 
 /*
- * Copyright (C) 2006-2007 Sun Microsystems, Inc.
+ * Copyright (C) 2006-2007 Oracle Corporation
  *
  * This file is part of VirtualBox Open Source Edition (OSE), as
  * available from http://www.virtualbox.org. This file is free software;
@@ -22,10 +22,6 @@
  *
  * You may elect to license modified versions of this file under the
  * terms and conditions of either the GPL or the CDDL or both.
- *
- * Please contact Sun Microsystems, Inc., 4150 Network Circle, Santa
- * Clara, CA 95054 USA or visit http://www.sun.com if you need
- * additional information or have any questions.
  */
 
 #include "VBGLInternal.h"
@@ -64,9 +60,9 @@
 // #define DUMPHEAP
 
 #ifdef DUMPHEAP
-#define VBGL_PH_dprintf(a) AssertMsg2 a
+# define VBGL_PH_dprintf(a) RTAssertMsg2Weak a
 #else
-#define VBGL_PH_dprintf(a)
+# define VBGL_PH_dprintf(a)
 #endif
 
 /* Heap block signature */
@@ -318,6 +314,7 @@ static VBGLPHYSHEAPBLOCK *vbglPhysHeapChunkAlloc (uint32_t cbSize)
 
     if (!pChunk)
     {
+        LogRel(("vbglPhysHeapChunkAlloc: failed to alloc %u contiguous bytes.\n", cbSize));
         return NULL;
     }
 
