@@ -1,10 +1,10 @@
-/* $Revision: 57546 $ */
+/* $Revision: 60692 $ */
 /** @file
  * VBoxGuestLibR0 - Central calls.
  */
 
 /*
- * Copyright (C) 2006-2007 Sun Microsystems, Inc.
+ * Copyright (C) 2006-2007 Oracle Corporation
  *
  * This file is part of VirtualBox Open Source Edition (OSE), as
  * available from http://www.virtualbox.org. This file is free software;
@@ -22,10 +22,6 @@
  *
  * You may elect to license modified versions of this file under the
  * terms and conditions of either the GPL or the CDDL or both.
- *
- * Please contact Sun Microsystems, Inc., 4150 Network Circle, Santa
- * Clara, CA 95054 USA or visit http://www.sun.com if you need
- * additional information or have any questions.
  */
 
 /* Entire file is ifdef'ed with !VBGL_VBOXGUEST */
@@ -33,7 +29,7 @@
 
 #ifdef RT_OS_LINUX
 # include "VBoxCalls.h"
-# define DbgPrint AssertMsg2
+# define DbgPrint RTAssertMsg2Weak
 #else
 # include "VBoxCalls.h"
 #endif
