@@ -125,6 +125,8 @@ static uint32_t g_TimeSyncSetThreshold = 20*60*1000;
 /** Whether the next adjustment should just set the time instead of trying to
  * adjust it. This is used to implement --timesync-set-start.  */
 static bool volatile g_fTimeSyncSetNext = false;
+/** Whether to set the time when the VM was restored. */
+static bool g_fTimeSyncSetOnRestore = true;
 
 /** Current error count. Used to knowing when to bitch and when not to. */
 static uint32_t g_cTimeSyncErrors = 0;
@@ -132,6 +134,9 @@ static uint32_t g_cTimeSyncErrors = 0;
 /** The semaphore we're blocking on. */
 static RTSEMEVENTMULTI g_TimeSyncEvent = NIL_RTSEMEVENTMULTI;
 
+/** The VM session ID. Changes whenever the VM is restored or reset. */
+static uint64_t g_idTimeSyncSession;
+
 #ifdef RT_OS_WINDOWS
 /** Process token. */
 static HANDLE g_hTokenProcess = NULL;
@@ -159,7 +164,7 @@ static DECLCALLBACK(int) VBoxServiceTimeSyncPreInit(void)
     int rc = VbglR3GuestPropConnect(&uGuestPropSvcClientID);
     if (RT_FAILURE(rc))
     {
-        VBoxServiceError("Failed to connect to the guest property service! Error: %Rrc\n", rc);
+        VBoxServiceError("VBoxServiceTimeSyncPreInit: Failed to connect to the guest property service! Error: %Rrc\n", rc);
     }
     else
     {
@@ -201,6 +206,15 @@ static DECLCALLBACK(int) VBoxServiceTimeSyncPreInit(void)
                 RTStrFree(pszValue);
             }
         }
+        if (   RT_SUCCESS(rc)
+            || rc == VERR_NOT_FOUND)
+        {
+            uint32_t value;
+            rc = VBoxServiceReadPropUInt32(uGuestPropSvcClientID, "/VirtualBox/GuestAdd/VBoxService/--timesync-set-on-restore",
+                                           &value, 1, 1);
+            if (RT_SUCCESS(rc))
+                g_fTimeSyncSetOnRestore = !!value;
+        }
 
         VbglR3GuestPropDisconnect(uGuestPropSvcClientID);
     }
@@ -219,6 +233,7 @@ static DECLCALLBACK(int) VBoxServiceTimeSyncPreInit(void)
 static DECLCALLBACK(int) VBoxServiceTimeSyncOption(const char **ppszShort, int argc, char **argv, int *pi)
 {
     int rc = -1;
+    uint32_t value;
     if (ppszShort)
         /* no short options */;
     else if (!strcmp(argv[*pi], "--timesync-interval"))
@@ -241,6 +256,12 @@ static DECLCALLBACK(int) VBoxServiceTimeSyncOption(const char **ppszShort, int a
         g_fTimeSyncSetNext = true;
         rc = VINF_SUCCESS;
     }
+    else if (!strcmp(argv[*pi], "--timesync-set-on-restore"))
+    {
+        rc = VBoxServiceArgUInt32(argc, argv, "", pi, &value, 1, 1);
+        if (RT_SUCCESS(rc))
+            g_fTimeSyncSetOnRestore = !!value;
+    }
 
     return rc;
 }
@@ -258,6 +279,9 @@ static DECLCALLBACK(int) VBoxServiceTimeSyncInit(void)
     if (!g_TimeSyncInterval)
         g_TimeSyncInterval = 10 * 1000;
 
+    VbglR3GetSessionId(&g_idTimeSyncSession);
+    /* The status code is ignored as this information is not available with VBox < 3.2.10. */
+
     int rc = RTSemEventMultiCreate(&g_TimeSyncEvent);
     AssertRC(rc);
 #ifdef RT_OS_WINDOWS
@@ -281,14 +305,14 @@ static DECLCALLBACK(int) VBoxServiceTimeSyncInit(void)
                 {
                     DWORD dwErr = GetLastError();
                     rc = RTErrConvertFromWin32(dwErr);
-                    VBoxServiceError("Adjusting token privileges (SE_SYSTEMTIME_NAME) failed with status code %u/%Rrc!\n", dwErr, rc);
+                    VBoxServiceError("VBoxServiceTimeSyncInit: Adjusting token privileges (SE_SYSTEMTIME_NAME) failed with status code %u/%Rrc!\n", dwErr, rc);
                 }
             }
             else
             {
                 DWORD dwErr = GetLastError();
                 rc = RTErrConvertFromWin32(dwErr);
-                VBoxServiceError("Looking up token privileges (SE_SYSTEMTIME_NAME) failed with status code %u/%Rrc!\n", dwErr, rc);
+                VBoxServiceError("VBoxServiceTimeSyncInit: Looking up token privileges (SE_SYSTEMTIME_NAME) failed with status code %u/%Rrc!\n", dwErr, rc);
             }
             if (RT_FAILURE(rc))
             {
@@ -300,19 +324,19 @@ static DECLCALLBACK(int) VBoxServiceTimeSyncInit(void)
         {
             DWORD dwErr = GetLastError();
             rc = RTErrConvertFromWin32(dwErr);
-            VBoxServiceError("Opening process token (SE_SYSTEMTIME_NAME) failed with status code %u/%Rrc!\n", dwErr, rc);
+            VBoxServiceError("VBoxServiceTimeSyncInit: Opening process token (SE_SYSTEMTIME_NAME) failed with status code %u/%Rrc!\n", dwErr, rc);
             g_hTokenProcess = NULL;
         }
     }
 
     if (GetSystemTimeAdjustment(&g_dwWinTimeAdjustment, &g_dwWinTimeIncrement, &g_bWinTimeAdjustmentDisabled))
-        VBoxServiceVerbose(3, "Windows time adjustment: Initially %ld (100ns) units per %ld (100 ns) units interval, disabled=%d\n",
+        VBoxServiceVerbose(3, "VBoxServiceTimeSyncInit: Initially %ld (100ns) units per %ld (100 ns) units interval, disabled=%d\n",
                            g_dwWinTimeAdjustment, g_dwWinTimeIncrement, g_bWinTimeAdjustmentDisabled ? 1 : 0);
     else
     {
         DWORD dwErr = GetLastError();
         rc = RTErrConvertFromWin32(dwErr);
-        VBoxServiceError("Could not get time adjustment values! Last error: %ld!\n", dwErr);
+        VBoxServiceError("VBoxServiceTimeSyncInit: Could not get time adjustment values! Last error: %ld!\n", dwErr);
     }
 #endif /* RT_OS_WINDOWS */
 
@@ -365,8 +389,8 @@ static bool VBoxServiceTimeSyncAdjust(PCRTTIMESPEC pDrift)
             }
         }
 
-        VBoxServiceVerbose(3, "Windows time adjustment: Drift=%lldms\n", RTTimeSpecGetMilli(pDrift));
-        VBoxServiceVerbose(3, "Windows time adjustment: OrgTA=%ld, CurTA=%ld, NewTA=%ld, DiffNew=%ld, DiffMax=%ld\n",
+        VBoxServiceVerbose(3, "VBoxServiceTimeSyncAdjust: Drift=%lldms\n", RTTimeSpecGetMilli(pDrift));
+        VBoxServiceVerbose(3, "VBoxServiceTimeSyncAdjust: OrgTA=%ld, CurTA=%ld, NewTA=%ld, DiffNew=%ld, DiffMax=%ld\n",
                            g_dwWinTimeAdjustment, dwWinTimeAdjustment, dwWinNewTimeAdjustment, dwDiffNew, dwDiffMax);
         if (SetSystemTimeAdjustment(dwWinNewTimeAdjustment, FALSE /* Periodic adjustments enabled. */))
         {
@@ -375,10 +399,10 @@ static bool VBoxServiceTimeSyncAdjust(PCRTTIMESPEC pDrift)
         }
 
         if (g_cTimeSyncErrors++ < 10)
-             VBoxServiceError("SetSystemTimeAdjustment failed, error=%u\n", GetLastError());
+             VBoxServiceError("VBoxServiceTimeSyncAdjust: SetSystemTimeAdjustment failed, error=%u\n", GetLastError());
     }
     else if (g_cTimeSyncErrors++ < 10)
-        VBoxServiceError("GetSystemTimeAdjustment failed, error=%ld\n", GetLastError());
+        VBoxServiceError("VBoxServiceTimeSyncAdjust: GetSystemTimeAdjustment failed, error=%ld\n", GetLastError());
 
 #elif defined(RT_OS_OS2)
     /* No API for doing gradual time adjustments. */
@@ -392,7 +416,7 @@ static bool VBoxServiceTimeSyncAdjust(PCRTTIMESPEC pDrift)
     if (adjtime(&tv, NULL) == 0)
     {
         if (g_cVerbosity >= 1)
-            VBoxServiceVerbose(1, "adjtime by %RDtimespec\n", pDrift);
+            VBoxServiceVerbose(1, "VBoxServiceTimeSyncAdjust: adjtime by %RDtimespec\n", pDrift);
         g_cTimeSyncErrors = 0;
         return true;
     }
@@ -416,9 +440,9 @@ static void VBoxServiceTimeSyncCancelAdjust(void)
     if (g_hTokenProcess == NULL) /* No process token (anymore)? */
         return;
     if (SetSystemTimeAdjustment(0, TRUE /* Periodic adjustments disabled. */))
-        VBoxServiceVerbose(3, "Windows Time Adjustment is now disabled.\n");
+        VBoxServiceVerbose(3, "VBoxServiceTimeSyncCancelAdjust: Windows Time Adjustment is now disabled.\n");
     else if (g_cTimeSyncErrors++ < 10)
-        VBoxServiceError("SetSystemTimeAdjustment(,disable) failed, error=%u\n", GetLastError());
+        VBoxServiceError("VBoxServiceTimeSyncCancelAdjust: SetSystemTimeAdjustment(,disable) failed, error=%u\n", GetLastError());
 #endif /* !RT_OS_WINDOWS */
 }
 
@@ -457,7 +481,7 @@ static void VBoxServiceTimeSyncSet(PCRTTIMESPEC pDrift)
         }
     }
     else if (g_cTimeSyncErrors++ < 10)
-        VBoxServiceError("RTTimeSet(%RDtimespec) failed: %Rrc\n", &NewGuestTime, rc);
+        VBoxServiceError("VBoxServiceTimeSyncSet: RTTimeSet(%RDtimespec) failed: %Rrc\n", &NewGuestTime, rc);
 }
 
 
@@ -491,7 +515,7 @@ DECLCALLBACK(int) VBoxServiceTimeSyncWorker(bool volatile *pfShutdown)
             if (RT_FAILURE(rc2))
             {
                 if (g_cTimeSyncErrors++ < 10)
-                    VBoxServiceError("VbglR3GetHostTime failed; rc2=%Rrc\n", rc2);
+                    VBoxServiceError("VBoxServiceTimeSyncWorker: VbglR3GetHostTime failed; rc2=%Rrc\n", rc2);
                 break;
             }
             RTTimeNow(&GuestNow);
@@ -501,6 +525,23 @@ DECLCALLBACK(int) VBoxServiceTimeSyncWorker(bool volatile *pfShutdown)
             RTTimeSpecSub(&GuestElapsed, &GuestNow0);
             if ((uint32_t)RTTimeSpecGetMilli(&GuestElapsed) < g_TimeSyncMaxLatency)
             {
+                /*
+                 * Set the time once after we were restored.
+                 * (Of course only if the drift is bigger than MinAdjust)
+                 */
+                uint32_t TimeSyncSetThreshold = g_TimeSyncSetThreshold;
+                if (g_fTimeSyncSetOnRestore)
+                {
+                    uint64_t idNewSession = g_idTimeSyncSession;
+                    VbglR3GetSessionId(&idNewSession);
+                    if (idNewSession != g_idTimeSyncSession)
+                    {
+                        VBoxServiceVerbose(3, "VBoxServiceTimeSyncWorker: The VM session ID changed, forcing resync.\n");
+                        TimeSyncSetThreshold = 0;
+                        g_idTimeSyncSession  = idNewSession;
+                    }
+                }
+
                 /*
                  * Calculate the adjustment threshold and the current drift.
                  */
@@ -517,9 +558,9 @@ DECLCALLBACK(int) VBoxServiceTimeSyncWorker(bool volatile *pfShutdown)
                 RTTimeSpecAbsolute(&AbsDrift);
                 if (g_cVerbosity >= 3)
                 {
-                    VBoxServiceVerbose(3, "Host:    %s    (MinAdjust: %RU32 ms)\n",
+                    VBoxServiceVerbose(3, "VBoxServiceTimeSyncWorker: Host:    %s    (MinAdjust: %RU32 ms)\n",
                                        RTTimeToString(RTTimeExplode(&Time, &HostNow), sz, sizeof(sz)), MinAdjust);
-                    VBoxServiceVerbose(3, "Guest: - %s => %RDtimespec drift\n",
+                    VBoxServiceVerbose(3, "VBoxServiceTimeSyncWorker: Guest: - %s => %RDtimespec drift\n",
                                        RTTimeToString(RTTimeExplode(&Time, &GuestNow), sz, sizeof(sz)),
                                        &Drift);
                 }
@@ -534,7 +575,7 @@ DECLCALLBACK(int) VBoxServiceTimeSyncWorker(bool volatile *pfShutdown)
                      * too big, fall back on just setting the time.
                      */
 
-                    if (    AbsDriftMilli > g_TimeSyncSetThreshold
+                    if (    AbsDriftMilli > TimeSyncSetThreshold
                         ||  g_fTimeSyncSetNext
                         ||  !VBoxServiceTimeSyncAdjust(&Drift))
                     {
@@ -546,7 +587,7 @@ DECLCALLBACK(int) VBoxServiceTimeSyncWorker(bool volatile *pfShutdown)
                     VBoxServiceTimeSyncCancelAdjust();
                 break;
             }
-            VBoxServiceVerbose(3, "%RDtimespec: latency too high (%RDtimespec) sleeping 1s\n", GuestElapsed);
+            VBoxServiceVerbose(3, "VBoxServiceTimeSyncWorker: %RDtimespec: latency too high (%RDtimespec) sleeping 1s\n", GuestElapsed);
             RTThreadSleep(1000);
         } while (--cTries > 0);
 
@@ -566,7 +607,7 @@ DECLCALLBACK(int) VBoxServiceTimeSyncWorker(bool volatile *pfShutdown)
             break;
         if (rc2 != VERR_TIMEOUT && RT_FAILURE(rc2))
         {
-            VBoxServiceError("RTSemEventMultiWait failed; rc2=%Rrc\n", rc2);
+            VBoxServiceError("VBoxServiceTimeSyncWorker: RTSemEventMultiWait failed; rc2=%Rrc\n", rc2);
             rc = rc2;
             break;
         }
@@ -598,7 +639,7 @@ static DECLCALLBACK(void) VBoxServiceTimeSyncTerm(void)
         if (!AdjustTokenPrivileges(g_hTokenProcess, FALSE, &g_TkOldPrivileges, sizeof(TOKEN_PRIVILEGES), NULL, NULL))
         {
             DWORD dwErr = GetLastError();
-            VBoxServiceError("Restoring token privileges (SE_SYSTEMTIME_NAME) failed with code %u!\n", dwErr);
+            VBoxServiceError("VBoxServiceTimeSyncTerm: Restoring token privileges (SE_SYSTEMTIME_NAME) failed with code %u!\n", dwErr);
         }
         CloseHandle(g_hTokenProcess);
         g_hTokenProcess = NULL;
@@ -626,6 +667,7 @@ VBOXSERVICE g_TimeSync =
     "              [--timesync-interval <ms>] [--timesync-min-adjust <ms>]\n"
     "              [--timesync-latency-factor <x>] [--timesync-max-latency <ms>]\n"
     "              [--timesync-set-threshold <ms>] [--timesync-set-start]"
+    "              [--timesync-set-restore 0|1]\n"
     ,
     /* pszOptions. */
     "    --timesync-interval     Specifies the interval at which to synchronize the\n"
@@ -644,6 +686,9 @@ VBOXSERVICE g_TimeSync =
     "                            where to start setting the time instead of trying to\n"
     "                            adjust it. The default is 20 min.\n"
     "    --timesync-set-start    Set the time when starting the time sync service.\n"
+    "    --timesync-set-on-restore 0|1\n"
+    "                            Immediately set the time when the VM was restored.\n"
+    "                            1 = set (default), 0 = don't set.\n"
     ,
     /* methods */
     VBoxServiceTimeSyncPreInit,
