@@ -1,10 +1,10 @@
 #! /bin/sh
-# Sun VirtualBox
-# Linux Additions kernel module init script ($Revision: 62854 $)
+#
+# Linux Additions kernel module init script ($Revision: 65760 $)
 #
 
 #
-# Copyright (C) 2006-2009 Oracle Corporation
+# Copyright (C) 2006-2010 Oracle Corporation
 #
 # This file is part of VirtualBox Open Source Edition (OSE), as
 # available from http://www.virtualbox.org. This file is free software;
@@ -30,9 +30,10 @@
 
 PATH=$PATH:/bin:/sbin:/usr/sbin
 PACKAGE=VBoxGuestAdditions
-BUILDVBOXGUEST=`/bin/ls /usr/src/vboxguest*/build_in_tmp 2>/dev/null|cut -d' ' -f1`
-BUILDVBOXSF=`/bin/ls /usr/src/vboxsf*/build_in_tmp 2>/dev/null|cut -d' ' -f1`
-BUILDVBOXVIDEO=`/bin/ls /usr/src/vboxvideo*/build_in_tmp 2>/dev/null|cut -d' ' -f1`
+BUILDVBOXGUEST=`/bin/ls /usr/src/vboxguest*/vboxguest/build_in_tmp 2>/dev/null|cut -d' ' -f1`
+BUILDVBOXSF=`/bin/ls /usr/src/vboxguest*/vboxsf/build_in_tmp 2>/dev/null|cut -d' ' -f1`
+BUILDVBOXVIDEO=`/bin/ls /usr/src/vboxvideo*/vboxvideo/build_in_tmp 2>/dev/null|cut -d' ' -f1`
+DODKMS=`/bin/ls /usr/src/vboxguest*/do_dkms 2>/dev/null|cut -d' ' -f1`
 LOG="/var/log/vboxadd-install.log"
 MODPROBE=/sbin/modprobe
 
@@ -308,6 +309,9 @@ restart()
 setup()
 {
     # don't stop the old modules here -- they might be in use
+    begin "Uninstalling old VirtualBox DKMS kernel modules"
+    $DODKMS uninstall > $LOG
+    succ_msg
     if find /lib/modules/`uname -r` -name "vboxvideo\.*" 2>/dev/null|grep -q vboxvideo; then
         begin "Removing old VirtualBox vboxvideo kernel module"
         find /lib/modules/`uname -r` -name "vboxvideo\.*" 2>/dev/null|xargs rm -f 2>/dev/null
@@ -324,8 +328,9 @@ setup()
         succ_msg
     fi
     begin "Building the VirtualBox Guest Additions kernel modules"
+
     if ! sh /usr/share/$PACKAGE/test/build_in_tmp \
-        --no-dkms --no-print-directory > $LOG 2>&1; then
+        --no-print-directory >> $LOG 2>&1; then
         fail_msg
         printf "Your system does not seem to be set up to build kernel modules.\nLook at $LOG to find out what went wrong.  Once you have corrected it, you can\nrun\n\n  /etc/init.d/vboxadd setup\n\nto build them."
         BUILDVBOXGUEST=""
@@ -333,40 +338,42 @@ setup()
         BUILDVBOXVIDEO=""
     else
         if ! sh /usr/share/$PACKAGE/test_drm/build_in_tmp \
-            --no-dkms --no-print-directory >> $LOG 2>&1; then
+            --no-print-directory >> $LOG 2>&1; then
             printf "\nYour guest system does not seem to have sufficient OpenGL support to enable\naccelerated 3D effects (this requires Linux 2.6.27 or later in the guest\nsystem).  This Guest Additions feature will be disabled.\n\n"
             BUILDVBOXVIDEO=""
         fi
     fi
     echo
-    if [ -n "$BUILDVBOXGUEST" ]; then
-        begin "Building the main Guest Additions module"
-        if ! $BUILDVBOXGUEST \
-            --save-module-symvers /tmp/vboxguest-Module.symvers \
-            --no-print-directory install >> $LOG 2>&1; then
-            fail "Look at $LOG to find out what went wrong"
+    if ! $DODKMS install >> $LOG 2>&1; then
+        if [ -n "$BUILDVBOXGUEST" ]; then
+            begin "Building the main Guest Additions module"
+            if ! $BUILDVBOXGUEST \
+                --save-module-symvers /tmp/vboxguest-Module.symvers \
+                --no-print-directory install >> $LOG 2>&1; then
+                fail "Look at $LOG to find out what went wrong"
+            fi
+            succ_msg
         fi
-        succ_msg
-    fi
-    if [ -n "$BUILDVBOXSF" ]; then
-        begin "Building the shared folder support module"
-        if ! $BUILDVBOXSF \
-            --use-module-symvers /tmp/vboxguest-Module.symvers \
-            --no-print-directory install >> $LOG 2>&1; then
-            fail "Look at $LOG to find out what went wrong"
+        if [ -n "$BUILDVBOXSF" ]; then
+            begin "Building the shared folder support module"
+            if ! $BUILDVBOXSF \
+                --use-module-symvers /tmp/vboxguest-Module.symvers \
+                --no-print-directory install >> $LOG 2>&1; then
+                fail "Look at $LOG to find out what went wrong"
+            fi
+            succ_msg
         fi
-        succ_msg
-    fi
-    if [ -n "$BUILDVBOXVIDEO" ]; then
-        begin "Building the OpenGL support module"
-        if ! $BUILDVBOXVIDEO \
-            --use-module-symvers /tmp/vboxguest-Module.symvers \
-            --no-print-directory install >> $LOG 2>&1; then
-            fail "Look at $LOG to find out what went wrong"
+        if [ -n "$BUILDVBOXVIDEO" ]; then
+            begin "Building the OpenGL support module"
+            if ! $BUILDVBOXVIDEO \
+                --use-module-symvers /tmp/vboxguest-Module.symvers \
+                --no-print-directory install >> $LOG 2>&1; then
+                fail "Look at $LOG to find out what went wrong"
+            fi
+            succ_msg
         fi
-        succ_msg
+        depmod
     fi
-    depmod
 
     begin "Doing non-kernel setup of the Guest Additions"
     echo "Creating user for the Guest Additions." >> $LOG
