@@ -1198,6 +1198,10 @@ void slirp_select_poll(PNATState pData, struct pollfd *polls, int ndfs)
             }
             /* mark the socket for termination _after_ it was drained */
             so->so_close = 1;
+            /* No idea about Windows but on Posix, POLLHUP means that we can't send more */
+#ifndef RT_OS_WINDOWS
+            sofcantsendmore(so);
+#endif
             CONTINUE(tcp);
         }
 
