@@ -358,11 +358,7 @@ tcp_input(PNATState pData, register struct mbuf *m, int iphlen, struct socket *i
      * (vvl) ip_input substracts IP header length from ip->ip_len value.
      * here we do the test the same as input method of UDP protocol.
      */
-#ifdef VBOX_WITH_SLIRP_BSD_MBUF
     Assert((ip->ip_len + iphlen == m_length(m, NULL)));
-#else
-    Assert((ip->ip_len  + iphlen == m->m_len));
-#endif
     save_ip = *ip;
     save_ip.ip_len+= iphlen;
 
@@ -788,7 +784,6 @@ findso:
                     tp->t_socket->so_m = NULL;
                 }
                 tp = tcp_close(pData, tp);
-                m_freem(pData, m);
             }
             else
             {
