@@ -4,7 +4,7 @@
  */
 
 /*
- * Copyright (C) 2004-2009 Sun Microsystems, Inc.
+ * Copyright (C) 2004-2009 Oracle Corporation
  *
  * This file is part of VirtualBox Open Source Edition (OSE), as
  * available from http://www.virtualbox.org. This file is free software;
@@ -13,21 +13,20 @@
  * Foundation, in version 2 as it comes in the "COPYING" file of the
  * VirtualBox OSE distribution. VirtualBox OSE is distributed in the
  * hope that it will be useful, but WITHOUT ANY WARRANTY of any kind.
- *
- * Please contact Sun Microsystems, Inc., 4150 Network Circle, Santa
- * Clara, CA 95054 USA or visit http://www.sun.com if you need
- * additional information or have any questions.
  */
 
 /* The I/O port of the BusLogic SCSI adapter. */
 #define BUSLOGIC_ISA_IO_PORT 0x330
 /* The I/O port of the LsiLogic SCSI adapter. */
 #define LSILOGIC_ISA_IO_PORT 0x340
+/* The I/O port of the LsiLogic SAS adapter. */
+#define LSILOGIC_SAS_ISA_IO_PORT 0x350
 
 #define VBOXSCSI_REGISTER_STATUS   0
 #define VBOXSCSI_REGISTER_COMMAND  0
 #define VBOXSCSI_REGISTER_DATA_IN  1
 #define VBOXSCSI_REGISTER_IDENTIFY 2
+#define VBOXSCSI_REGISTER_RESET    3
 
 #define VBOXSCSI_MAX_DEVICES 16 /* Maximum number of devices a SCSI device can have. */
 
@@ -187,6 +186,7 @@ int scsi_read_sectors(device_id, count, lba, segment, offset)
     if (device_id > BX_MAX_SCSI_DEVICES)
         BX_PANIC("scsi_read_sectors: device_id out of range %d\n", device_id);
 
+    ebda_seg = read_word(0x0040, 0x000E);
     // Reset count of transferred data
     write_word(ebda_seg, &EbdaData->ata.trsfsectors,0);
     write_dword(ebda_seg, &EbdaData->ata.trsfbytes,0L);
@@ -203,7 +203,6 @@ int scsi_read_sectors(device_id, count, lba, segment, offset)
     write_byte(get_SS(), aCDB + 8, (uint8_t)(count));
     write_byte(get_SS(), aCDB + 9, 0);
 
-    ebda_seg = read_word(0x0040, 0x000E);
     io_base = read_word(ebda_seg, &EbdaData->scsi.devices[device_id].io_base);
     target_id = read_byte(ebda_seg, &EbdaData->scsi.devices[device_id].target_id);
 
@@ -241,6 +240,7 @@ int scsi_write_sectors(device_id, count, lba, segment, offset)
     if (device_id > BX_MAX_SCSI_DEVICES)
         BX_PANIC("scsi_write_sectors: device_id out of range %d\n", device_id);
 
+    ebda_seg = read_word(0x0040, 0x000E);
     // Reset count of transferred data
     write_word(ebda_seg, &EbdaData->ata.trsfsectors,0);
     write_dword(ebda_seg, &EbdaData->ata.trsfbytes,0L);
@@ -257,7 +257,6 @@ int scsi_write_sectors(device_id, count, lba, segment, offset)
     write_byte(get_SS(), aCDB + 8, (uint8_t)(count));
     write_byte(get_SS(), aCDB + 9, 0);
 
-    ebda_seg = read_word(0x0040, 0x000E);
     io_base = read_word(ebda_seg, &EbdaData->scsi.devices[device_id].io_base);
     target_id = read_byte(ebda_seg, &EbdaData->scsi.devices[device_id].target_id);
 
@@ -287,8 +286,6 @@ void scsi_enumerate_attached_devices(io_base)
 
     ebda_seg = read_word(0x0040, 0x000E);
 
-    write_byte(ebda_seg, &EbdaData->scsi.hdcount, 0);
-
     /* Go through target devices. */
     for (i = 0; i < VBOXSCSI_MAX_DEVICES; i++)
     {
@@ -368,7 +365,7 @@ void scsi_enumerate_attached_devices(io_base)
                     }
                     cylinders = (uint32_t)(sectors / (heads * sectors_per_track));
                 }
-                else if (io_base == LSILOGIC_ISA_IO_PORT)
+                else if (io_base == LSILOGIC_ISA_IO_PORT || io_base == LSILOGIC_SAS_ISA_IO_PORT)
                 {
                     /* This is from the BusLogic driver in the Linux kernel. */
                     if (sectors >= 4 * 1024 * 1024)
@@ -444,6 +441,11 @@ void scsi_enumerate_attached_devices(io_base)
 void scsi_init( )
 {
     Bit8u identifier;
+    Bit16u ebda_seg;
+
+
+    ebda_seg = read_word(0x0040, 0x000E);
+    write_byte(ebda_seg, &EbdaData->scsi.hdcount, 0);
 
     identifier = 0;
 
@@ -455,6 +457,7 @@ void scsi_init( )
     {
         /* Detected - Enumerate attached devices. */
         VBOXSCSI_DEBUG("scsi_init: BusLogic SCSI adapter detected\n");
+        outb(BUSLOGIC_ISA_IO_PORT+VBOXSCSI_REGISTER_RESET, 0);
         scsi_enumerate_attached_devices(BUSLOGIC_ISA_IO_PORT);
     }
     else
@@ -470,11 +473,28 @@ void scsi_init( )
     {
         /* Detected - Enumerate attached devices. */
         VBOXSCSI_DEBUG("scsi_init: LsiLogic SCSI adapter detected\n");
+        outb(LSILOGIC_ISA_IO_PORT+VBOXSCSI_REGISTER_RESET, 0);
         scsi_enumerate_attached_devices(LSILOGIC_ISA_IO_PORT);
     }
     else
     {
         VBOXSCSI_DEBUG("scsi_init: LsiLogic SCSI adapter not detected\n");
     }
+
+    /* Detect LsiLogic SAS adapter. */
+    outb(LSILOGIC_SAS_ISA_IO_PORT+VBOXSCSI_REGISTER_IDENTIFY, 0x55);
+    identifier = inb(LSILOGIC_SAS_ISA_IO_PORT+VBOXSCSI_REGISTER_IDENTIFY);
+
+    if (identifier == 0x55)
+    {
+        /* Detected - Enumerate attached devices. */
+        VBOXSCSI_DEBUG("scsi_init: LsiLogic SAS adapter detected\n");
+        outb(LSILOGIC_SAS_ISA_IO_PORT+VBOXSCSI_REGISTER_RESET, 0);
+        scsi_enumerate_attached_devices(LSILOGIC_SAS_ISA_IO_PORT);
+    }
+    else
+    {
+        VBOXSCSI_DEBUG("scsi_init: LsiLogic SAS adapter not detected\n");
+    }
 }
 
