@@ -4,7 +4,7 @@
  */
 
 /*
- * Copyright (C) 2006-2007 Sun Microsystems, Inc.
+ * Copyright (C) 2006-2007 Oracle Corporation
  *
  * This file is part of VirtualBox Open Source Edition (OSE), as
  * available from http://www.virtualbox.org. This file is free software;
@@ -13,10 +13,6 @@
  * Foundation, in version 2 as it comes in the "COPYING" file of the
  * VirtualBox OSE distribution. VirtualBox OSE is distributed in the
  * hope that it will be useful, but WITHOUT ANY WARRANTY of any kind.
- *
- * Please contact Sun Microsystems, Inc., 4150 Network Circle, Santa
- * Clara, CA 95054 USA or visit http://www.sun.com if you need
- * additional information or have any questions.
  */
 
 /*******************************************************************************
@@ -63,7 +59,7 @@ static DECLCALLBACK(int) pcarchIOPortFPURead(PPDMDEVINS pDevIns, void *pvUser, R
 {
     int rc;
     NOREF(pvUser); NOREF(pDevIns); NOREF(pu32);
-    rc = PDMDeviceDBGFStop(pDevIns, RT_SRC_POS, "Port=%#x cb=%d\n", Port, cb);
+    rc = PDMDevHlpDBGFStop(pDevIns, RT_SRC_POS, "Port=%#x cb=%d\n", Port, cb);
     if (rc == VINF_SUCCESS)
         rc = VERR_IOM_IOPORT_UNUSED;
     return rc;
@@ -99,14 +95,14 @@ static DECLCALLBACK(int) pcarchIOPortFPUWrite(PPDMDEVINS pDevIns, void *pvUser,
                 if (!u32)
                     rc = PDMDeviceDBGFStop(pDevIns, RT_SRC_POS, "Port=%#x cb=%d u32=%#x\n", Port, cb, u32);
 #endif
-                /* pDevIns->pDevHlp->pfnPICSetIrq(pDevIns, 13, 0); */
+                /* pDevIns->pHlp->pfnPICSetIrq(pDevIns, 13, 0); */
                 break;
 
             /* Reset. */
             case 0xf1:
                 Log2(("PCARCH: FPU Reset cb=%d u32=%#x\n", Port, cb, u32));
                 /** @todo figure out what the difference between FPU ports 0xf0 and 0xf1 are... */
-                /* pDevIns->pDevHlp->pfnPICSetIrq(pDevIns, 13, 0); */
+                /* pDevIns->pHlp->pfnPICSetIrq(pDevIns, 13, 0); */
                 break;
 
             /* opcode transfers */
@@ -114,14 +110,14 @@ static DECLCALLBACK(int) pcarchIOPortFPUWrite(PPDMDEVINS pDevIns, void *pvUser,
             case 0xfa:
             case 0xfc:
             default:
-                rc = PDMDeviceDBGFStop(pDevIns, RT_SRC_POS, "Port=%#x cb=%d u32=%#x\n", Port, cb, u32);
+                rc = PDMDevHlpDBGFStop(pDevIns, RT_SRC_POS, "Port=%#x cb=%d u32=%#x\n", Port, cb, u32);
                 break;
         }
         /* this works better, but probably not entirely correct. */
         PDMDevHlpISASetIrq(pDevIns, 13, 0);
     }
     else
-        rc = PDMDeviceDBGFStop(pDevIns, RT_SRC_POS, "Port=%#x cb=%d u32=%#x\n", Port, cb, u32);
+        rc = PDMDevHlpDBGFStop(pDevIns, RT_SRC_POS, "Port=%#x cb=%d u32=%#x\n", Port, cb, u32);
     return rc;
 }
 
@@ -144,28 +140,30 @@ static DECLCALLBACK(int) pcarchIOPortFPUWrite(PPDMDEVINS pDevIns, void *pvUser,
  *
  * @remark  Ralph Brown and friends have this to say about this port:
  *
- *  0092  RW  PS/2 system control port A  (port B is at PORT 0061h) (see #P0415)
- *
- *  Bitfields for PS/2 system control port A:
- *  Bit(s)	Description	(Table P0415)
- *   7-6	any bit set to 1 turns activity light on
- *   5	unused
- *   4	watchdog timout occurred
- *   3	=0 RTC/CMOS security lock (on password area) unlocked
- *  	=1 CMOS locked (done by POST)
- *   2	unused
- *   1	A20 is active
- *   0	=0 system reset or write
- *  	=1 pulse alternate reset pin (high-speed alternate CPU reset)
- *  Notes:	once set, bit 3 may only be cleared by a power-on reset
- *  	on at least the C&T 82C235, bit 0 remains set through a CPU reset to
- *  	  allow the BIOS to determine the reset method
- *  	on the PS/2 30-286 & "Tortuga" the INT 15h/87h memory copy does
- *  	  not use this port for A20 control, but instead uses the keyboard
- *  	  controller (8042). Reportedly this may cause the system to crash
- *  	  when access to the 8042 is disabled in password server mode
- *  	  (see #P0398).
- *  SeeAlso: #P0416,#P0417,MSR 00001000h
+ * @verbatim
+0092  RW  PS/2 system control port A  (port B is at PORT 0061h) (see #P0415)
+
+Bitfields for PS/2 system control port A:
+Bit(s)  Description     (Table P0415)
+ 7-6    any bit set to 1 turns activity light on
+ 5      unused
+ 4      watchdog timout occurred
+ 3      =0 RTC/CMOS security lock (on password area) unlocked
+        =1 CMOS locked (done by POST)
+ 2      unused
+ 1      A20 is active
+ 0      =0 system reset or write
+        =1 pulse alternate reset pin (high-speed alternate CPU reset)
+Notes:  once set, bit 3 may only be cleared by a power-on reset
+        on at least the C&T 82C235, bit 0 remains set through a CPU reset to
+          allow the BIOS to determine the reset method
+        on the PS/2 30-286 & "Tortuga" the INT 15h/87h memory copy does
+          not use this port for A20 control, but instead uses the keyboard
+          controller (8042). Reportedly this may cause the system to crash
+          when access to the 8042 is disabled in password server mode
+          (see #P0398).
+SeeAlso: #P0416,#P0417,MSR 00001000h
+ * @endverbatim
  */
 static DECLCALLBACK(int) pcarchIOPortPS2SysControlPortARead(PPDMDEVINS pDevIns, void *pvUser, RTIOPORT Port, uint32_t *pu32, unsigned cb)
 {
@@ -174,7 +172,7 @@ static DECLCALLBACK(int) pcarchIOPortPS2SysControlPortARead(PPDMDEVINS pDevIns,
         *pu32 = PDMDevHlpA20IsEnabled(pDevIns) << 1;
         return VINF_SUCCESS;
     }
-    return PDMDeviceDBGFStop(pDevIns, RT_SRC_POS, "Port=%#x cb=%d\n", Port, cb);
+    return PDMDevHlpDBGFStop(pDevIns, RT_SRC_POS, "Port=%#x cb=%d\n", Port, cb);
 }
 
 
@@ -207,24 +205,14 @@ static DECLCALLBACK(int) pcarchIOPortPS2SysControlPortAWrite(PPDMDEVINS pDevIns,
         PDMDevHlpA20Set(pDevIns, !!(u32 & 2));
         return VINF_SUCCESS;
     }
-    return PDMDeviceDBGFStop(pDevIns, RT_SRC_POS, "Port=%#x cb=%d u32=%#x\n", Port, cb, u32);
+    return PDMDevHlpDBGFStop(pDevIns, RT_SRC_POS, "Port=%#x cb=%d u32=%#x\n", Port, cb, u32);
 }
 
 
 /**
- * Construct a device instance for a VM.
- *
- * @returns VBox status.
- * @param   pDevIns     The device instance data.
- *                      If the registration structure is needed, pDevIns->pDevReg points to it.
- * @param   iInstance   Instance number. Use this to figure out which registers and such to use.
- *                      The device number is also found in pDevIns->iInstance, but since it's
- *                      likely to be freqently used PDM passes it as parameter.
- * @param   pCfgHandle  Configuration node handle for the device. Use this to obtain the configuration
- *                      of the device instance. It's also found in pDevIns->pCfgHandle, but like
- *                      iInstance it's expected to be used a bit in this function.
+ * @interface_method_impl{PDMDEVREG,pfnConstruct}
  */
-static DECLCALLBACK(int)  pcarchConstruct(PPDMDEVINS pDevIns, int iInstance, PCFGMNODE pCfgHandle)
+static DECLCALLBACK(int)  pcarchConstruct(PPDMDEVINS pDevIns, int iInstance, PCFGMNODE pCfg)
 {
     PDEVPCARCH  pThis = PDMINS_2_DATA(pDevIns, PDEVPCARCH);
     int         rc;
@@ -233,7 +221,7 @@ static DECLCALLBACK(int)  pcarchConstruct(PPDMDEVINS pDevIns, int iInstance, PCF
     /*
      * Validate configuration.
      */
-    if (!CFGMR3AreValuesValid(pCfgHandle, "\0"))
+    if (!CFGMR3AreValuesValid(pCfg, "\0"))
         return VERR_PDM_DEVINS_UNKNOWN_CFG_VALUES;
 
     /*
@@ -262,7 +250,7 @@ const PDMDEVREG g_DevicePcArch =
 {
     /* u32Version */
     PDM_DEVREG_VERSION,
-    /* szDeviceName */
+    /* szName */
     "pcarch",
     /* szRCMod */
     "",
@@ -271,7 +259,7 @@ const PDMDEVREG g_DevicePcArch =
     /* pszDescription */
     "PC Architecture Device",
     /* fFlags */
-    PDM_DEVREG_FLAGS_HOST_BITS_DEFAULT | PDM_DEVREG_FLAGS_GUEST_BITS_32,
+    PDM_DEVREG_FLAGS_HOST_BITS_DEFAULT | PDM_DEVREG_FLAGS_GUEST_BITS_DEFAULT,
     /* fClass */
     PDM_DEVREG_CLASS_ARCH,
     /* cMaxInstances */
