@@ -6,7 +6,7 @@ This file Copyright (C) 2003 Michael Brown <mbrown@fensystems.co.uk>
 of Fen Systems Ltd. (http://www.fensystems.co.uk/).  All rights
 reserved.
 
-$Id: undi.h 16040 2006-12-01 13:25:05Z klaus $
+$Id: undi.h $
 ***************************************************************************/
 
 /*
@@ -16,6 +16,15 @@ $Id: undi.h 16040 2006-12-01 13:25:05Z klaus $
  * your option) any later version.
  */
 
+/*
+ * Sun GPL Disclaimer: For the avoidance of doubt, except that if any license choice
+ * other than GPL or LGPL is available it will apply instead, Sun elects to use only
+ * the General Public License version 2 (GPLv2) at this time for any software where
+ * a choice of GPL license versions is made available with the language indicating
+ * that GPLv2 or any later version may be used, or where a choice of which version
+ * of the GPL is applied is otherwise unspecified.
+ */
+
 #include "pxe.h"
 #include "pic8259.h"
 
