@@ -4,7 +4,7 @@
  */
 
 /*
- * Copyright (C) 2006-2007 Sun Microsystems, Inc.
+ * Copyright (C) 2006-2007 Oracle Corporation
  *
  * This file is part of VirtualBox Open Source Edition (OSE), as
  * available from http://www.virtualbox.org. This file is free software;
@@ -13,10 +13,6 @@
  * Foundation, in version 2 as it comes in the "COPYING" file of the
  * VirtualBox OSE distribution. VirtualBox OSE is distributed in the
  * hope that it will be useful, but WITHOUT ANY WARRANTY of any kind.
- *
- * Please contact Sun Microsystems, Inc., 4150 Network Circle, Santa
- * Clara, CA 95054 USA or visit http://www.sun.com if you need
- * additional information or have any questions.
  */
 
 #ifndef __HostDrvBase_h__
@@ -24,13 +20,17 @@
 
 #include <VBox/cdefs.h>
 
-__BEGIN_DECLS
+RT_C_DECLS_BEGIN
 
 
 /** Pointer to host base drive access driver instance data. */
 typedef struct DRVHOSTBASE *PDRVHOSTBASE;
 /**
  * Host base drive access driver instance data.
+ *
+ * @implements PDMIMOUNT
+ * @implements PDMIBLOCKBIOS
+ * @implements PDMIBLOCK
  */
 typedef struct DRVHOSTBASE
 {
@@ -80,7 +80,7 @@ typedef struct DRVHOSTBASE
     /** The size of the media currently in the drive.
      * This is invalid if no drive is in the drive. */
     uint64_t volatile       cbSize;
-#ifndef RT_OS_DARWIN
+#if !defined(RT_OS_DARWIN)
     /** The filehandle of the device. */
     RTFILE                  FileDevice;
 #endif
@@ -96,7 +96,7 @@ typedef struct DRVHOSTBASE
     RTSEMEVENT              EventPoller;
 #endif
     /** The poller interval. */
-    unsigned                cMilliesPoller;
+    RTMSINTERVAL            cMilliesPoller;
     /** The shutdown indicator. */
     bool volatile           fShutdownPoller;
 
@@ -136,6 +136,16 @@ typedef struct DRVHOSTBASE
     uint8_t                *pbDoubleBuffer;
 #endif
 
+#ifdef RT_OS_FREEBSD
+    /** The block size. Set when querying the media size. */
+    uint32_t                cbBlock;
+    /** SCSI bus number. */
+    path_id_t               ScsiBus;
+    /** target ID of the passthrough device. */
+    target_id_t             ScsiTargetID;
+    /** LUN of the passthrough device. */
+    lun_id_t                ScsiLunID;
+#endif
 
     /**
      * Performs the locking / unlocking of the device.
@@ -170,14 +180,14 @@ typedef struct DRVHOSTBASE
 } DRVHOSTBASE;
 
 
-int DRVHostBaseInitData(PPDMDRVINS pDrvIns, PCFGMNODE pCfgHandle, PDMBLOCKTYPE enmType);
+int DRVHostBaseInitData(PPDMDRVINS pDrvIns, PCFGMNODE pCfg, PDMBLOCKTYPE enmType);
 int DRVHostBaseInitFinish(PDRVHOSTBASE pThis);
 int DRVHostBaseMediaPresent(PDRVHOSTBASE pThis);
 void DRVHostBaseMediaNotPresent(PDRVHOSTBASE pThis);
 DECLCALLBACK(void) DRVHostBaseDestruct(PPDMDRVINS pDrvIns);
-#ifdef RT_OS_DARWIN
+#if defined(RT_OS_DARWIN) || defined(RT_OS_FREEBSD)
 DECLCALLBACK(int) DRVHostBaseScsiCmd(PDRVHOSTBASE pThis, const uint8_t *pbCmd, size_t cbCmd, PDMBLOCKTXDIR enmTxDir,
-                                     void *pvBuf, size_t *pcbBuf, uint8_t *pbSense, size_t cbSense, uint32_t cTimeoutMillies);
+                                     void *pvBuf, uint32_t *pcbBuf, uint8_t *pbSense, size_t cbSense, uint32_t cTimeoutMillies);
 #endif
 
 
@@ -187,6 +197,6 @@ DECLCALLBACK(int) DRVHostBaseScsiCmd(PDRVHOSTBASE pThis, const uint8_t *pbCmd, s
 /** Makes a PDRVHOSTBASE out of a PPDMIBLOCK. */
 #define PDMIBLOCK_2_DRVHOSTBASE(pInterface)        ( (PDRVHOSTBASE)((uintptr_t)pInterface - RT_OFFSETOF(DRVHOSTBASE, IBlock)) )
 
-__END_DECLS
+RT_C_DECLS_END
 
 #endif
