@@ -78,15 +78,17 @@ DECLINLINE(int) vdiError(PVDIIMAGEDESC pImage, int rc, RT_SRC_POS_DECL,
 }
 
 
-static int vdiFileOpen(PVDIIMAGEDESC pImage, bool fReadonly, bool fCreate)
+static int vdiFileOpen(PVDIIMAGEDESC pImage, bool fReadonly, bool fShareable,
+                       bool fCreate)
 {
     int rc = VINF_SUCCESS;
 
-    AssertMsg(!(fReadonly && fCreate), ("Image can't be opened readonly whilebeing created\n"));
+    AssertMsg(!(fReadonly && fCreate), ("Image can't be opened readonly while being created\n"));
 
 #ifndef VBOX_WITH_NEW_IO_CODE
-    uint32_t fOpen = fReadonly ? RTFILE_O_READ      | RTFILE_O_DENY_NONE
-                               : RTFILE_O_READWRITE | RTFILE_O_DENY_WRITE;
+    uint32_t fDeny = fReadonly | fShareable ? RTFILE_O_DENY_NONE : RTFILE_O_DENY_WRITE;
+    uint32_t fOpen = fReadonly ? RTFILE_O_READ : RTFILE_O_READWRITE;
+    fOpen |= fDeny;
 
     if (fCreate)
         fOpen |= RTFILE_O_CREATE;
@@ -100,6 +102,9 @@ static int vdiFileOpen(PVDIIMAGEDESC pImage, bool fReadonly, bool fCreate)
     if (fCreate)
         uOpenFlags |= VD_INTERFACEASYNCIO_OPEN_FLAGS_CREATE;
 
+    if (fShareable)
+        uOpenFlags |= VD_INTERFACEASYNCIO_OPEN_FLAGS_DONT_LOCK;
+
     rc = pImage->pInterfaceIOCallbacks->pfnOpen(pImage->pInterfaceIO->pvUser,
                                                 pImage->pszFilename,
                                                 uOpenFlags,
@@ -512,8 +517,9 @@ static int vdiCreateImage(PVDIIMAGEDESC pImage, uint64_t cbSize,
                           unsigned uImageFlags, const char *pszComment,
                           PCPDMMEDIAGEOMETRY pPCHSGeometry,
                           PCPDMMEDIAGEOMETRY pLCHSGeometry, PCRTUUID pUuid,
-                          PFNVDPROGRESS pfnProgress, void *pvUser,
-                          unsigned uPercentStart, unsigned uPercentSpan)
+                          unsigned uOpenFlags, PFNVDPROGRESS pfnProgress,
+                          void *pvUser, unsigned uPercentStart,
+                          unsigned uPercentSpan)
 {
     int rc;
     uint64_t cbTotal;
@@ -578,7 +584,9 @@ static int vdiCreateImage(PVDIIMAGEDESC pImage, uint64_t cbSize,
     vdiSetupImageDesc(pImage);
 
     /* Create image file. */
-    rc = vdiFileOpen(pImage, false /* fReadonly */, true /* fCreate */);
+    rc = vdiFileOpen(pImage, false /* fReadonly */,
+                     !!(uOpenFlags & VD_OPEN_FLAGS_SHAREABLE),
+                     true /* fCreate */);
     if (RT_FAILURE(rc))
     {
         rc = vdiError(pImage, rc, RT_SRC_POS, N_("VDI: cannot create image '%s'"), pImage->pszFilename);
@@ -731,7 +739,10 @@ static int vdiOpenImage(PVDIIMAGEDESC pImage, unsigned uOpenFlags)
     /*
      * Open the image.
      */
-    rc = vdiFileOpen(pImage, !!(uOpenFlags & VD_OPEN_FLAGS_READONLY), false /* fCreate */);
+    rc = vdiFileOpen(pImage,
+                     !!(uOpenFlags & VD_OPEN_FLAGS_READONLY),
+                     !!(uOpenFlags & VD_OPEN_FLAGS_SHAREABLE),
+                     false /* fCreate */);
     if (RT_FAILURE(rc))
     {
         /* Do NOT signal an appropriate error here, as the VD layer has the
@@ -873,6 +884,7 @@ static int vdiUpdateHeaderAsync(PVDIIMAGEDESC pImage, PVDIOCTX pIoCtx)
             rc = vdiFileWriteMetaAsync(pImage, sizeof(VDIPREHEADER),
                                        &pImage->Header.u.v0, sizeof(pImage->Header.u.v0),
                                        pIoCtx, NULL, NULL);
+
             break;
         case 1:
             if (pImage->Header.u.v1plus.cbHeader < sizeof(pImage->Header.u.v1plus))
@@ -1150,7 +1162,7 @@ static int vdiCreate(const char *pszFilename, uint64_t cbSize,
     pImage->pVDIfsImage = pVDIfsImage;
 
     rc = vdiCreateImage(pImage, cbSize, uImageFlags, pszComment,
-                        pPCHSGeometry, pLCHSGeometry, pUuid,
+                        pPCHSGeometry, pLCHSGeometry, pUuid, uOpenFlags,
                         pfnProgress, pvUser, uPercentStart, uPercentSpan);
     if (RT_SUCCESS(rc))
     {
@@ -1650,7 +1662,7 @@ static int vdiSetOpenFlags(void *pBackendData, unsigned uOpenFlags)
 
     /* Image must be opened and the new flags must be valid. Just readonly and
      * info flags are supported. */
-    if (!pImage || (uOpenFlags & ~(VD_OPEN_FLAGS_READONLY | VD_OPEN_FLAGS_INFO | VD_OPEN_FLAGS_ASYNC_IO)))
+    if (!pImage || (uOpenFlags & ~(VD_OPEN_FLAGS_READONLY | VD_OPEN_FLAGS_INFO | VD_OPEN_FLAGS_ASYNC_IO | VD_OPEN_FLAGS_SHAREABLE)))
     {
         rc = VERR_INVALID_PARAMETER;
         goto out;
