@@ -5,7 +5,7 @@
  */
 
 /*
- * Copyright (C) 2006-2007 Sun Microsystems, Inc.
+ * Copyright (C) 2006-2007 Oracle Corporation
  *
  * This file is part of VirtualBox Open Source Edition (OSE), as
  * available from http://www.virtualbox.org. This file is free software;
@@ -14,10 +14,6 @@
  * Foundation, in version 2 as it comes in the "COPYING" file of the
  * VirtualBox OSE distribution. VirtualBox OSE is distributed in the
  * hope that it will be useful, but WITHOUT ANY WARRANTY of any kind.
- *
- * Please contact Sun Microsystems, Inc., 4150 Network Circle, Santa
- * Clara, CA 95054 USA or visit http://www.sun.com if you need
- * additional information or have any questions.
  */
 
 
@@ -52,16 +48,17 @@
 static int disCoreOne(PDISCPUSTATE pCpu, RTUINTPTR InstructionAddr, unsigned *pcbInstruction);
 #if !defined(DIS_CORE_ONLY) && defined(LOG_ENABLED)
 static void disasmAddString(char *psz, const char *pszString);
-static void disasmAddStringF(char *psz, uint32_t cbString, const char *pszFormat, ...);
+static void disasmAddStringF(char *psz, const char *pszFormat, ...);
 static void disasmAddChar(char *psz, char ch);
+# define disasmAddStringF1(psz, pszFmt, a1)         disasmAddStringF(psz, pszFmt, a1)
+# define disasmAddStringF2(psz, pszFmt, a1, a2)     disasmAddStringF(psz, pszFmt, a1, a2)
+# define disasmAddStringF3(psz, pszFmt, a1, a2, a3) disasmAddStringF(psz, pszFmt, a1, a2, a3)
 #else
-# define disasmAddString(psz, pszString)        do {} while (0)
-# ifdef _MSC_VER
-#  define disasmAddStringF __noop
-# else
-#  define disasmAddStringF(psz, cbString, pszFormat...)   do {} while (0)  /* Arg wanna get rid of that warning */
-# endif
-# define disasmAddChar(psz, ch)                 do {} while (0)
+# define disasmAddString(psz, pszString)            do {} while (0)
+# define disasmAddStringF1(psz, pszFmt, a1)         do {} while (0)
+# define disasmAddStringF2(psz, pszFmt, a1, a2)     do {} while (0)
+# define disasmAddStringF3(psz, pszFmt, a1, a2, a3) do {} while (0)
+# define disasmAddChar(psz, ch)                     do {} while (0)
 #endif
 
 static unsigned QueryModRM(RTUINTPTR pu8CodeBlock, PCOPCODE pOp, POP_PARAMETER pParam, PDISCPUSTATE pCpu, unsigned *pSibInc = NULL);
@@ -569,8 +566,6 @@ void UseSIB(RTUINTPTR lpszCodeBlock, PCOPCODE pOp, POP_PARAMETER pParam, PDISCPU
     unsigned scale, base, index, regtype;
     const char **ppszSIBIndexReg;
     const char **ppszSIBBaseReg;
-    char szTemp[32];
-    szTemp[0] = '\0';
 
     scale = pCpu->SIB.Bits.Scale;
     base  = pCpu->SIB.Bits.Base;
@@ -601,20 +596,19 @@ void UseSIB(RTUINTPTR lpszCodeBlock, PCOPCODE pOp, POP_PARAMETER pParam, PDISCPU
          }
 
          if (base == 5 && pCpu->ModRM.Bits.Mod == 0)
-             disasmAddStringF(szTemp, sizeof(szTemp), "%s%s", ppszSIBIndexReg[index], szSIBScale[scale]);
+             disasmAddStringF2(pParam->szParam, "%s%s", ppszSIBIndexReg[index], szSIBScale[scale]);
          else
-             disasmAddStringF(szTemp, sizeof(szTemp), "%s+%s%s", ppszSIBBaseReg[base], ppszSIBIndexReg[index], szSIBScale[scale]);
+             disasmAddStringF3(pParam->szParam, "%s+%s%s", ppszSIBBaseReg[base], ppszSIBIndexReg[index], szSIBScale[scale]);
     }
     else
     {
          if (base != 5 || pCpu->ModRM.Bits.Mod != 0)
-             disasmAddStringF(szTemp, sizeof(szTemp), "%s", ppszSIBBaseReg[base]);
+             disasmAddStringF1(pParam->szParam, "%s", ppszSIBBaseReg[base]);
     }
 
     if (base == 5 && pCpu->ModRM.Bits.Mod == 0)
     {
         // [scaled index] + disp32
-        disasmAddString(pParam->szParam, &szTemp[0]);
         if (pCpu->addrmode == CPUMODE_32BIT)
         {
             pParam->flags |= USE_DISPLACEMENT32;
@@ -632,8 +626,6 @@ void UseSIB(RTUINTPTR lpszCodeBlock, PCOPCODE pOp, POP_PARAMETER pParam, PDISCPU
     }
     else
     {
-        disasmAddString(pParam->szParam, szTemp);
-
         pParam->flags |= USE_BASE | regtype;
         pParam->base.reg_gen = base;
     }
@@ -737,18 +729,18 @@ unsigned UseModRM(RTUINTPTR lpszCodeBlock, PCOPCODE pOp, POP_PARAMETER pParam, P
                 else
                     pParam->base.reg_ctrl = reg;
 
-                disasmAddStringF(pParam->szParam, sizeof(pParam->szParam), "CR%d", pParam->base.reg_ctrl);
+                disasmAddStringF1(pParam->szParam, "CR%d", pParam->base.reg_ctrl);
                 return 0;
 
             case OP_PARM_D: //debug register
-                disasmAddStringF(pParam->szParam, sizeof(pParam->szParam), "DR%d", reg);
+                disasmAddStringF1(pParam->szParam, "DR%d", reg);
                 pParam->flags |= USE_REG_DBG;
                 pParam->base.reg_dbg = reg;
                 return 0;
 
             case OP_PARM_P: //MMX register
                 reg &= 7;   /* REX.R has no effect here */
-                disasmAddStringF(pParam->szParam, sizeof(pParam->szParam), "MM%d", reg);
+                disasmAddStringF1(pParam->szParam, "MM%d", reg);
                 pParam->flags |= USE_REG_MMX;
                 pParam->base.reg_mmx = reg;
                 return 0;
@@ -761,7 +753,7 @@ unsigned UseModRM(RTUINTPTR lpszCodeBlock, PCOPCODE pOp, POP_PARAMETER pParam, P
 
             case OP_PARM_T: //test register
                 reg &= 7;   /* REX.R has no effect here */
-                disasmAddStringF(pParam->szParam, sizeof(pParam->szParam), "TR%d", reg);
+                disasmAddStringF1(pParam->szParam, "TR%d", reg);
                 pParam->flags |= USE_REG_TEST;
                 pParam->base.reg_test = reg;
                 return 0;
@@ -773,7 +765,7 @@ unsigned UseModRM(RTUINTPTR lpszCodeBlock, PCOPCODE pOp, POP_PARAMETER pParam, P
                 /* else no break */
 
             case OP_PARM_V: //XMM register
-                disasmAddStringF(pParam->szParam, sizeof(pParam->szParam), "XMM%d", reg);
+                disasmAddStringF1(pParam->szParam, "XMM%d", reg);
                 pParam->flags |= USE_REG_XMM;
                 pParam->base.reg_xmm = reg;
                 return 0;
@@ -813,7 +805,7 @@ unsigned UseModRM(RTUINTPTR lpszCodeBlock, PCOPCODE pOp, POP_PARAMETER pParam, P
                 {
                     pParam->flags |= USE_RIPDISPLACEMENT32;
                     pParam->disp32 = pCpu->disp;
-                    disasmAddStringF(pParam->szParam, sizeof(pParam->szParam), "RIP+");
+                    disasmAddString(pParam->szParam, "RIP+");
                     disasmPrintDisp32(pParam);
                 }
             }
@@ -1208,7 +1200,7 @@ unsigned ParseImmByte(RTUINTPTR lpszCodeBlock, PCOPCODE pOp, POP_PARAMETER pPara
     pParam->flags |= USE_IMMEDIATE8;
     pParam->size   = sizeof(uint8_t);
 
-    disasmAddStringF(pParam->szParam, sizeof(pParam->szParam), "0%02Xh", (uint32_t)pParam->parval);
+    disasmAddStringF1(pParam->szParam, "0%02Xh", (uint32_t)pParam->parval);
     return sizeof(uint8_t);
 }
 //*****************************************************************************
@@ -1226,7 +1218,7 @@ unsigned ParseImmByteSX(RTUINTPTR lpszCodeBlock, PCOPCODE pOp, POP_PARAMETER pPa
         pParam->parval = (uint32_t)(int8_t)DISReadByte(pCpu, lpszCodeBlock);
         pParam->flags |= USE_IMMEDIATE32_SX8;
         pParam->size   = sizeof(uint32_t);
-        disasmAddStringF(pParam->szParam, sizeof(pParam->szParam), "0%08Xh", (uint32_t)pParam->parval);
+        disasmAddStringF1(pParam->szParam, "0%08Xh", (uint32_t)pParam->parval);
     }
     else
     if (pCpu->opmode == CPUMODE_64BIT)
@@ -1234,14 +1226,14 @@ unsigned ParseImmByteSX(RTUINTPTR lpszCodeBlock, PCOPCODE pOp, POP_PARAMETER pPa
         pParam->parval = (uint64_t)(int8_t)DISReadByte(pCpu, lpszCodeBlock);
         pParam->flags |= USE_IMMEDIATE64_SX8;
         pParam->size   = sizeof(uint64_t);
-        disasmAddStringF(pParam->szParam, sizeof(pParam->szParam), "0%016RX64h", pParam->parval);
+        disasmAddStringF1(pParam->szParam, "0%016RX64h", pParam->parval);
     }
     else
     {
         pParam->parval = (uint16_t)(int8_t)DISReadByte(pCpu, lpszCodeBlock);
         pParam->flags |= USE_IMMEDIATE16_SX8;
         pParam->size   = sizeof(uint16_t);
-        disasmAddStringF(pParam->szParam, sizeof(pParam->szParam), "0%04Xh", (uint16_t)pParam->parval);
+        disasmAddStringF1(pParam->szParam, "0%04Xh", (uint16_t)pParam->parval);
     }
     return sizeof(uint8_t);
 }
@@ -1259,7 +1251,7 @@ unsigned ParseImmUshort(RTUINTPTR lpszCodeBlock, PCOPCODE pOp, POP_PARAMETER pPa
     pParam->flags |= USE_IMMEDIATE16;
     pParam->size   = sizeof(uint16_t);
 
-    disasmAddStringF(pParam->szParam, sizeof(pParam->szParam), "0%04Xh", (uint16_t)pParam->parval);
+    disasmAddStringF1(pParam->szParam, "0%04Xh", (uint16_t)pParam->parval);
     return sizeof(uint16_t);
 }
 //*****************************************************************************
@@ -1276,7 +1268,7 @@ unsigned ParseImmUlong(RTUINTPTR lpszCodeBlock, PCOPCODE pOp, POP_PARAMETER pPar
     pParam->flags |= USE_IMMEDIATE32;
     pParam->size   = sizeof(uint32_t);
 
-    disasmAddStringF(pParam->szParam, sizeof(pParam->szParam), "0%08Xh", (uint32_t)pParam->parval);
+    disasmAddStringF1(pParam->szParam, "0%08Xh", (uint32_t)pParam->parval);
     return sizeof(uint32_t);
 }
 //*****************************************************************************
@@ -1293,8 +1285,8 @@ unsigned ParseImmQword(RTUINTPTR lpszCodeBlock, PCOPCODE pOp, POP_PARAMETER pPar
     pParam->flags |= USE_IMMEDIATE64;
     pParam->size   = sizeof(uint64_t);
 
-    disasmAddStringF(pParam->szParam, sizeof(pParam->szParam), "0%08X", (uint32_t)pParam->parval);
-    disasmAddStringF(&pParam->szParam[9], sizeof(pParam->szParam)-9, "%08Xh", (uint32_t)(pParam->parval >> 32));
+    disasmAddStringF2(pParam->szParam, "0%08X%08Xh",
+                      (uint32_t)pParam->parval, (uint32_t)(pParam->parval >> 32));
     return sizeof(uint64_t);
 }
 //*****************************************************************************
@@ -1313,7 +1305,7 @@ unsigned ParseImmV(RTUINTPTR lpszCodeBlock, PCOPCODE pOp, POP_PARAMETER pParam,
         pParam->flags |= USE_IMMEDIATE32;
         pParam->size   = sizeof(uint32_t);
 
-        disasmAddStringF(pParam->szParam, sizeof(pParam->szParam), "0%08Xh", (uint32_t)pParam->parval);
+        disasmAddStringF1(pParam->szParam, "0%08Xh", (uint32_t)pParam->parval);
         return sizeof(uint32_t);
     }
     else
@@ -1323,7 +1315,7 @@ unsigned ParseImmV(RTUINTPTR lpszCodeBlock, PCOPCODE pOp, POP_PARAMETER pParam,
         pParam->flags |= USE_IMMEDIATE64;
         pParam->size   = sizeof(uint64_t);
 
-        disasmAddStringF(pParam->szParam, sizeof(pParam->szParam), "0%RX64h", pParam->parval);
+        disasmAddStringF1(pParam->szParam, "0%RX64h", pParam->parval);
         return sizeof(uint64_t);
     }
     else
@@ -1332,7 +1324,7 @@ unsigned ParseImmV(RTUINTPTR lpszCodeBlock, PCOPCODE pOp, POP_PARAMETER pParam,
         pParam->flags |= USE_IMMEDIATE16;
         pParam->size   = sizeof(uint16_t);
 
-        disasmAddStringF(pParam->szParam, sizeof(pParam->szParam), "0%04Xh", (uint32_t)pParam->parval);
+        disasmAddStringF1(pParam->szParam, "0%04Xh", (uint32_t)pParam->parval);
         return sizeof(uint16_t);
     }
 }
@@ -1359,7 +1351,7 @@ unsigned ParseImmZ(RTUINTPTR lpszCodeBlock, PCOPCODE pOp, POP_PARAMETER pParam,
         pParam->flags |= USE_IMMEDIATE16;
         pParam->size   = sizeof(uint16_t);
 
-        disasmAddStringF(pParam->szParam, sizeof(pParam->szParam), "0%04Xh", (uint32_t)pParam->parval);
+        disasmAddStringF1(pParam->szParam, "0%04Xh", (uint32_t)pParam->parval);
         return sizeof(uint16_t);
     }
     else
@@ -1370,14 +1362,14 @@ unsigned ParseImmZ(RTUINTPTR lpszCodeBlock, PCOPCODE pOp, POP_PARAMETER pParam,
             pParam->parval = (uint64_t)(int32_t)DISReadDWord(pCpu, lpszCodeBlock);
             pParam->flags |= USE_IMMEDIATE64;
             pParam->size   = sizeof(uint64_t);
-            disasmAddStringF(pParam->szParam, sizeof(pParam->szParam), "0%RX64h", pParam->parval);
+            disasmAddStringF1(pParam->szParam, "0%RX64h", pParam->parval);
         }
         else
         {
             pParam->parval = DISReadDWord(pCpu, lpszCodeBlock);
             pParam->flags |= USE_IMMEDIATE32;
             pParam->size   = sizeof(uint32_t);
-            disasmAddStringF(pParam->szParam, sizeof(pParam->szParam), "0%08Xh", (uint32_t)pParam->parval);
+            disasmAddStringF1(pParam->szParam, "0%08Xh", (uint32_t)pParam->parval);
         }
         return sizeof(uint32_t);
     }
@@ -1401,7 +1393,7 @@ unsigned ParseImmBRel(RTUINTPTR lpszCodeBlock, PCOPCODE pOp, POP_PARAMETER pPara
     pParam->flags |= USE_IMMEDIATE8_REL;
     pParam->size   = sizeof(uint8_t);
 
-    disasmAddStringF(pParam->szParam, sizeof(pParam->szParam), " (0%02Xh)", (uint32_t)pParam->parval);
+    disasmAddStringF1(pParam->szParam, " (0%02Xh)", (uint32_t)pParam->parval);
     return sizeof(char);
 }
 //*****************************************************************************
@@ -1422,7 +1414,7 @@ unsigned ParseImmVRel(RTUINTPTR lpszCodeBlock, PCOPCODE pOp, POP_PARAMETER pPara
         pParam->flags |= USE_IMMEDIATE32_REL;
         pParam->size   = sizeof(int32_t);
 
-        disasmAddStringF(pParam->szParam, sizeof(pParam->szParam), " (0%08Xh)", (uint32_t)pParam->parval);
+        disasmAddStringF1(pParam->szParam, " (0%08Xh)", (uint32_t)pParam->parval);
         return sizeof(int32_t);
     }
     else
@@ -1433,7 +1425,7 @@ unsigned ParseImmVRel(RTUINTPTR lpszCodeBlock, PCOPCODE pOp, POP_PARAMETER pPara
         pParam->flags |= USE_IMMEDIATE64_REL;
         pParam->size   = sizeof(int64_t);
 
-        disasmAddStringF(pParam->szParam, sizeof(pParam->szParam), " (0%RX64h)", pParam->parval);
+        disasmAddStringF1(pParam->szParam, " (0%RX64h)", pParam->parval);
         return sizeof(int32_t);
     }
     else
@@ -1442,7 +1434,7 @@ unsigned ParseImmVRel(RTUINTPTR lpszCodeBlock, PCOPCODE pOp, POP_PARAMETER pPara
         pParam->flags |= USE_IMMEDIATE16_REL;
         pParam->size   = sizeof(int16_t);
 
-        disasmAddStringF(pParam->szParam, sizeof(pParam->szParam), " (0%04Xh)", (uint32_t)pParam->parval);
+        disasmAddStringF1(pParam->szParam, " (0%04Xh)", (uint32_t)pParam->parval);
         return sizeof(int16_t);
     }
 }
@@ -1470,7 +1462,7 @@ unsigned ParseImmAddr(RTUINTPTR lpszCodeBlock, PCOPCODE pOp, POP_PARAMETER pPara
             pParam->flags  |= USE_IMMEDIATE_ADDR_16_32;
             pParam->size   = sizeof(uint16_t) + sizeof(uint32_t);
 
-            disasmAddStringF(pParam->szParam, sizeof(pParam->szParam), "0%04X:0%08Xh", (uint32_t)(pParam->parval>>32), (uint32_t)pParam->parval);
+            disasmAddStringF2(pParam->szParam, "0%04X:0%08Xh", (uint32_t)(pParam->parval>>32), (uint32_t)pParam->parval);
             return sizeof(uint32_t) + sizeof(uint16_t);
         }
         else
@@ -1483,7 +1475,7 @@ unsigned ParseImmAddr(RTUINTPTR lpszCodeBlock, PCOPCODE pOp, POP_PARAMETER pPara
             pParam->flags |= USE_DISPLACEMENT32;
             pParam->size   = sizeof(uint32_t);
 
-            disasmAddStringF(pParam->szParam, sizeof(pParam->szParam), "[0%08Xh]", pParam->disp32);
+            disasmAddStringF1(pParam->szParam, "[0%08Xh]", pParam->disp32);
             return sizeof(uint32_t);
         }
     }
@@ -1500,7 +1492,7 @@ unsigned ParseImmAddr(RTUINTPTR lpszCodeBlock, PCOPCODE pOp, POP_PARAMETER pPara
         pParam->flags |= USE_DISPLACEMENT64;
         pParam->size   = sizeof(uint64_t);
 
-        disasmAddStringF(pParam->szParam, sizeof(pParam->szParam), "[0%08X%08Xh]", (uint32_t)(pParam->disp64 >> 32), (uint32_t)pParam->disp64);
+        disasmAddStringF2(pParam->szParam, "[0%08X%08Xh]", (uint32_t)(pParam->disp64 >> 32), (uint32_t)pParam->disp64);
         return sizeof(uint64_t);
     }
     else
@@ -1511,7 +1503,7 @@ unsigned ParseImmAddr(RTUINTPTR lpszCodeBlock, PCOPCODE pOp, POP_PARAMETER pPara
             pParam->flags |= USE_IMMEDIATE_ADDR_16_16;
             pParam->size   = 2*sizeof(uint16_t);
 
-            disasmAddStringF(pParam->szParam, sizeof(pParam->szParam), "0%04X:0%04Xh", (uint32_t)(pParam->parval>>16), (uint16_t)pParam->parval );
+            disasmAddStringF2(pParam->szParam, "0%04X:0%04Xh", (uint32_t)(pParam->parval>>16), (uint16_t)pParam->parval );
             return sizeof(uint32_t);
         }
         else
@@ -1524,7 +1516,7 @@ unsigned ParseImmAddr(RTUINTPTR lpszCodeBlock, PCOPCODE pOp, POP_PARAMETER pPara
             pParam->flags |= USE_DISPLACEMENT16;
             pParam->size   = sizeof(uint16_t);
 
-            disasmAddStringF(pParam->szParam, sizeof(pParam->szParam), "[0%04Xh]", (uint32_t)pParam->disp16);
+            disasmAddStringF1(pParam->szParam, "[0%04Xh]", (uint32_t)pParam->disp16);
             return sizeof(uint16_t);
         }
     }
@@ -1665,7 +1657,7 @@ unsigned ParseFixedReg(RTUINTPTR lpszCodeBlock, PCOPCODE pOp, POP_PARAMETER pPar
 unsigned ParseXv(RTUINTPTR pu8CodeBlock, PCOPCODE pOp, POP_PARAMETER pParam, PDISCPUSTATE pCpu)
 {
     disasmGetPtrString(pCpu, pOp, pParam);
-    disasmAddStringF(pParam->szParam, sizeof(pParam->szParam), (pCpu->addrmode == CPUMODE_32BIT) ? "DS:ESI" : "DS:SI");
+    disasmAddString(pParam->szParam, (pCpu->addrmode == CPUMODE_32BIT) ? "DS:ESI" : "DS:SI");
 
     pParam->flags |= USE_POINTER_DS_BASED;
     if (pCpu->addrmode == CPUMODE_32BIT)
@@ -1690,7 +1682,7 @@ unsigned ParseXv(RTUINTPTR pu8CodeBlock, PCOPCODE pOp, POP_PARAMETER pParam, PDI
 //*****************************************************************************
 unsigned ParseXb(RTUINTPTR pu8CodeBlock, PCOPCODE pOp, POP_PARAMETER pParam, PDISCPUSTATE pCpu)
 {
-    disasmAddStringF(pParam->szParam, sizeof(pParam->szParam), (pCpu->addrmode == CPUMODE_32BIT) ? "DS:ESI" : "DS:SI");
+    disasmAddString(pParam->szParam, (pCpu->addrmode == CPUMODE_32BIT) ? "DS:ESI" : "DS:SI");
 
     pParam->flags |= USE_POINTER_DS_BASED;
     if (pCpu->addrmode == CPUMODE_32BIT)
@@ -1716,7 +1708,7 @@ unsigned ParseXb(RTUINTPTR pu8CodeBlock, PCOPCODE pOp, POP_PARAMETER pParam, PDI
 unsigned ParseYv(RTUINTPTR pu8CodeBlock, PCOPCODE pOp, POP_PARAMETER pParam, PDISCPUSTATE pCpu)
 {
     disasmGetPtrString(pCpu, pOp, pParam);
-    disasmAddStringF(pParam->szParam, sizeof(pParam->szParam), (pCpu->addrmode == CPUMODE_32BIT) ? "ES:EDI" : "ES:DI");
+    disasmAddString(pParam->szParam, (pCpu->addrmode == CPUMODE_32BIT) ? "ES:EDI" : "ES:DI");
 
     pParam->flags |= USE_POINTER_ES_BASED;
     if (pCpu->addrmode == CPUMODE_32BIT)
@@ -1741,7 +1733,7 @@ unsigned ParseYv(RTUINTPTR pu8CodeBlock, PCOPCODE pOp, POP_PARAMETER pParam, PDI
 //*****************************************************************************
 unsigned ParseYb(RTUINTPTR pu8CodeBlock, PCOPCODE pOp, POP_PARAMETER pParam, PDISCPUSTATE pCpu)
 {
-    disasmAddStringF(pParam->szParam, sizeof(pParam->szParam), (pCpu->addrmode == CPUMODE_32BIT) ? "ES:EDI" : "ES:DI");
+    disasmAddString(pParam->szParam, (pCpu->addrmode == CPUMODE_32BIT) ? "ES:EDI" : "ES:DI");
 
     pParam->flags |= USE_POINTER_ES_BASED;
     if (pCpu->addrmode == CPUMODE_32BIT)
@@ -2424,31 +2416,31 @@ void disasmModRMSReg(PDISCPUSTATE pCpu, PCOPCODE pOp, unsigned idx, POP_PARAMETE
 //*****************************************************************************
 void disasmPrintAbs32(POP_PARAMETER pParam)
 {
-    disasmAddStringF(pParam->szParam, sizeof(pParam->szParam), "%08Xh", pParam->disp32);
+    disasmAddStringF1(pParam->szParam, "%08Xh", pParam->disp32);
 }
 //*****************************************************************************
 //*****************************************************************************
 void disasmPrintDisp32(POP_PARAMETER pParam)
 {
-    disasmAddStringF(pParam->szParam, sizeof(pParam->szParam), "%08Xh", pParam->disp32);
+    disasmAddStringF1(pParam->szParam, "%08Xh", pParam->disp32);
 }
 //*****************************************************************************
 //*****************************************************************************
 void disasmPrintDisp64(POP_PARAMETER pParam)
 {
-    disasmAddStringF(pParam->szParam, sizeof(pParam->szParam), "%16RX64h", pParam->disp64);
+    disasmAddStringF1(pParam->szParam, "%16RX64h", pParam->disp64);
 }
 //*****************************************************************************
 //*****************************************************************************
 void disasmPrintDisp8(POP_PARAMETER pParam)
 {
-    disasmAddStringF(pParam->szParam, sizeof(pParam->szParam), "%d", pParam->disp8);
+    disasmAddStringF1(pParam->szParam, "%d", pParam->disp8);
 }
 //*****************************************************************************
 //*****************************************************************************
 void disasmPrintDisp16(POP_PARAMETER pParam)
 {
-    disasmAddStringF(pParam->szParam, sizeof(pParam->szParam), "%04Xh", pParam->disp16);
+    disasmAddStringF1(pParam->szParam, "%04Xh", pParam->disp16);
 }
 //*****************************************************************************
 //*****************************************************************************
@@ -2510,9 +2502,8 @@ void disasmGetPtrString(PDISCPUSTATE pCpu, PCOPCODE pOp, POP_PARAMETER pParam)
         break; //no pointer type specified/necessary
     }
     if (pCpu->prefix & PREFIX_SEG)
-        disasmAddStringF(pParam->szParam, sizeof(pParam->szParam), "%s:", szModRMSegReg[pCpu->enmPrefixSeg]);
+        disasmAddStringF1(pParam->szParam, "%s:", szModRMSegReg[pCpu->enmPrefixSeg]);
 }
-#ifndef IN_RC
 //*****************************************************************************
 /* Read functions for getting the opcode bytes */
 //*****************************************************************************
@@ -2535,7 +2526,7 @@ uint8_t DISReadByte(PDISCPUSTATE pCpu, RTUINTPTR pAddress)
     AssertMsgFailed(("DISReadByte with no read callback in ring 0!!\n"));
     return 0;
 #else
-    else return *(uint8_t *)pAddress;
+    return *(uint8_t *)(uintptr_t)pAddress;
 #endif
 }
 //*****************************************************************************
@@ -2559,7 +2550,7 @@ uint16_t DISReadWord(PDISCPUSTATE pCpu, RTUINTPTR pAddress)
     AssertMsgFailed(("DISReadWord with no read callback in ring 0!!\n"));
     return 0;
 #else
-    else return *(uint16_t *)pAddress;
+    return *(uint16_t *)(uintptr_t)pAddress;
 #endif
 }
 //*****************************************************************************
@@ -2583,7 +2574,7 @@ uint32_t DISReadDWord(PDISCPUSTATE pCpu, RTUINTPTR pAddress)
     AssertMsgFailed(("DISReadDWord with no read callback in ring 0!!\n"));
     return 0;
 #else
-    else return *(uint32_t *)pAddress;
+    return *(uint32_t *)(uintptr_t)pAddress;
 #endif
 }
 //*****************************************************************************
@@ -2608,10 +2599,9 @@ uint64_t DISReadQWord(PDISCPUSTATE pCpu, RTUINTPTR pAddress)
     AssertMsgFailed(("DISReadQWord with no read callback in ring 0!!\n"));
     return 0;
 #else
-    else return *(uint64_t *)pAddress;
+    return *(uint64_t *)(uintptr_t)pAddress;
 #endif
 }
-#endif /* IN_RC */
 
 #if !defined(DIS_CORE_ONLY) && defined(LOG_ENABLED)
 //*****************************************************************************
@@ -2622,11 +2612,14 @@ void disasmAddString(char *psz, const char *pszAdd)
 }
 //*****************************************************************************
 //*****************************************************************************
-void disasmAddStringF(char *psz, uint32_t size, const char *pszFormat, ...)
+void disasmAddStringF(char *psz, const char *pszFormat, ...)
 {
     va_list args;
     va_start(args, pszFormat);
-    RTStrPrintfV(psz + strlen(psz), size, pszFormat, args);
+    size_t  cchCur = strlen(psz);
+    Assert(cchCur < RT_SIZEOFMEMB(OP_PARAMETER, szParam));
+    RTStrPrintfV(psz + cchCur, RT_SIZEOFMEMB(OP_PARAMETER, szParam) - cchCur,
+                 pszFormat, args);
     va_end(args);
 }
 
