@@ -4,7 +4,7 @@
  */
 
 /*
- * Copyright (C) 2006-2009 Sun Microsystems, Inc.
+ * Copyright (C) 2006-2009 Oracle Corporation
  *
  * This file is part of VirtualBox Open Source Edition (OSE), as
  * available from http://www.virtualbox.org. This file is free software;
@@ -13,10 +13,6 @@
  * Foundation, in version 2 as it comes in the "COPYING" file of the
  * VirtualBox OSE distribution. VirtualBox OSE is distributed in the
  * hope that it will be useful, but WITHOUT ANY WARRANTY of any kind.
- *
- * Please contact Sun Microsystems, Inc., 4150 Network Circle, Santa
- * Clara, CA 95054 USA or visit http://www.sun.com if you need
- * additional information or have any questions.
  */
 
 
@@ -115,6 +111,32 @@ int handleControlVM(HandlerArg *a)
         {
             CHECK_ERROR_BREAK(console, Reset());
         }
+        else if (!strcmp(a->argv[1], "unplugcpu"))
+        {
+            if (a->argc <= 1 + 1)
+            {
+                errorArgument("Missing argument to '%s'. Expected CPU number.", a->argv[1]);
+                rc = E_FAIL;
+                break;
+            }
+
+            unsigned n = parseNum(a->argv[2], 32, "CPU");
+
+            CHECK_ERROR_BREAK(sessionMachine, HotUnplugCPU(n));
+        }
+        else if (!strcmp(a->argv[1], "plugcpu"))
+        {
+            if (a->argc <= 1 + 1)
+            {
+                errorArgument("Missing argument to '%s'. Expected CPU number.", a->argv[1]);
+                rc = E_FAIL;
+                break;
+            }
+
+            unsigned n = parseNum(a->argv[2], 32, "CPU");
+
+            CHECK_ERROR_BREAK(sessionMachine, HotPlugCPU(n));
+        }
         else if (!strcmp(a->argv[1], "poweroff"))
         {
             ComPtr<IProgress> progress;
@@ -136,8 +158,16 @@ int handleControlVM(HandlerArg *a)
         }
         else if (!strcmp(a->argv[1], "savestate"))
         {
+            /* first pause so we don't trigger a live save which needs more time/resources */
+            CHECK_ERROR_BREAK(console, Pause());
+
             ComPtr<IProgress> progress;
-            CHECK_ERROR_BREAK(console, SaveState(progress.asOutParam()));
+            CHECK_ERROR(console, SaveState(progress.asOutParam()));
+            if (FAILED(rc))
+            {
+                console->Resume();
+                break;
+            }
 
             rc = showProgress(progress);
             if (FAILED(rc))
@@ -151,6 +181,7 @@ int handleControlVM(HandlerArg *a)
                 {
                     RTPrintf("Error: failed to save machine state. No error message available!\n");
                 }
+                console->Resume();
             }
         }
         else if (!strcmp(a->argv[1], "acpipowerbutton"))
@@ -526,6 +557,25 @@ int handleControlVM(HandlerArg *a)
                 CHECK_ERROR_BREAK(vrdpServer, COMSETTER(Ports)(vrdpports));
             }
         }
+        else if (!strcmp(a->argv[1], "vrdpvideochannelquality"))
+        {
+            if (a->argc <= 1 + 1)
+            {
+                errorArgument("Missing argument to '%s'", a->argv[1]);
+                rc = E_FAIL;
+                break;
+            }
+            /* get the corresponding VRDP server */
+            ComPtr<IVRDPServer> vrdpServer;
+            sessionMachine->COMGETTER(VRDPServer)(vrdpServer.asOutParam());
+            ASSERT(vrdpServer);
+            if (vrdpServer)
+            {
+                unsigned n = parseNum(a->argv[2], 100, "VRDP video channel quality in percent");
+
+                CHECK_ERROR(vrdpServer, COMSETTER(VideoChannelQuality)(n));
+            }
+        }
 #endif /* VBOX_WITH_VRDP */
         else if (   !strcmp(a->argv[1], "usbattach")
                  || !strcmp(a->argv[1], "usbdetach"))
@@ -617,6 +667,7 @@ int handleControlVM(HandlerArg *a)
             CHECK_ERROR_BREAK(console, COMGETTER(Guest)(guest.asOutParam()));
             CHECK_ERROR_BREAK(guest, SetCredentials(Bstr(a->argv[2]), Bstr(a->argv[3]), Bstr(a->argv[4]), fAllowLocalLogon));
         }
+#if 0 /* TODO: review & remove */
         else if (!strcmp(a->argv[1], "dvdattach"))
         {
             Bstr uuid;
@@ -737,9 +788,8 @@ int handleControlVM(HandlerArg *a)
             floppyMedium->COMGETTER(Id)(uuid.asOutParam());
             CHECK_ERROR(machine, MountMedium(Bstr("Floppy Controller"), 0, 0, uuid, FALSE /* aForce */));
         }
-#ifdef VBOX_WITH_MEM_BALLOONING
-        else if (   !strcmp(a->argv[1], "--guestmemoryballoon")
-                 || !strcmp(a->argv[1], "-guestmemoryballoon"))
+#endif /* obsolete dvdattach/floppyattach */
+        else if (!strcmp(a->argv[1], "guestmemoryballoon"))
         {
             if (a->argc != 3)
             {
@@ -756,41 +806,12 @@ int handleControlVM(HandlerArg *a)
                 rc = E_FAIL;
                 break;
             }
-
             /* guest is running; update IGuest */
             ComPtr <IGuest> guest;
-
             rc = console->COMGETTER(Guest)(guest.asOutParam());
             if (SUCCEEDED(rc))
                 CHECK_ERROR(guest, COMSETTER(MemoryBalloonSize)(uVal));
         }
-#endif
-        else if (   !strcmp(a->argv[1], "--gueststatisticsinterval")
-                 || !strcmp(a->argv[1], "-gueststatisticsinterval"))
-        {
-            if (a->argc != 3)
-            {
-                errorSyntax(USAGE_CONTROLVM, "Incorrect number of parameters");
-                rc = E_FAIL;
-                break;
-            }
-            uint32_t uVal;
-            int vrc;
-            vrc = RTStrToUInt32Ex(a->argv[2], NULL, 0, &uVal);
-            if (vrc != VINF_SUCCESS)
-            {
-                errorArgument("Error parsing guest statistics interval '%s'", a->argv[2]);
-                rc = E_FAIL;
-                break;
-            }
-
-            /* guest is running; update IGuest */
-            ComPtr <IGuest> guest;
-
-            rc = console->COMGETTER(Guest)(guest.asOutParam());
-            if (SUCCEEDED(rc))
-                CHECK_ERROR(guest, COMSETTER(StatisticsUpdateInterval)(uVal));
-        }
         else if (!strcmp(a->argv[1], "teleport"))
         {
             Bstr        bstrHostname;
@@ -808,7 +829,7 @@ int handleControlVM(HandlerArg *a)
                 { "--timeout",     't', RTGETOPT_REQ_UINT32 }
             };
             RTGETOPTSTATE GetOptState;
-            RTGetOptInit(&GetOptState, a->argc, a->argv, s_aTeleportOptions, RT_ELEMENTS(s_aTeleportOptions), 2, 0 /*fFlags*/);
+            RTGetOptInit(&GetOptState, a->argc, a->argv, s_aTeleportOptions, RT_ELEMENTS(s_aTeleportOptions), 2, RTGETOPTINIT_FLAGS_NO_STD_OPTS);
             int ch;
             RTGETOPTUNION Value;
             while (   SUCCEEDED(rc)
