@@ -4,7 +4,7 @@
  */
 
 /*
- * Copyright (C) 2006-2009 Oracle Corporation
+ * Copyright (C) 2006-2010 Oracle Corporation
  *
  * This file is part of VirtualBox Open Source Edition (OSE), as
  * available from http://www.virtualbox.org. This file is free software;
@@ -359,7 +359,7 @@ int handleUSBFilter (HandlerArg *a)
                     || (   !cmd.mGlobal
                         && !cmd.mMachine)
                     || (   cmd.mGlobal
-                        && cmd.mFilter.mRemote)
+                        && !cmd.mFilter.mRemote.isEmpty())
                    )
                 {
                     return errorSyntax(USAGE_USBFILTER_ADD, "Mandatory options not supplied");
