@@ -1,3 +1,4 @@
+/* $Id: VBoxVMInformationDlg.cpp $ */
 /** @file
  *
  * VBox frontends: Qt4 GUI ("VirtualBox"):
@@ -5,7 +6,7 @@
  */
 
 /*
- * Copyright (C) 2006-2009 Sun Microsystems, Inc.
+ * Copyright (C) 2006-2009 Oracle Corporation
  *
  * This file is part of VirtualBox Open Source Edition (OSE), as
  * available from http://www.virtualbox.org. This file is free software;
@@ -14,31 +15,36 @@
  * Foundation, in version 2 as it comes in the "COPYING" file of the
  * VirtualBox OSE distribution. VirtualBox OSE is distributed in the
  * hope that it will be useful, but WITHOUT ANY WARRANTY of any kind.
- *
- * Please contact Sun Microsystems, Inc., 4150 Network Circle, Santa
- * Clara, CA 95054 USA or visit http://www.sun.com if you need
- * additional information or have any questions.
  */
 
+#ifdef VBOX_WITH_PRECOMPILED_HEADERS
+# include "precomp.h"
+#else  /* !VBOX_WITH_PRECOMPILED_HEADERS */
 /* Global Includes */
 #include <QTimer>
+#include <QScrollBar>
 
 /* Local Includes */
 #include <VBoxVMInformationDlg.h>
 #include <VBoxGlobal.h>
-#include <VBoxConsoleView.h>
+#endif /* !VBOX_WITH_PRECOMPILED_HEADERS */
+
+#include "UIMachineLogic.h"
+#include "UIMachineWindow.h"
+#include "UIMachineView.h"
+#include "UISession.h"
 
 VBoxVMInformationDlg::InfoDlgMap VBoxVMInformationDlg::mSelfArray = InfoDlgMap();
 
-void VBoxVMInformationDlg::createInformationDlg (const CSession &aSession, VBoxConsoleView *aConsole)
+void VBoxVMInformationDlg::createInformationDlg(UIMachineWindow *pMachineWindow)
 {
-    CMachine machine = aSession.GetMachine();
+    CMachine machine = pMachineWindow->machineLogic()->uisession()->session().GetMachine();
     if (mSelfArray.find (machine.GetName()) == mSelfArray.end())
     {
         /* Creating new information dialog if there is no one existing */
-        VBoxVMInformationDlg *id = new VBoxVMInformationDlg (aConsole, aSession, Qt::Window);
-        id->centerAccording (aConsole);
-        connect (vboxGlobal().mainWindow(), SIGNAL (closing()), id, SLOT (close()));
+        VBoxVMInformationDlg *id = new VBoxVMInformationDlg(pMachineWindow, Qt::Window);
+        id->centerAccording (pMachineWindow->machineWindow());
+        // TODO_NEW_CORE: this seems not necessary, cause we set WA_DeleteOnClose.
         id->setAttribute (Qt::WA_DeleteOnClose);
         mSelfArray [machine.GetName()] = id;
     }
@@ -50,15 +56,14 @@ void VBoxVMInformationDlg::createInformationDlg (const CSession &aSession, VBoxC
     info->activateWindow();
 }
 
-VBoxVMInformationDlg::VBoxVMInformationDlg (VBoxConsoleView *aConsole, const CSession &aSession, Qt::WindowFlags aFlags)
-#ifdef Q_WS_MAC
-    : QIWithRetranslateUI2 <QIMainDialog> (aConsole, aFlags)
-#else /* Q_WS_MAC */
+VBoxVMInformationDlg::VBoxVMInformationDlg (UIMachineWindow *pMachineWindow, Qt::WindowFlags aFlags)
+# ifdef Q_WS_MAC
+    : QIWithRetranslateUI2 <QIMainDialog> (pMachineWindow->machineWindow(), aFlags)
+# else /* Q_WS_MAC */
     : QIWithRetranslateUI2 <QIMainDialog> (0, aFlags)
-#endif /* Q_WS_MAC */
+# endif /* Q_WS_MAC */
+    , mSession (pMachineWindow->session())
     , mIsPolished (false)
-    , mConsole (aConsole)
-    , mSession (aSession)
     , mStatTimer (new QTimer (this))
 {
     /* Apply UI decorations */
@@ -89,12 +94,14 @@ VBoxVMInformationDlg::VBoxVMInformationDlg (VBoxConsoleView *aConsole, const CSe
     mStatisticText->setViewportMargins (5, 5, 5, 5);
 
     /* Setup handlers */
+    connect (pMachineWindow->uisession(), SIGNAL (sigMediumChange(const CMediumAttachment&)), this, SLOT (updateDetails()));
+    connect (pMachineWindow->uisession(), SIGNAL (sigSharedFolderChange()), this, SLOT (updateDetails()));
+    /* TODO_NEW_CORE: this is ofc not really right in the mm sense. There are
+     * more than one screens. */
+    connect (pMachineWindow->machineView(), SIGNAL (resizeHintDone()), this, SLOT (processStatistics()));
     connect (mInfoStack, SIGNAL (currentChanged (int)), this, SLOT (onPageChanged (int)));
     connect (&vboxGlobal(), SIGNAL (mediumEnumFinished (const VBoxMediaList &)), this, SLOT (updateDetails()));
-    connect (mConsole, SIGNAL (mediaDriveChanged (VBoxDefs::MediumType)), this, SLOT (updateDetails()));
-    connect (mConsole, SIGNAL (sharedFoldersChanged()), this, SLOT (updateDetails()));
     connect (mStatTimer, SIGNAL (timeout()), this, SLOT (processStatistics()));
-    connect (mConsole, SIGNAL (resizeHintDone()), this, SLOT (processStatistics()));
 
     /* Loading language constants */
     retranslateUi();
@@ -442,10 +449,13 @@ void VBoxVMInformationDlg::refreshStatistics()
     /* Runtime Information */
     {
         CConsole console = mSession.GetConsole();
-        ULONG bpp = console.GetDisplay().GetBitsPerPixel();
+        ULONG width = 0;
+        ULONG height = 0;
+        ULONG bpp = 0;
+        console.GetDisplay().GetScreenResolution(0, width, height, bpp);
         QString resolution = QString ("%1x%2")
-            .arg (console.GetDisplay().GetWidth())
-            .arg (console.GetDisplay().GetHeight());
+            .arg (width)
+            .arg (height);
         if (bpp)
             resolution += QString ("x%1").arg (bpp);
         QString virtualization = console.GetDebugger().GetHWVirtExEnabled() ?
@@ -454,14 +464,22 @@ void VBoxVMInformationDlg::refreshStatistics()
         QString nested = console.GetDebugger().GetHWVirtExNestedPagingEnabled() ?
             VBoxGlobal::tr ("Enabled", "details report (Nested Paging)") :
             VBoxGlobal::tr ("Disabled", "details report (Nested Paging)");
-        QString addInfo = console.GetGuest().GetAdditionsVersion();
-        uint addVersion = addInfo.toUInt();
-        QString addVerisonStr = !addInfo.isNull() ?
-            tr ("Version %1.%2", "guest additions")
-                .arg (RT_HIWORD (addVersion)).arg (RT_LOWORD (addVersion)) :
-            tr ("Not Detected", "guest additions");
+        QString addVersion = m.GetGuestPropertyValue("/VirtualBox/GuestAdd/Version");
+        QString addRevision = m.GetGuestPropertyValue("/VirtualBox/GuestAdd/Revision");
+        QString addVersionStr;
+        if (!addVersion.isEmpty() && !addRevision.isEmpty())
+        {
+            QString addInfo = console.GetGuest().GetAdditionsVersion();
+            addVersionStr = (addInfo.isEmpty() ? "(" : "")
+                          + addVersion
+                          + " r"
+                          + m.GetGuestPropertyValue("/VirtualBox/GuestAdd/Revision")
+                          + (addInfo.isEmpty() ? ")" : "");
+        }
+        else
+            addVersionStr = tr ("Not Detected", "guest additions");
         QString osType = console.GetGuest().GetOSTypeId();
-        if (osType.isNull())
+        if (osType.isEmpty())
             osType = tr ("Not Detected", "guest os type");
         else
             osType = vboxGlobal().vmGuestOSTypeDescription (osType);
@@ -472,7 +490,7 @@ void VBoxVMInformationDlg::refreshStatistics()
 
         /* Searching for longest string */
         QStringList valuesList;
-        valuesList << resolution << virtualization << nested << addVerisonStr << osType << vrdpInfo;
+        valuesList << resolution << virtualization << nested << addVersionStr << osType << vrdpInfo;
         int maxLength = 0;
         foreach (const QString &value, valuesList)
             maxLength = maxLength < fontMetrics().width (value) ?
@@ -482,7 +500,7 @@ void VBoxVMInformationDlg::refreshStatistics()
         result += formatValue (tr ("Screen Resolution"), resolution, maxLength);
         result += formatValue (VBoxGlobal::tr ("VT-x/AMD-V", "details report"), virtualization, maxLength);
         result += formatValue (VBoxGlobal::tr ("Nested Paging", "details report"), nested, maxLength);
-        result += formatValue (tr ("Guest Additions"), addVerisonStr, maxLength);
+        result += formatValue (tr ("Guest Additions"), addVersionStr, maxLength);
         result += formatValue (tr ("Guest OS Type"), osType, maxLength);
         result += formatValue (VBoxGlobal::tr ("Remote Display Server Port", "details report (VRDP Server)"), vrdpInfo, maxLength);
         result += paragraph;
