@@ -365,8 +365,12 @@ QString QIFileDialog::getExistingDirectory (const QString &aDir,
 #else
 
     QFileDialog::Options o;
+# if defined (Q_WS_X11)
+    if (vboxGlobal().isKWinManaged())
+      o |= QFileDialog::DontUseNativeDialog;
+# endif
     if (aDirOnly)
-        o = QFileDialog::ShowDirsOnly;
+        o |= QFileDialog::ShowDirsOnly;
     if (!aResolveSymlinks)
         o |= QFileDialog::DontResolveSymlinks;
     return QFileDialog::getExistingDirectory (aParent, aCaption, aDir, o);
@@ -573,6 +577,10 @@ QString QIFileDialog::getSaveFileName (const QString &aStartWith,
 #else
 
     QFileDialog::Options o;
+# if defined (Q_WS_X11)
+    if (vboxGlobal().isKWinManaged())
+      o |= QFileDialog::DontUseNativeDialog;
+# endif
     if (!aResolveSymlinks)
         o |= QFileDialog::DontResolveSymlinks;
     o |= QFileDialog::DontConfirmOverwrite;
@@ -633,7 +641,8 @@ QStringList QIFileDialog::getOpenFileNames (const QString &aStartWith,
                                             bool           aResolveSymlinks /* = true */,
                                             bool           aSingleFile /* = false */)
 {
-#if defined Q_WS_WIN
+/* It seems, running QFileDialog in separate thread is NOT needed under windows any more: */
+#if defined (Q_WS_WIN) && (QT_VERSION < 0x040403)
 
     /**
      *  QEvent class reimplementation to carry Win32 API native dialog's
@@ -816,6 +825,11 @@ QStringList QIFileDialog::getOpenFileNames (const QString &aStartWith,
     QFileDialog::Options o;
     if (!aResolveSymlinks)
         o |= QFileDialog::DontResolveSymlinks;
+# if defined (Q_WS_X11)
+    if (vboxGlobal().isKWinManaged())
+      o |= QFileDialog::DontUseNativeDialog;
+# endif
+
     if (aSingleFile)
         return QStringList() << QFileDialog::getOpenFileName (aParent, aCaption, aStartWith,
                                                               aFilters, aSelectedFilter, o);
