@@ -749,6 +749,13 @@ void UISession::powerUp()
 # endif
 #endif
 
+#ifdef VBOX_WITH_VIDEOHWACCEL
+    LogRel(("2D video acceleration is %s.\n",
+           machine.GetAccelerate2DVideoEnabled() && VBoxGlobal::isAcceleration2DVideoAvailable()
+                 ? "enabled"
+                 : "disabled"));
+#endif
+
     /* Warn listeners about machine was started: */
     emit sigMachineStarted();
 }
@@ -898,6 +905,18 @@ void UISession::sltInstallGuestAdditionsFrom(const QString &strSource)
 
 void UISession::sltCloseVirtualSession()
 {
+     /* Use a single shot with timeout 0 to allow the widgets to cleanly close and test then again.
+      * If all open widgets are closed destroy ourself: */
+    QWidget *widget = QApplication::activeModalWidget() ?
+                      QApplication::activeModalWidget() :
+                      QApplication::activePopupWidget() ?
+                      QApplication::activePopupWidget() : 0;
+    if (widget)
+    {
+        widget->hide();
+        QTimer::singleShot(0, this, SLOT(sltCloseVirtualSession()));
+        return;
+    }
     m_pMachine->closeVirtualMachine();
 }
 
@@ -1523,7 +1542,15 @@ void UISession::preparePowerUp()
         vboxProblem().remindAboutAutoCapture();
 
     /* Shows first run wizard if necessary: */
-    if (isFirstTimeStarted())
+    const CMachine &machine = session().GetMachine();
+    /* Check if we are in teleportation waiting mode. In that case no first run
+     * wizard is necessary. */
+    KMachineState state = machine.GetState();
+    if (   isFirstTimeStarted()
+        && !((   state == KMachineState_PoweredOff
+              || state == KMachineState_Aborted
+              || state == KMachineState_Teleported)
+             && machine.GetTeleporterEnabled()))
     {
         UIFirstRunWzd wzd(mainMachineWindow(), session().GetMachine());
         wzd.exec();
