@@ -161,6 +161,7 @@ void VBoxVMSettingsDisplay::getFrom (const CMachine &aMachine)
         mCbVRDPMethod->setCurrentIndex (mCbVRDPMethod->
                                         findText (vboxGlobal().toString (vrdp.GetAuthType())));
         mLeVRDPTimeout->setText (QString::number (vrdp.GetAuthTimeout()));
+        mCbMultipleConn->setChecked(vrdp.GetAllowMultiConnection());
     }
     else
     {
@@ -193,6 +194,7 @@ void VBoxVMSettingsDisplay::putBackTo()
         vrdp.SetPorts (mLeVRDPPort->text());
         vrdp.SetAuthType (vboxGlobal().toVRDPAuthType (mCbVRDPMethod->currentText()));
         vrdp.SetAuthTimeout (mLeVRDPTimeout->text().toULong());
+        vrdp.SetAllowMultiConnection(mCbMultipleConn->isChecked());
     }
 }
 
@@ -264,6 +266,7 @@ void VBoxVMSettingsDisplay::setOrderAfter (QWidget *aWidget)
     setTabOrder (mCbVRDP, mLeVRDPPort);
     setTabOrder (mLeVRDPPort, mCbVRDPMethod);
     setTabOrder (mCbVRDPMethod, mLeVRDPTimeout);
+    setTabOrder (mLeVRDPTimeout, mCbMultipleConn);
 }
 
 void VBoxVMSettingsDisplay::retranslateUi()
