@@ -5,7 +5,7 @@
  */
 
 /*
- * Copyright (C) 2008 Sun Microsystems, Inc.
+ * Copyright (C) 2008 Oracle Corporation
  *
  * This file is part of VirtualBox Open Source Edition (OSE), as
  * available from http://www.virtualbox.org. This file is free software;
@@ -14,13 +14,10 @@
  * Foundation, in version 2 as it comes in the "COPYING" file of the
  * VirtualBox OSE distribution. VirtualBox OSE is distributed in the
  * hope that it will be useful, but WITHOUT ANY WARRANTY of any kind.
- *
- * Please contact Sun Microsystems, Inc., 4150 Network Circle, Santa
- * Clara, CA 95054 USA or visit http://www.sun.com if you need
- * additional information or have any questions.
  */
 
 #include "state.h"
+#include "state_internals.h"
 #include "state/cr_statetypes.h"
 #include "state/cr_texture.h"
 #include "cr_mem.h"
@@ -53,9 +50,6 @@
  * It could be done for the first way as well, but requires tons of bit checks.
  */
 
-/*@todo move with the one from state_texture.c to some header*/
-#define MAX_MIPMAP_LEVELS 20
-
 static int32_t crStateAllocAndSSMR3GetMem(PSSMHANDLE pSSM, void **pBuffer, size_t cbBuffer)
 {
     CRASSERT(pSSM && pBuffer && cbBuffer>0);
@@ -70,14 +64,16 @@ static int32_t crStateAllocAndSSMR3GetMem(PSSMHANDLE pSSM, void **pBuffer, size_
 static int32_t crStateSaveTextureObjData(CRTextureObj *pTexture, PSSMHANDLE pSSM)
 {
     int32_t rc, face, i;
+    GLboolean bound = GL_FALSE;
     
     CRASSERT(pTexture && pSSM);
 
+    crDebug("crStateSaveTextureObjData %u. START", pTexture->name);
+
     for (face = 0; face < 6; face++) {
         CRASSERT(pTexture->level[face]);
 
-        /*@todo, check if safe to go till MAX_MIPMAP_LEVELS intead of TextureState->maxLevel*/
-        for (i = 0; i < MAX_MIPMAP_LEVELS; i++) {
+        for (i = 0; i < CR_MAX_MIPMAP_LEVELS; i++) {
             CRTextureLevel *ptl = &(pTexture->level[face][i]);
             rc = SSMR3PutMem(pSSM, ptl, sizeof(*ptl));
             AssertRCReturn(rc, rc);
@@ -97,12 +93,67 @@ static int32_t crStateSaveTextureObjData(CRTextureObj *pTexture, PSSMHANDLE pSSM
             else if (ptl->bytes)
             {
                 char *pImg;
+                GLenum target;
 
+                if (!bound)
+                {
+                    diff_api.BindTexture(pTexture->target, pTexture->name);
+                    bound = GL_TRUE;
+                }
+
+                if (pTexture->target!=GL_TEXTURE_CUBE_MAP_ARB)
+                {
+                    target = pTexture->target;
+                }
+                else
+                {
+                    target = GL_TEXTURE_CUBE_MAP_POSITIVE_X + face;
+                }
+
+#ifdef DEBUG
+                pImg = crAlloc(ptl->bytes+4);
+#else
                 pImg = crAlloc(ptl->bytes);
+#endif
                 if (!pImg) return VERR_NO_MEMORY;
 
-                diff_api.BindTexture(pTexture->target, pTexture->name);
-                diff_api.GetTexImage(pTexture->target, i, ptl->format, ptl->type, pImg);
+#ifdef DEBUG
+                {
+                    GLint w,h=0;
+                    *(int*)((char*)pImg+ptl->bytes) = 0xDEADDEAD;
+                    crDebug("get image: compressed %i, face %i, level %i, width %i, height %i, bytes %i",
+                            ptl->compressed, face, i, ptl->width, ptl->height, ptl->bytes);
+                    diff_api.GetTexLevelParameteriv(target, i, GL_TEXTURE_WIDTH, &w);
+                    diff_api.GetTexLevelParameteriv(target, i, GL_TEXTURE_HEIGHT, &h);
+                    if (w!=ptl->width || h!=ptl->height)
+                    {
+                        crWarning("!!!tex size mismatch %i, %i!!!", w, h);
+                    }
+                }
+#endif
+
+                /*@todo: ugly workaround for crashes inside ati driver,
+                 *       they overwrite their own allocated memory in cases where texlevel >=4
+                         and width or height <=2.
+                 */
+                if (i<4 || (ptl->width>2 && ptl->height>2))
+                {
+                    if (!ptl->compressed)
+                    {
+                        diff_api.GetTexImage(target, i, ptl->format, ptl->type, pImg);
+                    }
+                    else
+                    {
+                        diff_api.GetCompressedTexImageARB(target, i, pImg);
+                    }
+                }
+
+#ifdef DEBUG
+                if (*(int*)((char*)pImg+ptl->bytes) != 0xDEADDEAD)
+                {
+                    crWarning("Texture is bigger than expected!!!");
+                }
+#endif
 
                 rc = SSMR3PutMem(pSSM, pImg, ptl->bytes);
                 crFree(pImg);
@@ -112,6 +163,8 @@ static int32_t crStateSaveTextureObjData(CRTextureObj *pTexture, PSSMHANDLE pSSM
         }
     }
 
+    crDebug("crStateSaveTextureObjData %u. END", pTexture->name);
+
     return VINF_SUCCESS;
 }
 
@@ -124,7 +177,7 @@ static int32_t crStateLoadTextureObjData(CRTextureObj *pTexture, PSSMHANDLE pSSM
     for (face = 0; face < 6; face++) {
         CRASSERT(pTexture->level[face]);
 
-        for (i = 0; i < MAX_MIPMAP_LEVELS; i++) {
+        for (i = 0; i < CR_MAX_MIPMAP_LEVELS; i++) {
             CRTextureLevel *ptl = &(pTexture->level[face][i]);
             CRASSERT(!ptl->img);
 
@@ -152,8 +205,6 @@ static int32_t crStateLoadTextureObjData(CRTextureObj *pTexture, PSSMHANDLE pSSM
             }
 #endif
             crStateTextureInitTextureFormat(ptl, ptl->internalFormat);
-
-            //FILLDIRTY(ptl->dirty);
         }
     }
 
@@ -797,6 +848,164 @@ static void crStateSaveGLSLProgramCB(unsigned long key, void *data1, void *data2
     }
 }
 
+static int32_t crStateSaveClientPointer(CRVertexArrays *pArrays, int32_t index, PSSMHANDLE pSSM)
+{
+    int32_t rc;
+    CRClientPointer *cp;
+
+    cp = crStateGetClientPointerByIndex(index, pArrays);
+
+    rc = SSMR3PutU32(pSSM, cp->buffer->name);
+    AssertRCReturn(rc, rc);
+
+#ifdef CR_EXT_compiled_vertex_array
+    if (cp->locked)
+    {
+        CRASSERT(cp->p);
+        rc = SSMR3PutMem(pSSM, cp->p, cp->stride*(pArrays->lockFirst+pArrays->lockCount));
+        AssertRCReturn(rc, rc);
+    }
+#endif
+
+    return VINF_SUCCESS;
+}
+
+static int32_t crStateLoadClientPointer(CRVertexArrays *pArrays, int32_t index, CRContext *pContext, PSSMHANDLE pSSM)
+{
+    int32_t rc;
+    uint32_t ui;
+    CRClientPointer *cp;
+
+    cp = crStateGetClientPointerByIndex(index, pArrays);
+
+    rc = SSMR3GetU32(pSSM, &ui);
+    AssertRCReturn(rc, rc);
+    cp->buffer = ui==0 ? pContext->bufferobject.nullBuffer : crHashtableSearch(pContext->bufferobject.buffers, ui);
+
+#ifdef CR_EXT_compiled_vertex_array
+    if (cp->locked)
+    {
+        rc = crStateAllocAndSSMR3GetMem(pSSM, (void**)&cp->p, cp->stride*(pArrays->lockFirst+pArrays->lockCount));
+        AssertRCReturn(rc, rc);
+    }
+#endif
+
+    return VINF_SUCCESS;
+}
+
+static int32_t crStateSaveCurrentBits(CRStateBits *pBits, PSSMHANDLE pSSM)
+{
+    int32_t rc, i;
+
+    rc = SSMR3PutMem(pSSM, pBits, sizeof(*pBits));
+    AssertRCReturn(rc, rc);
+
+    rc = SSMR3PutMem(pSSM, pBits->client.v, GLCLIENT_BIT_ALLOC*sizeof(CRbitvalue));
+    AssertRCReturn(rc, rc);
+    rc = SSMR3PutMem(pSSM, pBits->client.n, GLCLIENT_BIT_ALLOC*sizeof(CRbitvalue));
+    AssertRCReturn(rc, rc);
+    rc = SSMR3PutMem(pSSM, pBits->client.c, GLCLIENT_BIT_ALLOC*sizeof(CRbitvalue));
+    AssertRCReturn(rc, rc);
+    rc = SSMR3PutMem(pSSM, pBits->client.s, GLCLIENT_BIT_ALLOC*sizeof(CRbitvalue));
+    AssertRCReturn(rc, rc);
+    rc = SSMR3PutMem(pSSM, pBits->client.i, GLCLIENT_BIT_ALLOC*sizeof(CRbitvalue));
+    AssertRCReturn(rc, rc);
+    for (i=0; i<CR_MAX_TEXTURE_UNITS; i++)
+    {
+        rc = SSMR3PutMem(pSSM, pBits->client.t[i], GLCLIENT_BIT_ALLOC*sizeof(CRbitvalue));
+        AssertRCReturn(rc, rc);
+    }
+    rc = SSMR3PutMem(pSSM, pBits->client.e, GLCLIENT_BIT_ALLOC*sizeof(CRbitvalue));
+    AssertRCReturn(rc, rc);
+    rc = SSMR3PutMem(pSSM, pBits->client.f, GLCLIENT_BIT_ALLOC*sizeof(CRbitvalue));
+    AssertRCReturn(rc, rc);
+#ifdef CR_NV_vertex_program
+    for (i=0; i<CR_MAX_VERTEX_ATTRIBS; i++)
+    {
+        rc = SSMR3PutMem(pSSM, pBits->client.a[i], GLCLIENT_BIT_ALLOC*sizeof(CRbitvalue));
+        AssertRCReturn(rc, rc);
+    }
+#endif
+
+    rc = SSMR3PutMem(pSSM, pBits->lighting.light, CR_MAX_LIGHTS*sizeof(pBits->lighting.light));
+    AssertRCReturn(rc, rc);
+
+    return VINF_SUCCESS;
+}
+
+static int32_t crStateLoadCurrentBits(CRStateBits *pBits, PSSMHANDLE pSSM)
+{
+    int32_t rc, i;
+    CRClientBits client;
+    CRLightingBits lighting;
+
+    CRASSERT(pBits);
+
+    client.v = pBits->client.v;
+    client.n = pBits->client.n;
+    client.c = pBits->client.c;
+    client.s = pBits->client.s;
+    client.i = pBits->client.i;
+    client.e = pBits->client.e;
+    client.f = pBits->client.f;
+    for (i=0; i<CR_MAX_TEXTURE_UNITS; i++)
+    {
+        client.t[i] = pBits->client.t[i];
+    }
+#ifdef CR_NV_vertex_program
+    for (i=0; i<CR_MAX_VERTEX_ATTRIBS; i++)
+    {
+        client.a[i] = pBits->client.a[i];
+    }
+#endif
+    lighting.light = pBits->lighting.light;
+
+    rc = SSMR3GetMem(pSSM, pBits, sizeof(*pBits));
+    AssertRCReturn(rc, rc);
+
+    pBits->client.v = client.v;
+    rc = SSMR3GetMem(pSSM, pBits->client.v, GLCLIENT_BIT_ALLOC*sizeof(CRbitvalue));
+    AssertRCReturn(rc, rc);
+    pBits->client.n = client.n;
+    rc = SSMR3GetMem(pSSM, pBits->client.n, GLCLIENT_BIT_ALLOC*sizeof(CRbitvalue));
+    AssertRCReturn(rc, rc);
+    pBits->client.c = client.c;
+    rc = SSMR3GetMem(pSSM, pBits->client.c, GLCLIENT_BIT_ALLOC*sizeof(CRbitvalue));
+    AssertRCReturn(rc, rc);
+    pBits->client.s = client.s;
+    rc = SSMR3GetMem(pSSM, pBits->client.s, GLCLIENT_BIT_ALLOC*sizeof(CRbitvalue));
+    AssertRCReturn(rc, rc);
+    pBits->client.i = client.i;
+    rc = SSMR3GetMem(pSSM, pBits->client.i, GLCLIENT_BIT_ALLOC*sizeof(CRbitvalue));
+    AssertRCReturn(rc, rc);
+    pBits->client.e = client.e;
+    rc = SSMR3GetMem(pSSM, pBits->client.e, GLCLIENT_BIT_ALLOC*sizeof(CRbitvalue));
+    AssertRCReturn(rc, rc);
+    pBits->client.f = client.f;
+    rc = SSMR3GetMem(pSSM, pBits->client.f, GLCLIENT_BIT_ALLOC*sizeof(CRbitvalue));
+    AssertRCReturn(rc, rc);
+    for (i=0; i<CR_MAX_TEXTURE_UNITS; i++)
+    {
+        pBits->client.t[i] = client.t[i];
+        rc = SSMR3GetMem(pSSM, pBits->client.t[i], GLCLIENT_BIT_ALLOC*sizeof(CRbitvalue));
+        AssertRCReturn(rc, rc);
+    }
+#ifdef CR_NV_vertex_program
+    for (i=0; i<CR_MAX_VERTEX_ATTRIBS; i++)
+    {
+        pBits->client.a[i] = client.a[i];
+        rc = SSMR3GetMem(pSSM, pBits->client.a[i], GLCLIENT_BIT_ALLOC*sizeof(CRbitvalue));
+        AssertRCReturn(rc, rc);
+    }
+#endif
+
+    pBits->lighting.light = lighting.light;
+    rc = SSMR3GetMem(pSSM, pBits->lighting.light, CR_MAX_LIGHTS*sizeof(pBits->lighting.light));
+    AssertRCReturn(rc, rc);
+
+    return VINF_SUCCESS;
+}
+
 int32_t crStateSaveContext(CRContext *pContext, PSSMHANDLE pSSM)
 {
     int32_t rc, i;
@@ -972,33 +1181,29 @@ int32_t crStateSaveContext(CRContext *pContext, PSSMHANDLE pSSM)
     AssertRCReturn(rc, rc);
     rc = SSMR3PutU32(pSSM, pContext->bufferobject.elementsBuffer->name);
     AssertRCReturn(rc, rc);
-    /* Save bound array buffers*/ /*@todo vertexArrayStack*/
-    rc = SSMR3PutU32(pSSM, pContext->client.array.v.buffer->name);
+#ifdef CR_ARB_pixel_buffer_object
+    rc = SSMR3PutU32(pSSM, pContext->bufferobject.packBuffer->name);
     AssertRCReturn(rc, rc);
-    rc = SSMR3PutU32(pSSM, pContext->client.array.c.buffer->name);
+    rc = SSMR3PutU32(pSSM, pContext->bufferobject.unpackBuffer->name);
     AssertRCReturn(rc, rc);
-    rc = SSMR3PutU32(pSSM, pContext->client.array.f.buffer->name);
-    AssertRCReturn(rc, rc);
-    rc = SSMR3PutU32(pSSM, pContext->client.array.s.buffer->name);
-    AssertRCReturn(rc, rc);
-    rc = SSMR3PutU32(pSSM, pContext->client.array.e.buffer->name);
-    AssertRCReturn(rc, rc);
-    rc = SSMR3PutU32(pSSM, pContext->client.array.i.buffer->name);
-    AssertRCReturn(rc, rc);
-    rc = SSMR3PutU32(pSSM, pContext->client.array.n.buffer->name);
-    AssertRCReturn(rc, rc);
-    for (i = 0; i < CR_MAX_TEXTURE_UNITS; i++)
+#endif
+    /* Save clint pointers and buffer bindings*/
+    for (i=0; i<CRSTATECLIENT_MAX_VERTEXARRAYS; ++i)
     {
-        rc = SSMR3PutU32(pSSM, pContext->client.array.t[i].buffer->name);
+        rc = crStateSaveClientPointer(&pContext->client.array, i, pSSM);
         AssertRCReturn(rc, rc);
     }
-# ifdef CR_NV_vertex_program
-    for (i = 0; i < CR_MAX_VERTEX_ATTRIBS; i++)
+
+    crDebug("client.vertexArrayStackDepth %i", pContext->client.vertexArrayStackDepth);
+    for (i=0; i<pContext->client.vertexArrayStackDepth; ++i)
     {
-        rc = SSMR3PutU32(pSSM, pContext->client.array.a[i].buffer->name);
-        AssertRCReturn(rc, rc);
+        CRVertexArrays *pArray = &pContext->client.vertexArrayStack[i];
+        for (j=0; j<CRSTATECLIENT_MAX_VERTEXARRAYS; ++j)
+        {
+            rc = crStateSaveClientPointer(pArray, j, pSSM);
+            AssertRCReturn(rc, rc);
+        }
     }
-# endif
 #endif /*CR_ARB_vertex_buffer_object*/
 
     /* Save pixel/vertex programs */
@@ -1070,8 +1275,11 @@ int32_t crStateSaveContext(CRContext *pContext, PSSMHANDLE pSSM)
         diff_api.PixelStorei(GL_PACK_SWAP_BYTES, 0);
         diff_api.PixelStorei(GL_PACK_LSB_FIRST, 0);
 
+        diff_api.ReadBuffer(GL_FRONT);
         diff_api.ReadPixels(0, 0, pVP->viewportW, pVP->viewportH, GL_RGBA, GL_UNSIGNED_BYTE, pData);
 
+        diff_api.ReadBuffer(pContext->framebufferobject.readFB ? 
+                            pContext->framebufferobject.readFB->readbuffer : pContext->buffer.readBuffer);
         diff_api.PixelStorei(GL_PACK_SKIP_ROWS, packing.skipRows);
         diff_api.PixelStorei(GL_PACK_SKIP_PIXELS, packing.skipPixels);
         diff_api.PixelStorei(GL_PACK_ALIGNMENT, packing.alignment);
@@ -1324,12 +1532,9 @@ int32_t crStateLoadContext(CRContext *pContext, PSSMHANDLE pSSM)
         rc = SSMR3GetMem(pSSM, pTexture, sizeof(*pTexture));
         AssertRCReturn(rc, rc);
 
-        //DIRTY(pTexture->dirty, pContext->neg_bitid);
-        //DIRTY(pTexture->imageBit, pContext->neg_bitid);
-
         /*allocate actual memory*/
         for (i=0; i<6; ++i) {
-            pTexture->level[i] = (CRTextureLevel *) crCalloc(sizeof(CRTextureLevel) * MAX_MIPMAP_LEVELS);
+            pTexture->level[i] = (CRTextureLevel *) crCalloc(sizeof(CRTextureLevel) * CR_MAX_MIPMAP_LEVELS);
             if (!pTexture->level[i]) return VERR_NO_MEMORY;
         }
 
@@ -1345,7 +1550,6 @@ int32_t crStateLoadContext(CRContext *pContext, PSSMHANDLE pSSM)
         rc = crStateLoadTexUnitCurrentTexturePtrs(&pContext->texture.unit[i], pContext, pSSM);
         AssertRCReturn(rc, rc);
     }
-    //FILLDIRTY(GetCurrentBits()->texture.dirty);
 
     /* Mark textures for resending to GPU */
     pContext->texture.bResyncNeeded = GL_TRUE;
@@ -1442,10 +1646,6 @@ int32_t crStateLoadContext(CRContext *pContext, PSSMHANDLE pSSM)
             pBufferObj->data = crAlloc(pBufferObj->size);
             rc = SSMR3GetMem(pSSM, pBufferObj->data, pBufferObj->size);
             AssertRCReturn(rc, rc);
-
-            //DIRTY(pBufferObj->dirty, pContext->neg_bitid);
-            //pBufferObj->dirtyStart = 0;
-            //pBufferObj->dirtyLength = pBufferObj->size;
         } 
         else if (pBufferObj->name!=0 && pBufferObj->size>0)
         {
@@ -1464,7 +1664,6 @@ int32_t crStateLoadContext(CRContext *pContext, PSSMHANDLE pSSM)
         if (key!=0)
             crHashtableAdd(pContext->bufferobject.buffers, key, pBufferObj);        
     }
-    //FILLDIRTY(GetCurrentBits()->bufferobject.dirty);
     /* Load pointers */
 #define CRS_GET_BO(name) (((name)==0) ? (pContext->bufferobject.nullBuffer) : crHashtableSearch(pContext->bufferobject.buffers, name))
     rc = SSMR3GetU32(pSSM, &ui);
@@ -1473,52 +1672,32 @@ int32_t crStateLoadContext(CRContext *pContext, PSSMHANDLE pSSM)
     rc = SSMR3GetU32(pSSM, &ui);
     AssertRCReturn(rc, rc);
     pContext->bufferobject.elementsBuffer = CRS_GET_BO(ui);
-
-    /* Load bound array buffers*/
+#ifdef CR_ARB_pixel_buffer_object
     rc = SSMR3GetU32(pSSM, &ui);
     AssertRCReturn(rc, rc);
-    pContext->client.array.v.buffer = CRS_GET_BO(ui);
-
+    pContext->bufferobject.packBuffer = CRS_GET_BO(ui);
     rc = SSMR3GetU32(pSSM, &ui);
     AssertRCReturn(rc, rc);
-    pContext->client.array.c.buffer = CRS_GET_BO(ui);
-
-    rc = SSMR3GetU32(pSSM, &ui);
-    AssertRCReturn(rc, rc);
-    pContext->client.array.f.buffer = CRS_GET_BO(ui);
-
-    rc = SSMR3GetU32(pSSM, &ui);
-    AssertRCReturn(rc, rc);
-    pContext->client.array.s.buffer = CRS_GET_BO(ui);
-
-    rc = SSMR3GetU32(pSSM, &ui);
-    AssertRCReturn(rc, rc);
-    pContext->client.array.e.buffer = CRS_GET_BO(ui);
-
-    rc = SSMR3GetU32(pSSM, &ui);
-    AssertRCReturn(rc, rc);
-    pContext->client.array.i.buffer = CRS_GET_BO(ui);
-
-    rc = SSMR3GetU32(pSSM, &ui);
-    AssertRCReturn(rc, rc);
-    pContext->client.array.n.buffer = CRS_GET_BO(ui);
-
-    for (i = 0; i < CR_MAX_TEXTURE_UNITS; i++)
-    {
-        rc = SSMR3GetU32(pSSM, &ui);
-        AssertRCReturn(rc, rc);
-        pContext->client.array.t[i].buffer = CRS_GET_BO(ui);
-    }
-# ifdef CR_NV_vertex_program
-    for (i = 0; i < CR_MAX_VERTEX_ATTRIBS; i++)
-    {
-        rc = SSMR3GetU32(pSSM, &ui);
-        AssertRCReturn(rc, rc);
-        pContext->client.array.a[i].buffer = CRS_GET_BO(ui);
-    }
-# endif
+    pContext->bufferobject.unpackBuffer = CRS_GET_BO(ui);
+#endif
 #undef CRS_GET_BO
 
+    /* Load client pointers and array buffer bindings*/
+    for (i=0; i<CRSTATECLIENT_MAX_VERTEXARRAYS; ++i)
+    {
+        rc = crStateLoadClientPointer(&pContext->client.array, i, pContext, pSSM);
+        AssertRCReturn(rc, rc);
+    }
+    for (j=0; j<pContext->client.vertexArrayStackDepth; ++j)
+    {
+        CRVertexArrays *pArray = &pContext->client.vertexArrayStack[j];
+        for (i=0; i<CRSTATECLIENT_MAX_VERTEXARRAYS; ++i)
+        {
+            rc = crStateLoadClientPointer(pArray, i, pContext, pSSM);
+            AssertRCReturn(rc, rc);
+        }
+    }
+
     pContext->bufferobject.bResyncNeeded = GL_TRUE;
 #endif
 
@@ -1528,10 +1707,8 @@ int32_t crStateLoadContext(CRContext *pContext, PSSMHANDLE pSSM)
     /* Load defauls programs */
     rc = crStateLoadProgram(&pContext->program.defaultVertexProgram, pSSM);
     AssertRCReturn(rc, rc);
-    //FILLDIRTY(pContext->program.defaultVertexProgram->dirtyProgram);
     rc = crStateLoadProgram(&pContext->program.defaultFragmentProgram, pSSM);
     AssertRCReturn(rc, rc);
-    //FILLDIRTY(pContext->program.defaultFragmentProgram->dirtyProgram);
     /* Load all the rest */
     for (ui=0; ui<uiNumElems; ++ui)
     {
@@ -1542,7 +1719,6 @@ int32_t crStateLoadContext(CRContext *pContext, PSSMHANDLE pSSM)
         //DIRTY(pProgram->dirtyProgram, pContext->neg_bitid);
         
     }
-    //FILLDIRTY(GetCurrentBits()->program.dirty);
     /* Load Pointers */
     rc = SSMR3GetU32(pSSM, &ui);
     AssertRCReturn(rc, rc);
@@ -1729,42 +1905,293 @@ int32_t crStateLoadContext(CRContext *pContext, PSSMHANDLE pSSM)
     pContext->glsl.bResyncNeeded = GL_TRUE;
 #endif
 
+
+    /*Restore front buffer image*/
     {
         CRViewportState *pVP = &pContext->viewport;
-        CRPixelPackState unpack = pContext->client.unpack;
         GLint cbData = crPixelSize(GL_RGBA, GL_UNSIGNED_BYTE) * pVP->viewportH * pVP->viewportW;
         void *pData = crAlloc(cbData);
 
         if (!pData)
         {
+            pContext->pImage = NULL;
             return VERR_NO_MEMORY;
         }
 
         rc = SSMR3GetMem(pSSM, pData, cbData);
         AssertRCReturn(rc, rc);
 
-        diff_api.PixelStorei(GL_UNPACK_SKIP_ROWS, 0);
-        diff_api.PixelStorei(GL_UNPACK_SKIP_PIXELS, 0);
-        diff_api.PixelStorei(GL_UNPACK_ALIGNMENT, 1);
-        diff_api.PixelStorei(GL_UNPACK_ROW_LENGTH, 0);
-        diff_api.PixelStorei(GL_UNPACK_IMAGE_HEIGHT, 0);
-        diff_api.PixelStorei(GL_UNPACK_SKIP_IMAGES, 0);
-        diff_api.PixelStorei(GL_UNPACK_SWAP_BYTES, 0);
-        diff_api.PixelStorei(GL_UNPACK_LSB_FIRST, 0);
+        pContext->pImage = pData;
+    }
 
-        diff_api.WindowPos2iARB(0, 0);
-        diff_api.DrawPixels(pVP->viewportW, pVP->viewportH, GL_RGBA, GL_UNSIGNED_BYTE, pData);
 
-        diff_api.PixelStorei(GL_UNPACK_SKIP_ROWS, unpack.skipRows);
-        diff_api.PixelStorei(GL_UNPACK_SKIP_PIXELS, unpack.skipPixels);
-        diff_api.PixelStorei(GL_UNPACK_ALIGNMENT, unpack.alignment);
-        diff_api.PixelStorei(GL_UNPACK_ROW_LENGTH, unpack.rowLength);
-        diff_api.PixelStorei(GL_UNPACK_IMAGE_HEIGHT, unpack.imageHeight);
-        diff_api.PixelStorei(GL_UNPACK_SKIP_IMAGES, unpack.skipImages);
-        diff_api.PixelStorei(GL_UNPACK_SWAP_BYTES, unpack.swapBytes);
-        diff_api.PixelStorei(GL_UNPACK_LSB_FIRST, unpack.psLSBFirst);
+    /*Mark all as dirty to make sure we'd restore correct context state*/
+    {
+        CRStateBits *pBits = GetCurrentBits();
 
-        crFree(pData);
+        FILLDIRTY(pBits->attrib.dirty);
+
+        FILLDIRTY(pBits->buffer.dirty);
+        FILLDIRTY(pBits->buffer.enable);
+        FILLDIRTY(pBits->buffer.alphaFunc);
+        FILLDIRTY(pBits->buffer.depthFunc);
+        FILLDIRTY(pBits->buffer.blendFunc);
+        FILLDIRTY(pBits->buffer.logicOp);
+        FILLDIRTY(pBits->buffer.indexLogicOp);
+        FILLDIRTY(pBits->buffer.drawBuffer);
+        FILLDIRTY(pBits->buffer.readBuffer);
+        FILLDIRTY(pBits->buffer.indexMask);
+        FILLDIRTY(pBits->buffer.colorWriteMask);
+        FILLDIRTY(pBits->buffer.clearColor);
+        FILLDIRTY(pBits->buffer.clearIndex);
+        FILLDIRTY(pBits->buffer.clearDepth);
+        FILLDIRTY(pBits->buffer.clearAccum);
+        FILLDIRTY(pBits->buffer.depthMask);
+#ifdef CR_EXT_blend_color
+        FILLDIRTY(pBits->buffer.blendColor);
+#endif
+#if defined(CR_EXT_blend_minmax) || defined(CR_EXT_blend_subtract) || defined(CR_EXT_blend_logic_op)
+        FILLDIRTY(pBits->buffer.blendEquation);
+#endif
+#if defined(CR_EXT_blend_func_separate)
+        FILLDIRTY(pBits->buffer.blendFuncSeparate);
+#endif
+
+#ifdef CR_ARB_vertex_buffer_object
+        FILLDIRTY(pBits->bufferobject.dirty);
+        FILLDIRTY(pBits->bufferobject.arrayBinding);
+        FILLDIRTY(pBits->bufferobject.elementsBinding);
+# ifdef CR_ARB_pixel_buffer_object
+        FILLDIRTY(pBits->bufferobject.packBinding);
+        FILLDIRTY(pBits->bufferobject.unpackBinding);
+# endif
+#endif
+
+        FILLDIRTY(pBits->client.dirty);
+        FILLDIRTY(pBits->client.pack);
+        FILLDIRTY(pBits->client.unpack);
+        FILLDIRTY(pBits->client.enableClientState);
+        FILLDIRTY(pBits->client.clientPointer);
+        FILLDIRTY(pBits->client.v);
+        FILLDIRTY(pBits->client.n);
+        FILLDIRTY(pBits->client.c);
+        FILLDIRTY(pBits->client.i);
+        FILLDIRTY(pBits->client.e);
+        FILLDIRTY(pBits->client.s);
+        FILLDIRTY(pBits->client.f);
+        for (i=0; i<CR_MAX_TEXTURE_UNITS; i++)
+        {
+            FILLDIRTY(pBits->client.t[i]);
+        }
+#ifdef CR_NV_vertex_program
+        for (i=0; i<CR_MAX_VERTEX_ATTRIBS; i++)
+        {
+            FILLDIRTY(pBits->client.a[i]);
+        }
+#endif
+
+        FILLDIRTY(pBits->current.dirty);
+        for (i=0; i<CR_MAX_VERTEX_ATTRIBS; i++)
+        {
+            FILLDIRTY(pBits->current.vertexAttrib[i]);
+        }
+        FILLDIRTY(pBits->current.edgeFlag);
+        FILLDIRTY(pBits->current.colorIndex);
+        FILLDIRTY(pBits->current.rasterPos);
+
+
+        FILLDIRTY(pBits->eval.dirty);
+        for (i=0; i<GLEVAL_TOT; i++)
+        {
+            FILLDIRTY(pBits->eval.eval1D[i]);
+            FILLDIRTY(pBits->eval.eval2D[i]);
+            FILLDIRTY(pBits->eval.enable1D[i]);
+            FILLDIRTY(pBits->eval.enable2D[i]);
+        }
+        FILLDIRTY(pBits->eval.enable);
+        FILLDIRTY(pBits->eval.grid1D);
+        FILLDIRTY(pBits->eval.grid2D);
+#ifdef CR_NV_vertex_program
+        /*@todo Those seems to be unused?
+        FILLDIRTY(pBits->eval.enableAttrib1D);
+        FILLDIRTY(pBits->eval.enableAttrib2D);
+        */
+#endif
+
+        FILLDIRTY(pBits->feedback.dirty);
+        FILLDIRTY(pBits->selection.dirty);
+
+        FILLDIRTY(pBits->fog.dirty);
+        FILLDIRTY(pBits->fog.color);
+        FILLDIRTY(pBits->fog.index);
+        FILLDIRTY(pBits->fog.density);
+        FILLDIRTY(pBits->fog.start);
+        FILLDIRTY(pBits->fog.end);
+        FILLDIRTY(pBits->fog.mode);
+        FILLDIRTY(pBits->fog.enable);
+#ifdef CR_NV_fog_distance
+        FILLDIRTY(pBits->fog.fogDistanceMode);
+#endif
+#ifdef CR_EXT_fog_coord
+        FILLDIRTY(pBits->fog.fogCoordinateSource);
+#endif
+
+        FILLDIRTY(pBits->hint.dirty);
+        FILLDIRTY(pBits->hint.perspectiveCorrection);
+        FILLDIRTY(pBits->hint.pointSmooth);
+        FILLDIRTY(pBits->hint.lineSmooth);
+        FILLDIRTY(pBits->hint.polygonSmooth);
+        FILLDIRTY(pBits->hint.fog);
+#ifdef CR_EXT_clip_volume_hint
+        FILLDIRTY(pBits->hint.clipVolumeClipping);
+
+#endif
+#ifdef CR_ARB_texture_compression
+        FILLDIRTY(pBits->hint.textureCompression);
+#endif
+#ifdef CR_SGIS_generate_mipmap
+        FILLDIRTY(pBits->hint.generateMipmap);
+#endif
+
+        FILLDIRTY(pBits->lighting.dirty);
+        FILLDIRTY(pBits->lighting.shadeModel);
+        FILLDIRTY(pBits->lighting.colorMaterial);
+        FILLDIRTY(pBits->lighting.lightModel);
+        FILLDIRTY(pBits->lighting.material);
+        FILLDIRTY(pBits->lighting.enable);
+        for (i=0; i<CR_MAX_LIGHTS; ++i)
+        {
+            FILLDIRTY(pBits->lighting.light[i].dirty);
+            FILLDIRTY(pBits->lighting.light[i].enable);
+            FILLDIRTY(pBits->lighting.light[i].ambient);
+            FILLDIRTY(pBits->lighting.light[i].diffuse);
+            FILLDIRTY(pBits->lighting.light[i].specular);
+            FILLDIRTY(pBits->lighting.light[i].position);
+            FILLDIRTY(pBits->lighting.light[i].attenuation);
+            FILLDIRTY(pBits->lighting.light[i].spot);
+        }
+
+        FILLDIRTY(pBits->line.dirty);
+        FILLDIRTY(pBits->line.enable);
+        FILLDIRTY(pBits->line.width);
+        FILLDIRTY(pBits->line.stipple);
+
+        FILLDIRTY(pBits->lists.dirty);
+        FILLDIRTY(pBits->lists.base);
+
+        FILLDIRTY(pBits->multisample.dirty);
+        FILLDIRTY(pBits->multisample.enable);
+        FILLDIRTY(pBits->multisample.sampleAlphaToCoverage);
+        FILLDIRTY(pBits->multisample.sampleAlphaToOne);
+        FILLDIRTY(pBits->multisample.sampleCoverage);
+        FILLDIRTY(pBits->multisample.sampleCoverageValue);
+
+#if CR_ARB_occlusion_query
+        FILLDIRTY(pBits->occlusion.dirty);
+#endif
+
+        FILLDIRTY(pBits->pixel.dirty);
+        FILLDIRTY(pBits->pixel.transfer);
+        FILLDIRTY(pBits->pixel.zoom);
+        FILLDIRTY(pBits->pixel.maps);
+
+        FILLDIRTY(pBits->point.dirty);
+        FILLDIRTY(pBits->point.enableSmooth);
+        FILLDIRTY(pBits->point.size);
+#ifdef CR_ARB_point_parameters
+        FILLDIRTY(pBits->point.minSize);
+        FILLDIRTY(pBits->point.maxSize);
+        FILLDIRTY(pBits->point.fadeThresholdSize);
+        FILLDIRTY(pBits->point.distanceAttenuation);
+#endif
+#ifdef CR_ARB_point_sprite
+        FILLDIRTY(pBits->point.enableSprite);
+        for (i=0; i<CR_MAX_TEXTURE_UNITS; ++i)
+        {
+            FILLDIRTY(pBits->point.coordReplacement[i]);
+        }
+#endif
+
+        FILLDIRTY(pBits->polygon.dirty);
+        FILLDIRTY(pBits->polygon.enable);
+        FILLDIRTY(pBits->polygon.offset);
+        FILLDIRTY(pBits->polygon.mode);
+        FILLDIRTY(pBits->polygon.stipple);
+
+        FILLDIRTY(pBits->program.dirty);
+        FILLDIRTY(pBits->program.vpEnable);
+        FILLDIRTY(pBits->program.fpEnable);
+        FILLDIRTY(pBits->program.vpBinding);
+        FILLDIRTY(pBits->program.fpBinding);
+        for (i=0; i<CR_MAX_VERTEX_ATTRIBS; ++i)
+        {
+            FILLDIRTY(pBits->program.vertexAttribArrayEnable[i]);
+            FILLDIRTY(pBits->program.map1AttribArrayEnable[i]);
+            FILLDIRTY(pBits->program.map2AttribArrayEnable[i]);
+        }
+        for (i=0; i<CR_MAX_VERTEX_PROGRAM_ENV_PARAMS; ++i)
+        {
+            FILLDIRTY(pBits->program.vertexEnvParameter[i]);
+        }
+        for (i=0; i<CR_MAX_FRAGMENT_PROGRAM_ENV_PARAMS; ++i)
+        {
+            FILLDIRTY(pBits->program.fragmentEnvParameter[i]);
+        }
+        FILLDIRTY(pBits->program.vertexEnvParameters);
+        FILLDIRTY(pBits->program.fragmentEnvParameters);
+        for (i=0; i<CR_MAX_VERTEX_PROGRAM_ENV_PARAMS/4; ++i)
+        {
+            FILLDIRTY(pBits->program.trackMatrix[i]);
+        }
+
+        FILLDIRTY(pBits->regcombiner.dirty);
+        FILLDIRTY(pBits->regcombiner.enable);
+        FILLDIRTY(pBits->regcombiner.regCombinerVars);
+        FILLDIRTY(pBits->regcombiner.regCombinerColor0);
+        FILLDIRTY(pBits->regcombiner.regCombinerColor1);
+        for (i=0; i<CR_MAX_GENERAL_COMBINERS; ++i)
+        {
+            FILLDIRTY(pBits->regcombiner.regCombinerStageColor0[i]);
+            FILLDIRTY(pBits->regcombiner.regCombinerStageColor1[i]);
+            FILLDIRTY(pBits->regcombiner.regCombinerInput[i]);
+            FILLDIRTY(pBits->regcombiner.regCombinerOutput[i]);
+        }
+        FILLDIRTY(pBits->regcombiner.regCombinerFinalInput);
+
+        FILLDIRTY(pBits->stencil.dirty);
+        FILLDIRTY(pBits->stencil.enable);
+        FILLDIRTY(pBits->stencil.func);
+        FILLDIRTY(pBits->stencil.op);
+        FILLDIRTY(pBits->stencil.clearValue);
+        FILLDIRTY(pBits->stencil.writeMask);
+
+        FILLDIRTY(pBits->texture.dirty);
+        for (i=0; i<CR_MAX_TEXTURE_UNITS; ++i)
+        {
+            FILLDIRTY(pBits->texture.enable[i]);
+            FILLDIRTY(pBits->texture.current[i]);
+            FILLDIRTY(pBits->texture.objGen[i]);
+            FILLDIRTY(pBits->texture.eyeGen[i]);
+            FILLDIRTY(pBits->texture.genMode[i]);
+            FILLDIRTY(pBits->texture.envBit[i]);
+        }
+
+        FILLDIRTY(pBits->transform.dirty);
+        FILLDIRTY(pBits->transform.matrixMode);
+        FILLDIRTY(pBits->transform.modelviewMatrix);
+        FILLDIRTY(pBits->transform.projectionMatrix);
+        FILLDIRTY(pBits->transform.colorMatrix);
+        FILLDIRTY(pBits->transform.textureMatrix);
+        FILLDIRTY(pBits->transform.programMatrix);
+        FILLDIRTY(pBits->transform.clipPlane);
+        FILLDIRTY(pBits->transform.enable);
+        FILLDIRTY(pBits->transform.base);
+
+        FILLDIRTY(pBits->viewport.dirty);
+        FILLDIRTY(pBits->viewport.v_dims);
+        FILLDIRTY(pBits->viewport.s_dims);
+        FILLDIRTY(pBits->viewport.enable);
+        FILLDIRTY(pBits->viewport.depth);
     }
 
     return VINF_SUCCESS;
