@@ -55,6 +55,7 @@
 #include <VBox/log.h>
 #include <iprt/alloc.h>
 #include <iprt/err.h>
+#include <iprt/asm.h>
 
 #ifdef VBOX_WITH_HARDENING
 # define VBOXDRV_PERM 0600
@@ -91,7 +92,7 @@ static moduledata_t         g_VBoxDrvFreeBSDModule =
 
 /** Declare the module as a pseudo device. */
 DECLARE_MODULE(vboxdrv,     g_VBoxDrvFreeBSDModule, SI_SUB_PSEUDO, SI_ORDER_ANY);
-MODULE_VERSION(vboxdrv, 1); 
+MODULE_VERSION(vboxdrv, 1);
 
 /**
  * The /dev/vboxdrv character device entry points.
@@ -158,8 +159,6 @@ static int VBoxDrvFreeBSDModuleEvent(struct module *pMod, int enmEventType, void
 
 static int VBoxDrvFreeBSDLoad(void)
 {
-    dprintf(("VBoxDrvFreeBSDLoad:\n"));
-
     g_cUsers = 0;
 
     /*
@@ -168,10 +167,12 @@ static int VBoxDrvFreeBSDLoad(void)
     int rc = RTR0Init(0);
     if (RT_SUCCESS(rc))
     {
+        Log(("VBoxDrvFreeBSDLoad:\n"));
+
         /*
          * Initialize the device extension.
          */
-        rc = supdrvInitDevExt(&g_VBoxDrvFreeBSDDevExt);
+        rc = supdrvInitDevExt(&g_VBoxDrvFreeBSDDevExt, sizeof(SUPDRVSESSION));
         if (RT_SUCCESS(rc))
         {
             /*
@@ -181,13 +182,13 @@ static int VBoxDrvFreeBSDLoad(void)
             g_VBoxDrvFreeBSDEHTag = EVENTHANDLER_REGISTER(dev_clone, VBoxDrvFreeBSDClone, 0, 1000);
             if (g_VBoxDrvFreeBSDEHTag)
             {
-                dprintf(("VBoxDrvFreeBSDLoad: returns successfully\n"));
+                Log(("VBoxDrvFreeBSDLoad: returns successfully\n"));
                 return VINF_SUCCESS;
             }
 
             printf("vboxdrv: EVENTHANDLER_REGISTER(dev_clone,,,) failed\n");
             clone_cleanup(&g_pVBoxDrvFreeBSDClones);
-            rc = SUPDRV_ERR_ALREADY_LOADED;
+            rc = VERR_ALREADY_LOADED;
             supdrvDeleteDevExt(&g_VBoxDrvFreeBSDDevExt);
         }
         else
@@ -201,7 +202,7 @@ static int VBoxDrvFreeBSDLoad(void)
 
 static int VBoxDrvFreeBSDUnload(void)
 {
-    dprintf(("VBoxDrvFreeBSDUnload:\n"));
+    Log(("VBoxDrvFreeBSDUnload:\n"));
 
     if (g_cUsers > 0)
         return EBUSY;
@@ -218,7 +219,7 @@ static int VBoxDrvFreeBSDUnload(void)
 
     memset(&g_VBoxDrvFreeBSDDevExt, 0, sizeof(g_VBoxDrvFreeBSDDevExt));
 
-    dprintf(("VBoxDrvFreeBSDUnload: returns\n"));
+    Log(("VBoxDrvFreeBSDUnload: returns\n"));
     return VINF_SUCCESS;
 }
 
@@ -231,7 +232,7 @@ static void VBoxDrvFreeBSDClone(void *pvArg, struct ucred *pCred, char *pszName,
     int iUnit;
     int rc;
 
-    dprintf(("VBoxDrvFreeBSDClone: pszName=%s ppDev=%p\n", pszName, ppDev));
+    Log(("VBoxDrvFreeBSDClone: pszName=%s ppDev=%p\n", pszName, ppDev));
 
     /*
      * One device node per user, si_drv1 points to the session.
@@ -243,14 +244,14 @@ static void VBoxDrvFreeBSDClone(void *pvArg, struct ucred *pCred, char *pszName,
         return;
     if (iUnit >= 256 || iUnit < 0)
     {
-        dprintf(("VBoxDrvFreeBSDClone: iUnit=%d >= 256 - rejected\n", iUnit));
+        Log(("VBoxDrvFreeBSDClone: iUnit=%d >= 256 - rejected\n", iUnit));
         return;
     }
 
-    dprintf(("VBoxDrvFreeBSDClone: pszName=%s iUnit=%d\n", pszName, iUnit));
+    Log(("VBoxDrvFreeBSDClone: pszName=%s iUnit=%d\n", pszName, iUnit));
 
     rc = clone_create(&g_pVBoxDrvFreeBSDClones, &g_VBoxDrvFreeBSDChrDevSW, &iUnit, ppDev, 0);
-    dprintf(("VBoxDrvFreeBSDClone: clone_create -> %d; iUnit=%d\n", rc, iUnit));
+    Log(("VBoxDrvFreeBSDClone: clone_create -> %d; iUnit=%d\n", rc, iUnit));
     if (rc)
     {
 #if __FreeBSD_version > 800061
@@ -262,16 +263,16 @@ static void VBoxDrvFreeBSDClone(void *pvArg, struct ucred *pCred, char *pszName,
         {
             dev_ref(*ppDev);
             (*ppDev)->si_flags |= SI_CHEAPCLONE;
-            dprintf(("VBoxDrvFreeBSDClone: Created *ppDev=%p iUnit=%d si_drv1=%p si_drv2=%p\n",
-                     *ppDev, iUnit, (*ppDev)->si_drv1, (*ppDev)->si_drv2));
+            Log(("VBoxDrvFreeBSDClone: Created *ppDev=%p iUnit=%d si_drv1=%p si_drv2=%p\n",
+                 *ppDev, iUnit, (*ppDev)->si_drv1, (*ppDev)->si_drv2));
             (*ppDev)->si_drv1 = (*ppDev)->si_drv2 = NULL;
         }
         else
             OSDBGPRINT(("VBoxDrvFreeBSDClone: make_dev iUnit=%d failed\n", iUnit));
     }
     else
-        dprintf(("VBoxDrvFreeBSDClone: Existing *ppDev=%p iUnit=%d si_drv1=%p si_drv2=%p\n",
-                 *ppDev, iUnit, (*ppDev)->si_drv1, (*ppDev)->si_drv2));
+        Log(("VBoxDrvFreeBSDClone: Existing *ppDev=%p iUnit=%d si_drv1=%p si_drv2=%p\n",
+             *ppDev, iUnit, (*ppDev)->si_drv1, (*ppDev)->si_drv2));
 }
 
 
@@ -296,9 +297,9 @@ static int VBoxDrvFreeBSDOpen(struct cdev *pDev, int fOpen, struct thread *pTd,
     int rc;
 
 #if __FreeBSD_version < 800062
-    dprintf(("VBoxDrvFreeBSDOpen: fOpen=%#x iUnit=%d\n", fOpen, minor2unit(minor(pDev))));
+    Log(("VBoxDrvFreeBSDOpen: fOpen=%#x iUnit=%d\n", fOpen, minor2unit(minor(pDev))));
 #else
-    dprintf(("VBoxDrvFreeBSDOpen: fOpen=%#x iUnit=%d\n", fOpen, minor(dev2udev(pDev))));
+    Log(("VBoxDrvFreeBSDOpen: fOpen=%#x iUnit=%d\n", fOpen, minor(dev2udev(pDev))));
 #endif
 
     /*
@@ -306,7 +307,7 @@ static int VBoxDrvFreeBSDOpen(struct cdev *pDev, int fOpen, struct thread *pTd,
      */
     if (fOpen != (FREAD|FWRITE /*=O_RDWR*/))
     {
-        dprintf(("VBoxDrvFreeBSDOpen: fOpen=%#x expected %#x\n", fOpen, O_RDWR));
+        Log(("VBoxDrvFreeBSDOpen: fOpen=%#x expected %#x\n", fOpen, O_RDWR));
         return EINVAL;
     }
 
@@ -352,9 +353,9 @@ static int VBoxDrvFreeBSDClose(struct cdev *pDev, int fFile, int DevType, struct
 {
     PSUPDRVSESSION pSession = (PSUPDRVSESSION)pDev->si_drv1;
 #if __FreeBSD_version < 800062
-    dprintf(("VBoxDrvFreeBSDClose: fFile=%#x iUnit=%d pSession=%p\n", fFile, minor2unit(minor(pDev)), pSession));
+    Log(("VBoxDrvFreeBSDClose: fFile=%#x iUnit=%d pSession=%p\n", fFile, minor2unit(minor(pDev)), pSession));
 #else
-    dprintf(("VBoxDrvFreeBSDClose: fFile=%#x iUnit=%d pSession=%p\n", fFile, minor(dev2udev(pDev)), pSession));
+    Log(("VBoxDrvFreeBSDClose: fFile=%#x iUnit=%d pSession=%p\n", fFile, minor(dev2udev(pDev)), pSession));
 #endif
 
     /*
@@ -495,7 +496,7 @@ static int VBoxDrvFreeBSDIOCtlSlow(PSUPDRVSESSION pSession, u_long ulCmd, caddr_
     }
     else
     {
-        dprintf(("VBoxDrvFreeBSDIOCtlSlow: huh? cbReq=%#x ulCmd=%#lx\n", cbReq, ulCmd));
+        Log(("VBoxDrvFreeBSDIOCtlSlow: huh? cbReq=%#x ulCmd=%#lx\n", cbReq, ulCmd));
         return EINVAL;
     }
 
@@ -520,7 +521,7 @@ static int VBoxDrvFreeBSDIOCtlSlow(PSUPDRVSESSION pSession, u_long ulCmd, caddr_
             if (RT_UNLIKELY(rc))
                 OSDBGPRINT(("VBoxDrvFreeBSDIOCtlSlow: copyout(%p,%p,%#x) -> %d; uCmd=%#lx!\n", pHdr, pvUser, cbOut, rc, ulCmd));
 
-            dprintf(("VBoxDrvFreeBSDIOCtlSlow: returns %d / %d ulCmd=%lx\n", 0, pHdr->rc, ulCmd));
+            Log(("VBoxDrvFreeBSDIOCtlSlow: returns %d / %d ulCmd=%lx\n", 0, pHdr->rc, ulCmd));
 
             /* cleanup */
             RTMemTmpFree(pHdr);
@@ -534,7 +535,7 @@ static int VBoxDrvFreeBSDIOCtlSlow(PSUPDRVSESSION pSession, u_long ulCmd, caddr_
         if (pvUser)
             RTMemTmpFree(pHdr);
 
-        dprintf(("VBoxDrvFreeBSDIOCtlSlow: ulCmd=%lx pData=%p failed, rc=%d\n", ulCmd, pvData, rc));
+        Log(("VBoxDrvFreeBSDIOCtlSlow: ulCmd=%lx pData=%p failed, rc=%d\n", ulCmd, pvData, rc));
         rc = EINVAL;
     }
 
@@ -600,6 +601,33 @@ bool VBOXCALL  supdrvOSGetForcedAsyncTscMode(PSUPDRVDEVEXT pDevExt)
 }
 
 
+int  VBOXCALL   supdrvOSLdrOpen(PSUPDRVDEVEXT pDevExt, PSUPDRVLDRIMAGE pImage, const char *pszFilename)
+{
+    NOREF(pDevExt); NOREF(pImage); NOREF(pszFilename);
+    return VERR_NOT_SUPPORTED;
+}
+
+
+int  VBOXCALL   supdrvOSLdrValidatePointer(PSUPDRVDEVEXT pDevExt, PSUPDRVLDRIMAGE pImage, void *pv, const uint8_t *pbImageBits)
+{
+    NOREF(pDevExt); NOREF(pImage); NOREF(pv); NOREF(pbImageBits);
+    return VERR_NOT_SUPPORTED;
+}
+
+
+int  VBOXCALL   supdrvOSLdrLoad(PSUPDRVDEVEXT pDevExt, PSUPDRVLDRIMAGE pImage, const uint8_t *pbImageBits)
+{
+    NOREF(pDevExt); NOREF(pImage); NOREF(pbImageBits);
+    return VERR_NOT_SUPPORTED;
+}
+
+
+void VBOXCALL   supdrvOSLdrUnload(PSUPDRVDEVEXT pDevExt, PSUPDRVLDRIMAGE pImage)
+{
+    NOREF(pDevExt); NOREF(pImage);
+}
+
+
 SUPR0DECL(int) SUPR0Printf(const char *pszFormat, ...)
 {
     va_list va;
