@@ -4,7 +4,7 @@
  */
 
 /*
- * Copyright (C) 2008 Sun Microsystems, Inc.
+ * Copyright (C) 2008 Oracle Corporation
  *
  * This file is part of VirtualBox Open Source Edition (OSE), as
  * available from http://www.virtualbox.org. This file is free software;
@@ -13,10 +13,6 @@
  * Foundation, in version 2 as it comes in the "COPYING" file of the
  * VirtualBox OSE distribution. VirtualBox OSE is distributed in the
  * hope that it will be useful, but WITHOUT ANY WARRANTY of any kind.
- *
- * Please contact Sun Microsystems, Inc., 4150 Network Circle, Santa
- * Clara, CA 95054 USA or visit http://www.sun.com if you need
- * additional information or have any questions.
  */
 
 #include <vbox/WinNetConfig.h>
@@ -36,14 +32,13 @@ static VOID winNetCfgLogger (LPCWSTR szString)
 }
 
 /** Wrapper aroung GetfullPathNameW that will try an alternative INF location.
- *  
- * The default location is the current directory.  If not found there, the 
- * alternative locatoin is the executable directory.  If not found there either, 
- * the first alternative is present to the caller. 
- */ 
+ *
+ * The default location is the current directory.  If not found there, the
+ * alternative locatoin is the executable directory.  If not found there either,
+ * the first alternative is present to the caller.
+ */
 static DWORD MyGetfullPathNameW(LPCWSTR pwszName, size_t cchFull, LPWSTR pwszFull)
 {
-#ifdef DEBUG_bird  /** @todo make this default behavior after 3.1. */
     LPWSTR pwszFilePart;
     DWORD dwSize = GetFullPathNameW(pwszName, cchFull, pwszFull, &pwszFilePart);
     if(dwSize <= 0)
@@ -65,7 +60,7 @@ static DWORD MyGetfullPathNameW(LPCWSTR pwszName, size_t cchFull, LPWSTR pwszFul
                 if(!wsz[cch])
                 {
                     dwSize = GetFullPathNameW(wsz, cchFull, pwszFull, NULL);
-                    if(   dwSize > 0 
+                    if(   dwSize > 0
                        && GetFileAttributesW(pwszFull) != INVALID_FILE_ATTRIBUTES)
                         return dwSize;
                     break;
@@ -76,7 +71,6 @@ static DWORD MyGetfullPathNameW(LPCWSTR pwszName, size_t cchFull, LPWSTR pwszFul
     }
 
     /* fallback */
-#endif
     return GetFullPathNameW(pwszName, cchFull, pwszFull, NULL);
 }
 
