@@ -1,5 +1,5 @@
 /*
- * Copyright (C) 2008 Sun Microsystems, Inc.
+ * Copyright (C) 2008 Oracle Corporation
  *
  * This file is part of VirtualBox Open Source Edition (OSE), as
  * available from http://www.virtualbox.org. This file is free software;
@@ -8,10 +8,6 @@
  * Foundation, in version 2 as it comes in the "COPYING" file of the
  * VirtualBox OSE distribution. VirtualBox OSE is distributed in the
  * hope that it will be useful, but WITHOUT ANY WARRANTY of any kind.
- *
- * Please contact Sun Microsystems, Inc., 4150 Network Circle, Santa
- * Clara, CA 95054 USA or visit http://www.sun.com if you need
- * additional information or have any questions.
  */
 /*
  * Based in part on Microsoft DDK sample code for Sample Notify Object
@@ -768,8 +764,6 @@ STDAPI DllRegisterServer(void)
 STDAPI DllUnregisterServer(void)
 {
     TraceMsg( L"DllUnregisterServer.\n");
-
-    _Module.UnregisterServer();
-
+    _Module.UnregisterServer(TRUE);
     return S_OK;
 }
