@@ -3,7 +3,7 @@
  */
 
 /*
- * Copyright (C) 2006-2007 Sun Microsystems, Inc.
+ * Copyright (C) 2006-2007 Oracle Corporation
  *
  * This file is part of VirtualBox Open Source Edition (OSE), as
  * available from http://www.virtualbox.org. This file is free software;
@@ -12,10 +12,6 @@
  * Foundation, in version 2 as it comes in the "COPYING" file of the
  * VirtualBox OSE distribution. VirtualBox OSE is distributed in the
  * hope that it will be useful, but WITHOUT ANY WARRANTY of any kind.
- *
- * Please contact Sun Microsystems, Inc., 4150 Network Circle, Santa
- * Clara, CA 95054 USA or visit http://www.sun.com if you need
- * additional information or have any questions.
  */
 
 #ifndef ___MAPPINGS_H
@@ -35,7 +31,7 @@ typedef struct
     bool        fWritable;
 } MAPPING, *PMAPPING;
 
-extern MAPPING FolderMapping[SHFL_MAX_MAPPINGS];
+void vbsfMappingInit(void);
 
 bool vbsfMappingQuery(uint32_t iMapping, PMAPPING *pMapping);
 
@@ -53,5 +49,8 @@ PCRTUTF16     vbsfMappingsQueryHostRoot (SHFLROOT root, uint32_t *pcbRoot);
 bool          vbsfIsGuestMappingCaseSensitive (SHFLROOT root);
 bool          vbsfIsHostMappingCaseSensitive (SHFLROOT root);
 
+int vbsfMappingLoaded (const MAPPING *pLoadedMapping, SHFLROOT root);
+MAPPING *vbsfMappingGetByRoot(SHFLROOT root);
+
 #endif /* !___MAPPINGS_H */
 
