@@ -73,20 +73,18 @@ crServerDispatchWindowCreateEx(const char *dpyName, GLint visBits, GLint preload
     if (mural) {
         CRMuralInfo *defaultMural = (CRMuralInfo *) crHashtableSearch(cr_server.muralTable, 0);
         CRASSERT(defaultMural);
-        mural->width = defaultMural->width;
-        mural->height = defaultMural->height;
-        mural->optimizeBucket = 0; /* might get enabled later */
-        mural->numExtents = defaultMural->numExtents;
-        mural->curExtent = 0;
-        crMemcpy(mural->extents, defaultMural->extents,
-                 defaultMural->numExtents * sizeof(CRExtent));
-        mural->underlyingDisplay[0] = 0;
-        mural->underlyingDisplay[1] = 0;
-        mural->underlyingDisplay[2] = dims[0];
-        mural->underlyingDisplay[3] = dims[1];
+        mural->gX = 0;
+        mural->gY = 0;
+        mural->width = dims[0];
+        mural->height = dims[1];
 
         mural->spuWindow = spuWindow;
-        crServerInitializeTiling(mural);
+        mural->screenId = 0;
+        mural->bVisible = GL_FALSE;
+        mural->bUseFBO = GL_FALSE;
+
+        mural->cVisibleRects = 0;
+        mural->pVisibleRects = NULL;
 
         /* generate ID for this new window/mural (special-case for file conns) */
         if (cr_server.curClient && cr_server.curClient->conn->type == CR_FILE)
@@ -122,6 +120,7 @@ void SERVER_DISPATCH_APIENTRY
 crServerDispatchWindowDestroy( GLint window )
 {
     CRMuralInfo *mural;
+    int32_t client;
     int pos;
 
     mural = (CRMuralInfo *) crHashtableSearch(cr_server.muralTable, window);
@@ -130,6 +129,13 @@ crServerDispatchWindowDestroy( GLint window )
          return;
     }
 
+    if (cr_server.currentWindow == window)
+    {
+        cr_server.currentWindow = -1;
+        crServerRedirMuralFBO(mural, GL_FALSE);
+        crServerDeleteMuralFBO(mural);
+    }
+
     crDebug("CRServer: Destroying window %d (spu window %d)", window, mural->spuWindow);
     cr_server.head_spu->dispatch_table.WindowDestroy( mural->spuWindow );
 
@@ -151,8 +157,6 @@ crServerDispatchWindowDestroy( GLint window )
         /*Same as with contexts, some apps destroy it not in a thread where it was created*/
         if (CR_MAX_WINDOWS==pos)
         {
-            int32_t client;
-
             for (client=0; client<cr_server.numClients; ++client)
             {
                 if (cr_server.clients[client]==cr_server.curClient)
@@ -180,19 +184,29 @@ crServerDispatchWindowDestroy( GLint window )
         CRASSERT(pos<CR_MAX_WINDOWS);
     }
 
-    if (cr_server.currentWindow == window)
+    /*Make sure this window isn't active in other clients*/
+    for (client=0; client<cr_server.numClients; ++client)
     {
-        cr_server.currentWindow = -1;
+        if (cr_server.clients[client]->currentMural == mural)
+        {
+            cr_server.clients[client]->currentMural = NULL;
+            cr_server.clients[client]->currentWindow = -1;
+        }
     }
 
     crHashtableDelete(cr_server.pWindowCreateInfoTable, window, crServerCreateInfoDeleteCB);
+
+    if (mural->pVisibleRects)
+    {
+        crFree(mural->pVisibleRects);
+    }
     crHashtableDelete(cr_server.muralTable, window, crFree);
 }
 
 void SERVER_DISPATCH_APIENTRY
 crServerDispatchWindowSize( GLint window, GLint width, GLint height )
 {
-  CRMuralInfo *mural;
+    CRMuralInfo *mural;
 
     /*  crDebug("CRServer: Window %d size %d x %d", window, width, height);*/
     mural = (CRMuralInfo *) crHashtableSearch(cr_server.muralTable, window);
@@ -202,9 +216,10 @@ crServerDispatchWindowSize( GLint window, GLint width, GLint height )
 #endif
          return;
     }
-    mural->underlyingDisplay[2] = width;
-    mural->underlyingDisplay[3] = height;
-    crServerInitializeTiling(mural);
+    mural->width = width;
+    mural->height = height;
+
+    crServerCheckMuralGeometry(mural);
 
     cr_server.head_spu->dispatch_table.WindowSize(mural->spuWindow, width, height);
 }
@@ -213,7 +228,7 @@ crServerDispatchWindowSize( GLint window, GLint width, GLint height )
 void SERVER_DISPATCH_APIENTRY
 crServerDispatchWindowPosition( GLint window, GLint x, GLint y )
 {
-  CRMuralInfo *mural = (CRMuralInfo *) crHashtableSearch(cr_server.muralTable, window);
+    CRMuralInfo *mural = (CRMuralInfo *) crHashtableSearch(cr_server.muralTable, window);
     /*  crDebug("CRServer: Window %d pos %d, %d", window, x, y);*/
     if (!mural) {
 #if EXTRA_WARN
@@ -221,28 +236,40 @@ crServerDispatchWindowPosition( GLint window, GLint x, GLint y )
 #endif
          return;
     }
-    mural->underlyingDisplay[0] = x;
-    mural->underlyingDisplay[1] = y;
+    mural->gX = x;
+    mural->gY = y;
 
-#if EXTRA_WARN /* don't believe this is needed */
-    crServerInitializeTiling(mural);
-#endif
-    cr_server.head_spu->dispatch_table.WindowPosition(mural->spuWindow, x, y);
+    crServerCheckMuralGeometry(mural);
 }
 
 void SERVER_DISPATCH_APIENTRY
 crServerDispatchWindowVisibleRegion( GLint window, GLint cRects, GLint *pRects )
 {
-  CRMuralInfo *mural = (CRMuralInfo *) crHashtableSearch(cr_server.muralTable, window);
+    CRMuralInfo *mural = (CRMuralInfo *) crHashtableSearch(cr_server.muralTable, window);
     if (!mural) {
 #if EXTRA_WARN
          crWarning("CRServer: invalid window %d passed to WindowVisibleRegion()", window);
 #endif
          return;
     }
-#if EXTRA_WARN /* don't believe this is needed */
-    crServerInitializeTiling(mural);
-#endif
+
+    if (mural->pVisibleRects)
+    {
+        crFree(mural->pVisibleRects);
+        mural->pVisibleRects = NULL;
+    }
+
+    mural->cVisibleRects = cRects;
+    if (cRects)
+    {
+        mural->pVisibleRects = (GLint*) crAlloc(4*sizeof(GLint)*cRects);
+        if (!mural->pVisibleRects)
+        {
+            crError("Out of memory in crServerDispatchWindowVisibleRegion");
+        }
+        crMemcpy(mural->pVisibleRects, pRects, 4*sizeof(GLint)*cRects);
+    }
+
     cr_server.head_spu->dispatch_table.WindowVisibleRegion(mural->spuWindow, cRects, pRects);
 }
 
@@ -251,21 +278,27 @@ crServerDispatchWindowVisibleRegion( GLint window, GLint cRects, GLint *pRects )
 void SERVER_DISPATCH_APIENTRY
 crServerDispatchWindowShow( GLint window, GLint state )
 {
-  CRMuralInfo *mural = (CRMuralInfo *) crHashtableSearch(cr_server.muralTable, window);
+    CRMuralInfo *mural = (CRMuralInfo *) crHashtableSearch(cr_server.muralTable, window);
     if (!mural) {
 #if EXTRA_WARN
          crWarning("CRServer: invalid window %d passed to WindowShow()", window);
 #endif
          return;
     }
-    cr_server.head_spu->dispatch_table.WindowShow(mural->spuWindow, state);
+
+    if (!mural->bUseFBO)
+    {
+        cr_server.head_spu->dispatch_table.WindowShow(mural->spuWindow, state);
+    }
+
+    mural->bVisible = state;
 }
 
 
 GLint
 crServerSPUWindowID(GLint serverWindow)
 {
-  CRMuralInfo *mural = (CRMuralInfo *) crHashtableSearch(cr_server.muralTable, serverWindow);
+    CRMuralInfo *mural = (CRMuralInfo *) crHashtableSearch(cr_server.muralTable, serverWindow);
     if (!mural) {
 #if EXTRA_WARN
          crWarning("CRServer: invalid window %d passed to crServerSPUWindowID()",
