@@ -1,10 +1,10 @@
 #!/bin/sh
 # $Id: vboxconfig.sh $
 
-# Sun VirtualBox
+#
 # VirtualBox Configuration Script, Solaris host.
 #
-# Copyright (C) 2009 Oracle Corporation
+# Copyright (C) 2009-2010 Oracle Corporation
 #
 # This file is part of VirtualBox Open Source Edition (OSE), as
 # available from http://www.virtualbox.org. This file is free software;
@@ -28,6 +28,7 @@ DIR_CONF="${BASEDIR}/platform/i86pc/kernel/drv"
 DIR_MOD_32="${BASEDIR}/platform/i86pc/kernel/drv"
 DIR_MOD_64=$DIR_MOD_32/amd64
 
+# Default paths, these will be overridden by 'which' if they don't exist
 BIN_ADDDRV=/usr/sbin/add_drv
 BIN_REMDRV=/usr/sbin/rem_drv
 BIN_MODLOAD=/usr/sbin/modload
@@ -38,6 +39,7 @@ BIN_BOOTADM=/sbin/bootadm
 BIN_SVCADM=/usr/sbin/svcadm
 BIN_SVCCFG=/usr/sbin/svccfg
 BIN_IFCONFIG=/sbin/ifconfig
+BIN_SVCS=/usr/bin/svcs
 BIN_ID=/usr/bin/id
 
 # "vboxdrv" is also used in sed lines here (change those as well if it ever changes)
@@ -95,21 +97,48 @@ errorprint()
     echo 1>&2 "## $1"
 }
 
+helpprint()
+{
+    echo 1>&2 "$1"
+}
 
-# check_bin_path()
+printusage()
+{
+    helpprint "VirtualBox Configuration Script"
+    helpprint "usage: $0 operation [options]"
+    helpprint
+    helpprint "operation must be one of the following:"
+    helpprint "  --postinstall           Perform full post installation procedure"
+    helpprint "  --preremove             Perform full pre remove procedure"
+    helpprint "  --installdrivers        Only install the drivers"
+    helpprint "  --removedrivers         Only remove the drivers"
+    helpprint "  --setupdrivers          Set up drivers, reloads any existing drivers"
+    helpprint
+    helpprint "[options] are one or more of the following:"
+    helpprint "  --silent                Silent mode"
+    helpprint "  --fatal                 Make failures fatal, don't continue"
+    helpprint "  --ips                   An IPS package installation"
+    helpprint "  --altkerndir            Use /usr/kernel/drv as the driver directory"
+}
+
+# find_bin_path()
 # !! failure is always fatal
-check_bin_path()
+find_bin_path()
 {
     if test -z "$1"; then
-        errorprint "missing argument to check_bin_path()"
+        errorprint "missing argument to find_bin_path()"
         exit 1
     fi
-
-    if test ! -x "$1"; then
+    
+    binfilename=`basename $1`
+    binfilepath=`which $binfilename 2> /dev/null`
+    if test -x "$binfilepath"; then
+        echo "$binfilepath"
+        return 0
+    else
         errorprint "$1 missing or is not an executable"
         exit 1
     fi
-    return 0
 }
 
 # find_bins()
@@ -117,22 +146,53 @@ check_bin_path()
 find_bins()
 {
     # Search only for binaries that might be in different locations
-    BIN_IFCONFIG=`which ifconfig 2> /dev/null`
-    BIN_SVCS=`which svcs 2> /dev/null`
-    BIN_ID=`which id 2> /dev/null`
+    if test ! -x "$BIN_ID"; then
+        BIN_ID=`find_bin_path "$BIN_ID"`
+    fi
 
-    check_bin_path "$BIN_ID"
-    check_bin_path "$BIN_ADDDRV"
-    check_bin_path "$BIN_REMDRV"
-    check_bin_path "$BIN_MODLOAD"
-    check_bin_path "$BIN_MODUNLOAD"
-    check_bin_path "$BIN_MODINFO"
-    check_bin_path "$BIN_DEVFSADM"
-    check_bin_path "$BIN_BOOTADM"
-    check_bin_path "$BIN_SVCADM"
-    check_bin_path "$BIN_SVCCFG"
-    check_bin_path "$BIN_SVCS"
-    check_bin_path "$BIN_IFCONFIG"
+    if test ! -x "$BIN_ADDDRV"; then
+        BIN_ADDDRV=`find_bin_path "$BIN_ADDDRV"`
+    fi
+
+    if test ! -x "$BIN_REMDRV"; then
+        BIN_REMDRV=`find_bin_path "$BIN_REMDRV"`
+    fi
+
+    if test ! -x "$BIN_MODLOAD"; then
+        BIN_MODLOAD=`check_bin_path "$BIN_MODLOAD"`
+    fi
+
+    if test ! -x "$BIN_MODUNLOAD"; then
+        BIN_MODUNLOAD=`find_bin_path "$BIN_MODUNLOAD"`
+    fi
+
+    if test ! -x "$BIN_MODINFO"; then
+        BIN_MODINFO=`find_bin_path "$BIN_MODINFO"`
+    fi
+
+    if test ! -x "$BIN_DEVFSADM"; then
+        BIN_DEVFSADM=`find_bin_path "$BIN_DEVFSADM"`
+    fi
+
+    if test ! -x "$BIN_BOOTADM"; then
+        BIN_BOOTADM=`find_bin_path "$BIN_BOOTADM"`
+    fi
+
+    if test ! -x "$BIN_SVCADM"; then
+        BIN_SVCADM=`find_bin_path "$BIN_SVCADM"`
+    fi
+
+    if test ! -x "$BIN_SVCCFG"; then
+        BIN_SVCCFG=`find_bin_path "$BIN_SVCCFG"`
+    fi
+
+    if test ! -x "$BIN_SVCS"; then
+        BIN_SVCS=`find_bin_path "$BIN_SVCS"`
+    fi
+
+    if test ! -x "$BIN_IFCONFIG"; then
+        BIN_IFCONFIG=`find_bin_path "$BIN_IFCONFIG"`
+    fi
 }
 
 # check_root()
@@ -785,13 +845,13 @@ preremove()
 
 
 # And it begins...
+find_bins
 check_root
 check_isa
 check_zone
-find_bins
 get_sysinfo
 
-if test "x${BASEDIR}" != "x/"; then
+if test "x${BASEDIR:=/}" != "x/"; then
     BASEDIR_OPT="-b ${BASEDIR}"
     REMOTEINST=1
 fi
@@ -816,6 +876,10 @@ do
             # Use alternate kernel driver config folder (dev only)
             DIR_CONF="/usr/kernel/drv"
             ;;
+        --help)
+            printusage
+            exit 1
+            ;;
         *)
             break
             ;;
@@ -843,7 +907,7 @@ case "$drvop" in
     install_drivers
     ;;
 *)
-    errorprint "Invalid operation $drvop"
+    printusage
     exit 1
 esac
 
