@@ -4,7 +4,7 @@
  */
 
 /*
- * Copyright (C) 2006-2007 Sun Microsystems, Inc.
+ * Copyright (C) 2006-2007 Oracle Corporation
  *
  * This file is part of VirtualBox Open Source Edition (OSE), as
  * available from http://www.virtualbox.org. This file is free software;
@@ -13,20 +13,38 @@
  * Foundation, in version 2 as it comes in the "COPYING" file of the
  * VirtualBox OSE distribution. VirtualBox OSE is distributed in the
  * hope that it will be useful, but WITHOUT ANY WARRANTY of any kind.
- *
- * Please contact Sun Microsystems, Inc., 4150 Network Circle, Santa
- * Clara, CA 95054 USA or visit http://www.sun.com if you need
- * additional information or have any questions.
  */
 
 #include "BIOSSettingsImpl.h"
 #include "MachineImpl.h"
-#include "Logging.h"
 #include "GuestOSTypeImpl.h"
 
-#include <iprt/cpputils.h>
+#include <iprt/cpp/utils.h>
 #include <VBox/settings.h>
 
+#include "AutoStateDep.h"
+#include "AutoCaller.h"
+#include "Logging.h"
+
+////////////////////////////////////////////////////////////////////////////////
+//
+// BIOSSettings private data definition
+//
+////////////////////////////////////////////////////////////////////////////////
+
+struct BIOSSettings::Data
+{
+    Data()
+        : pMachine(NULL)
+    { }
+
+    Machine * const             pMachine;
+    ComObjPtr<BIOSSettings>     pPeer;
+
+    // use the XML settings structure in the members for simplicity
+    Backupable<settings::BIOSSettings> bd;
+};
+
 // constructor / destructor
 /////////////////////////////////////////////////////////////////////////////
 
@@ -48,21 +66,23 @@ void BIOSSettings::FinalRelease()
  *
  * @returns COM result indicator
  */
-HRESULT BIOSSettings::init (Machine *aParent)
+HRESULT BIOSSettings::init(Machine *aParent)
 {
     LogFlowThisFuncEnter();
     LogFlowThisFunc(("aParent: %p\n", aParent));
 
-    ComAssertRet (aParent, E_INVALIDARG);
+    ComAssertRet(aParent, E_INVALIDARG);
 
     /* Enclose the state transition NotReady->InInit->Ready */
     AutoInitSpan autoInitSpan(this);
     AssertReturn(autoInitSpan.isOk(), E_FAIL);
 
-    /* share the parent weakly */
-    unconst(mParent) = aParent;
+    m = new Data();
 
-    mData.allocate();
+    /* share the parent weakly */
+    unconst(m->pMachine) = aParent;
+
+    m->bd.allocate();
 
     autoInitSpan.setSucceeded();
 
@@ -78,22 +98,24 @@ HRESULT BIOSSettings::init (Machine *aParent)
  *  @note This object must be destroyed before the original object
  *  it shares data with is destroyed.
  */
-HRESULT BIOSSettings::init (Machine *aParent, BIOSSettings *that)
+HRESULT BIOSSettings::init(Machine *aParent, BIOSSettings *that)
 {
     LogFlowThisFuncEnter();
     LogFlowThisFunc(("aParent: %p, that: %p\n", aParent, that));
 
-    ComAssertRet (aParent && that, E_INVALIDARG);
+    ComAssertRet(aParent && that, E_INVALIDARG);
 
     /* Enclose the state transition NotReady->InInit->Ready */
     AutoInitSpan autoInitSpan(this);
     AssertReturn(autoInitSpan.isOk(), E_FAIL);
 
-    mParent = aParent;
-    mPeer = that;
+    m = new Data();
 
-    AutoWriteLock thatlock (that);
-    mData.share (that->mData);
+    unconst(m->pMachine) = aParent;
+    m->pPeer = that;
+
+    AutoWriteLock thatlock(that COMMA_LOCKVAL_SRC_POS);
+    m->bd.share(that->m->bd);
 
     autoInitSpan.setSucceeded();
 
@@ -106,22 +128,24 @@ HRESULT BIOSSettings::init (Machine *aParent, BIOSSettings *that)
  *  (a kind of copy constructor). This object makes a private copy of data
  *  of the original object passed as an argument.
  */
-HRESULT BIOSSettings::initCopy (Machine *aParent, BIOSSettings *that)
+HRESULT BIOSSettings::initCopy(Machine *aParent, BIOSSettings *that)
 {
     LogFlowThisFuncEnter();
     LogFlowThisFunc(("aParent: %p, that: %p\n", aParent, that));
 
-    ComAssertRet (aParent && that, E_INVALIDARG);
+    ComAssertRet(aParent && that, E_INVALIDARG);
 
     /* Enclose the state transition NotReady->InInit->Ready */
     AutoInitSpan autoInitSpan(this);
     AssertReturn(autoInitSpan.isOk(), E_FAIL);
 
-    mParent = aParent;
+    m = new Data();
+
+    unconst(m->pMachine) = aParent;
     // mPeer is left null
 
-    AutoWriteLock thatlock (that);
-    mData.attachCopy (that->mData);
+    AutoWriteLock thatlock(that COMMA_LOCKVAL_SRC_POS);
+    m->bd.attachCopy(that->m->bd);
 
     autoInitSpan.setSucceeded();
 
@@ -142,10 +166,13 @@ void BIOSSettings::uninit()
     if (autoUninitSpan.uninitDone())
         return;
 
-    mData.free();
+    m->bd.free();
 
-    mPeer.setNull();
-    mParent.setNull();
+    unconst(m->pPeer) = NULL;
+    unconst(m->pMachine) = NULL;
+
+    delete m;
+    m = NULL;
 
     LogFlowThisFuncLeave();
 }
@@ -159,11 +186,11 @@ STDMETHODIMP BIOSSettings::COMGETTER(LogoFadeIn)(BOOL *enabled)
         return E_POINTER;
 
     AutoCaller autoCaller(this);
-    CheckComRCReturnRC(autoCaller.rc());
+    if (FAILED(autoCaller.rc())) return autoCaller.rc();
 
-    AutoReadLock alock(this);
+    AutoReadLock alock(this COMMA_LOCKVAL_SRC_POS);
 
-    *enabled = mData->mLogoFadeIn;
+    *enabled = m->bd->fLogoFadeIn;
 
     return S_OK;
 }
@@ -171,16 +198,20 @@ STDMETHODIMP BIOSSettings::COMGETTER(LogoFadeIn)(BOOL *enabled)
 STDMETHODIMP BIOSSettings::COMSETTER(LogoFadeIn)(BOOL enable)
 {
     AutoCaller autoCaller(this);
-    CheckComRCReturnRC(autoCaller.rc());
+    if (FAILED(autoCaller.rc())) return autoCaller.rc();
 
     /* the machine needs to be mutable */
-    Machine::AutoMutableStateDependency adep (mParent);
-    CheckComRCReturnRC(adep.rc());
+    AutoMutableStateDependency adep(m->pMachine);
+    if (FAILED(adep.rc())) return adep.rc();
 
-    AutoWriteLock alock(this);
+    AutoWriteLock alock(this COMMA_LOCKVAL_SRC_POS);
 
-    mData.backup();
-    mData->mLogoFadeIn = enable;
+    m->bd.backup();
+    m->bd->fLogoFadeIn = !!enable;
+
+    alock.release();
+    AutoWriteLock mlock(m->pMachine COMMA_LOCKVAL_SRC_POS);  // mParent is const, needs no locking
+    m->pMachine->setModified(Machine::IsModified_BIOS);
 
     return S_OK;
 }
@@ -191,11 +222,11 @@ STDMETHODIMP BIOSSettings::COMGETTER(LogoFadeOut)(BOOL *enabled)
         return E_POINTER;
 
     AutoCaller autoCaller(this);
-    CheckComRCReturnRC(autoCaller.rc());
+    if (FAILED(autoCaller.rc())) return autoCaller.rc();
 
-    AutoReadLock alock(this);
+    AutoReadLock alock(this COMMA_LOCKVAL_SRC_POS);
 
-    *enabled = mData->mLogoFadeOut;
+    *enabled = m->bd->fLogoFadeOut;
 
     return S_OK;
 }
@@ -203,16 +234,20 @@ STDMETHODIMP BIOSSettings::COMGETTER(LogoFadeOut)(BOOL *enabled)
 STDMETHODIMP BIOSSettings::COMSETTER(LogoFadeOut)(BOOL enable)
 {
     AutoCaller autoCaller(this);
-    CheckComRCReturnRC(autoCaller.rc());
+    if (FAILED(autoCaller.rc())) return autoCaller.rc();
 
     /* the machine needs to be mutable */
-    Machine::AutoMutableStateDependency adep (mParent);
-    CheckComRCReturnRC(adep.rc());
+    AutoMutableStateDependency adep(m->pMachine);
+    if (FAILED(adep.rc())) return adep.rc();
 
-    AutoWriteLock alock(this);
+    AutoWriteLock alock(this COMMA_LOCKVAL_SRC_POS);
 
-    mData.backup();
-    mData->mLogoFadeOut = enable;
+    m->bd.backup();
+    m->bd->fLogoFadeOut = !!enable;
+
+    alock.release();
+    AutoWriteLock mlock(m->pMachine COMMA_LOCKVAL_SRC_POS);  // mParent is const, needs no locking
+    m->pMachine->setModified(Machine::IsModified_BIOS);
 
     return S_OK;
 }
@@ -223,11 +258,11 @@ STDMETHODIMP BIOSSettings::COMGETTER(LogoDisplayTime)(ULONG *displayTime)
         return E_POINTER;
 
     AutoCaller autoCaller(this);
-    CheckComRCReturnRC(autoCaller.rc());
+    if (FAILED(autoCaller.rc())) return autoCaller.rc();
 
-    AutoReadLock alock(this);
+    AutoReadLock alock(this COMMA_LOCKVAL_SRC_POS);
 
-    *displayTime = mData->mLogoDisplayTime;
+    *displayTime = m->bd->ulLogoDisplayTime;
 
     return S_OK;
 }
@@ -235,16 +270,20 @@ STDMETHODIMP BIOSSettings::COMGETTER(LogoDisplayTime)(ULONG *displayTime)
 STDMETHODIMP BIOSSettings::COMSETTER(LogoDisplayTime)(ULONG displayTime)
 {
     AutoCaller autoCaller(this);
-    CheckComRCReturnRC(autoCaller.rc());
+    if (FAILED(autoCaller.rc())) return autoCaller.rc();
 
     /* the machine needs to be mutable */
-    Machine::AutoMutableStateDependency adep (mParent);
-    CheckComRCReturnRC(adep.rc());
+    AutoMutableStateDependency adep(m->pMachine);
+    if (FAILED(adep.rc())) return adep.rc();
 
-    AutoWriteLock alock(this);
+    AutoWriteLock alock(this COMMA_LOCKVAL_SRC_POS);
 
-    mData.backup();
-    mData->mLogoDisplayTime = displayTime;
+    m->bd.backup();
+    m->bd->ulLogoDisplayTime = displayTime;
+
+    alock.release();
+    AutoWriteLock mlock(m->pMachine COMMA_LOCKVAL_SRC_POS);  // mParent is const, needs no locking
+    m->pMachine->setModified(Machine::IsModified_BIOS);
 
     return S_OK;
 }
@@ -255,11 +294,11 @@ STDMETHODIMP BIOSSettings::COMGETTER(LogoImagePath)(BSTR *imagePath)
         return E_POINTER;
 
     AutoCaller autoCaller(this);
-    CheckComRCReturnRC(autoCaller.rc());
+    if (FAILED(autoCaller.rc())) return autoCaller.rc();
 
-    AutoReadLock alock(this);
+    AutoReadLock alock(this COMMA_LOCKVAL_SRC_POS);
 
-    mData->mLogoImagePath.cloneTo(imagePath);
+    m->bd->strLogoImagePath.cloneTo(imagePath);
     return S_OK;
 }
 
@@ -270,16 +309,20 @@ STDMETHODIMP BIOSSettings::COMSETTER(LogoImagePath)(IN_BSTR imagePath)
         return E_INVALIDARG;
 
     AutoCaller autoCaller(this);
-    CheckComRCReturnRC(autoCaller.rc());
+    if (FAILED(autoCaller.rc())) return autoCaller.rc();
 
     /* the machine needs to be mutable */
-    Machine::AutoMutableStateDependency adep (mParent);
-    CheckComRCReturnRC(adep.rc());
+    AutoMutableStateDependency adep(m->pMachine);
+    if (FAILED(adep.rc())) return adep.rc();
 
-    AutoWriteLock alock(this);
+    AutoWriteLock alock(this COMMA_LOCKVAL_SRC_POS);
 
-    mData.backup();
-    mData->mLogoImagePath = imagePath;
+    m->bd.backup();
+    m->bd->strLogoImagePath = imagePath;
+
+    alock.release();
+    AutoWriteLock mlock(m->pMachine COMMA_LOCKVAL_SRC_POS);  // mParent is const, needs no locking
+    m->pMachine->setModified(Machine::IsModified_BIOS);
 
     return S_OK;
 }
@@ -290,27 +333,31 @@ STDMETHODIMP BIOSSettings::COMGETTER(BootMenuMode)(BIOSBootMenuMode_T *bootMenuM
         return E_POINTER;
 
     AutoCaller autoCaller(this);
-    CheckComRCReturnRC(autoCaller.rc());
+    if (FAILED(autoCaller.rc())) return autoCaller.rc();
 
-    AutoReadLock alock(this);
+    AutoReadLock alock(this COMMA_LOCKVAL_SRC_POS);
 
-    *bootMenuMode = mData->mBootMenuMode;
+    *bootMenuMode = m->bd->biosBootMenuMode;
     return S_OK;
 }
 
 STDMETHODIMP BIOSSettings::COMSETTER(BootMenuMode)(BIOSBootMenuMode_T bootMenuMode)
 {
     AutoCaller autoCaller(this);
-    CheckComRCReturnRC(autoCaller.rc());
+    if (FAILED(autoCaller.rc())) return autoCaller.rc();
 
     /* the machine needs to be mutable */
-    Machine::AutoMutableStateDependency adep (mParent);
-    CheckComRCReturnRC(adep.rc());
+    AutoMutableStateDependency adep(m->pMachine);
+    if (FAILED(adep.rc())) return adep.rc();
 
-    AutoWriteLock alock(this);
+    AutoWriteLock alock(this COMMA_LOCKVAL_SRC_POS);
 
-    mData.backup();
-    mData->mBootMenuMode = bootMenuMode;
+    m->bd.backup();
+    m->bd->biosBootMenuMode = bootMenuMode;
+
+    alock.release();
+    AutoWriteLock mlock(m->pMachine COMMA_LOCKVAL_SRC_POS);  // mParent is const, needs no locking
+    m->pMachine->setModified(Machine::IsModified_BIOS);
 
     return S_OK;
 }
@@ -321,11 +368,11 @@ STDMETHODIMP BIOSSettings::COMGETTER(ACPIEnabled)(BOOL *enabled)
         return E_POINTER;
 
     AutoCaller autoCaller(this);
-    CheckComRCReturnRC(autoCaller.rc());
+    if (FAILED(autoCaller.rc())) return autoCaller.rc();
 
-    AutoReadLock alock(this);
+    AutoReadLock alock(this COMMA_LOCKVAL_SRC_POS);
 
-    *enabled = mData->mACPIEnabled;
+    *enabled = m->bd->fACPIEnabled;
 
     return S_OK;
 }
@@ -333,16 +380,20 @@ STDMETHODIMP BIOSSettings::COMGETTER(ACPIEnabled)(BOOL *enabled)
 STDMETHODIMP BIOSSettings::COMSETTER(ACPIEnabled)(BOOL enable)
 {
     AutoCaller autoCaller(this);
-    CheckComRCReturnRC(autoCaller.rc());
+    if (FAILED(autoCaller.rc())) return autoCaller.rc();
 
     /* the machine needs to be mutable */
-    Machine::AutoMutableStateDependency adep (mParent);
-    CheckComRCReturnRC(adep.rc());
+    AutoMutableStateDependency adep(m->pMachine);
+    if (FAILED(adep.rc())) return adep.rc();
 
-    AutoWriteLock alock(this);
+    AutoWriteLock alock(this COMMA_LOCKVAL_SRC_POS);
 
-    mData.backup();
-    mData->mACPIEnabled = enable;
+    m->bd.backup();
+    m->bd->fACPIEnabled = !!enable;
+
+    alock.release();
+    AutoWriteLock mlock(m->pMachine COMMA_LOCKVAL_SRC_POS);  // mParent is const, needs no locking
+    m->pMachine->setModified(Machine::IsModified_BIOS);
 
     return S_OK;
 }
@@ -353,11 +404,11 @@ STDMETHODIMP BIOSSettings::COMGETTER(IOAPICEnabled)(BOOL *enabled)
         return E_POINTER;
 
     AutoCaller autoCaller(this);
-    CheckComRCReturnRC(autoCaller.rc());
+    if (FAILED(autoCaller.rc())) return autoCaller.rc();
 
-    AutoReadLock alock(this);
+    AutoReadLock alock(this COMMA_LOCKVAL_SRC_POS);
 
-    *enabled = mData->mIOAPICEnabled;
+    *enabled = m->bd->fIOAPICEnabled;
 
     return S_OK;
 }
@@ -365,16 +416,20 @@ STDMETHODIMP BIOSSettings::COMGETTER(IOAPICEnabled)(BOOL *enabled)
 STDMETHODIMP BIOSSettings::COMSETTER(IOAPICEnabled)(BOOL enable)
 {
     AutoCaller autoCaller(this);
-    CheckComRCReturnRC(autoCaller.rc());
+    if (FAILED(autoCaller.rc())) return autoCaller.rc();
 
     /* the machine needs to be mutable */
-    Machine::AutoMutableStateDependency adep (mParent);
-    CheckComRCReturnRC(adep.rc());
+    AutoMutableStateDependency adep(m->pMachine);
+    if (FAILED(adep.rc())) return adep.rc();
 
-    AutoWriteLock alock(this);
+    AutoWriteLock alock(this COMMA_LOCKVAL_SRC_POS);
 
-    mData.backup();
-    mData->mIOAPICEnabled = enable;
+    m->bd.backup();
+    m->bd->fIOAPICEnabled = !!enable;
+
+    alock.release();
+    AutoWriteLock mlock(m->pMachine COMMA_LOCKVAL_SRC_POS);  // mParent is const, needs no locking
+    m->pMachine->setModified(Machine::IsModified_BIOS);
 
     return S_OK;
 }
@@ -385,11 +440,11 @@ STDMETHODIMP BIOSSettings::COMGETTER(PXEDebugEnabled)(BOOL *enabled)
         return E_POINTER;
 
     AutoCaller autoCaller(this);
-    CheckComRCReturnRC(autoCaller.rc());
+    if (FAILED(autoCaller.rc())) return autoCaller.rc();
 
-    AutoReadLock alock(this);
+    AutoReadLock alock(this COMMA_LOCKVAL_SRC_POS);
 
-    *enabled = mData->mPXEDebugEnabled;
+    *enabled = m->bd->fPXEDebugEnabled;
 
     return S_OK;
 }
@@ -397,16 +452,20 @@ STDMETHODIMP BIOSSettings::COMGETTER(PXEDebugEnabled)(BOOL *enabled)
 STDMETHODIMP BIOSSettings::COMSETTER(PXEDebugEnabled)(BOOL enable)
 {
     AutoCaller autoCaller(this);
-    CheckComRCReturnRC(autoCaller.rc());
+    if (FAILED(autoCaller.rc())) return autoCaller.rc();
 
     /* the machine needs to be mutable */
-    Machine::AutoMutableStateDependency adep (mParent);
-    CheckComRCReturnRC(adep.rc());
+    AutoMutableStateDependency adep(m->pMachine);
+    if (FAILED(adep.rc())) return adep.rc();
 
-    AutoWriteLock alock(this);
+    AutoWriteLock alock(this COMMA_LOCKVAL_SRC_POS);
 
-    mData.backup();
-    mData->mPXEDebugEnabled = enable;
+    m->bd.backup();
+    m->bd->fPXEDebugEnabled = !!enable;
+
+    alock.release();
+    AutoWriteLock mlock(m->pMachine COMMA_LOCKVAL_SRC_POS);  // mParent is const, needs no locking
+    m->pMachine->setModified(Machine::IsModified_BIOS);
 
     return S_OK;
 }
@@ -417,11 +476,11 @@ STDMETHODIMP BIOSSettings::COMGETTER(TimeOffset)(LONG64 *offset)
         return E_POINTER;
 
     AutoCaller autoCaller(this);
-    CheckComRCReturnRC(autoCaller.rc());
+    if (FAILED(autoCaller.rc())) return autoCaller.rc();
 
-    AutoReadLock alock(this);
+    AutoReadLock alock(this COMMA_LOCKVAL_SRC_POS);
 
-    *offset = mData->mTimeOffset;
+    *offset = m->bd->llTimeOffset;
 
     return S_OK;
 }
@@ -429,16 +488,20 @@ STDMETHODIMP BIOSSettings::COMGETTER(TimeOffset)(LONG64 *offset)
 STDMETHODIMP BIOSSettings::COMSETTER(TimeOffset)(LONG64 offset)
 {
     AutoCaller autoCaller(this);
-    CheckComRCReturnRC(autoCaller.rc());
+    if (FAILED(autoCaller.rc())) return autoCaller.rc();
 
     /* the machine needs to be mutable */
-    Machine::AutoMutableStateDependency adep (mParent);
-    CheckComRCReturnRC(adep.rc());
+    AutoMutableStateDependency adep(m->pMachine);
+    if (FAILED(adep.rc())) return adep.rc();
 
-    AutoWriteLock alock(this);
+    AutoWriteLock alock(this COMMA_LOCKVAL_SRC_POS);
 
-    mData.backup();
-    mData->mTimeOffset = offset;
+    m->bd.backup();
+    m->bd->llTimeOffset = offset;
+
+    alock.release();
+    AutoWriteLock mlock(m->pMachine COMMA_LOCKVAL_SRC_POS);  // mParent is const, needs no locking
+    m->pMachine->setModified(Machine::IsModified_BIOS);
 
     return S_OK;
 }
@@ -463,20 +526,10 @@ HRESULT BIOSSettings::loadSettings(const settings::BIOSSettings &data)
     AutoCaller autoCaller(this);
     AssertComRCReturnRC(autoCaller.rc());
 
-    AutoWriteLock alock(this);
+    AutoWriteLock alock(this COMMA_LOCKVAL_SRC_POS);
 
-    mData->mACPIEnabled = data.fACPIEnabled;
-    mData->mIOAPICEnabled = data.fIOAPICEnabled;
-
-    mData->mLogoFadeIn = data.fLogoFadeIn;
-    mData->mLogoFadeOut = data.fLogoFadeOut;
-    mData->mLogoDisplayTime = data.ulLogoDisplayTime;
-    mData->mLogoImagePath = data.strLogoImagePath;
-
-    mData->mBootMenuMode = data.biosBootMenuMode;
-
-    mData->mPXEDebugEnabled = data.fPXEDebugEnabled;
-    mData->mTimeOffset = data.llTimeOffset;
+    // simply copy
+    *m->bd.data() = data;
 
     return S_OK;
 }
@@ -493,45 +546,41 @@ HRESULT BIOSSettings::saveSettings(settings::BIOSSettings &data)
     AutoCaller autoCaller(this);
     AssertComRCReturnRC(autoCaller.rc());
 
-    AutoReadLock alock(this);
+    AutoReadLock alock(this COMMA_LOCKVAL_SRC_POS);
 
-    data.fACPIEnabled = !!mData->mACPIEnabled;
-    data.fIOAPICEnabled = !!mData->mIOAPICEnabled;
-
-    data.fLogoFadeIn = !!mData->mLogoFadeIn;
-    data.fLogoFadeOut = !!mData->mLogoFadeOut;
-    data.ulLogoDisplayTime = mData->mLogoDisplayTime;
-    data.strLogoImagePath = mData->mLogoImagePath;
-
-    data.biosBootMenuMode = mData->mBootMenuMode;
-    data.fPXEDebugEnabled = !!mData->mPXEDebugEnabled;
-    data.llTimeOffset = mData->mTimeOffset;
+    data = *m->bd.data();
 
     return S_OK;
 }
 
+void BIOSSettings::rollback()
+{
+    AutoWriteLock alock(this COMMA_LOCKVAL_SRC_POS);
+    m->bd.rollback();
+}
+
 void BIOSSettings::commit()
 {
     /* sanity */
     AutoCaller autoCaller(this);
-    AssertComRCReturnVoid (autoCaller.rc());
+    AssertComRCReturnVoid(autoCaller.rc());
 
     /* sanity too */
-    AutoCaller peerCaller (mPeer);
-    AssertComRCReturnVoid (peerCaller.rc());
+    AutoCaller peerCaller(m->pPeer);
+    AssertComRCReturnVoid(peerCaller.rc());
 
     /* lock both for writing since we modify both (mPeer is "master" so locked
      * first) */
-    AutoMultiWriteLock2 alock (mPeer, this);
+    AutoMultiWriteLock2 alock(m->pPeer, this COMMA_LOCKVAL_SRC_POS);
 
-    if (mData.isBackedUp())
+    if (m->bd.isBackedUp())
     {
-        mData.commit();
-        if (mPeer)
+        m->bd.commit();
+        if (m->pPeer)
         {
             /* attach new data to the peer and reshare it */
-            AutoWriteLock peerlock (mPeer);
-            mPeer->mData.attach (mData);
+            AutoWriteLock peerlock(m->pPeer COMMA_LOCKVAL_SRC_POS);
+            m->pPeer->m->bd.attach(m->bd);
         }
     }
 }
@@ -550,10 +599,11 @@ void BIOSSettings::copyFrom (BIOSSettings *aThat)
 
     /* peer is not modified, lock it for reading (aThat is "master" so locked
      * first) */
-    AutoMultiLock2 alock (aThat->rlock(), this->wlock());
+    AutoReadLock rl(aThat COMMA_LOCKVAL_SRC_POS);
+    AutoWriteLock wl(this COMMA_LOCKVAL_SRC_POS);
 
     /* this will back up current data */
-    mData.assignCopy (aThat->mData);
+    m->bd.assignCopy(aThat->m->bd);
 }
 
 void BIOSSettings::applyDefaults (GuestOSType *aOsType)
@@ -564,9 +614,10 @@ void BIOSSettings::applyDefaults (GuestOSType *aOsType)
     AutoCaller autoCaller(this);
     AssertComRCReturnVoid (autoCaller.rc());
 
-    AutoWriteLock alock(this);
+    AutoWriteLock alock(this COMMA_LOCKVAL_SRC_POS);
 
     /* Initialize default BIOS settings here */
-    mData->mIOAPICEnabled = aOsType->recommendedIOAPIC();
+    m->bd->fIOAPICEnabled = aOsType->recommendedIOAPIC();
 }
+
 /* vi: set tabstop=4 shiftwidth=4 expandtab: */
