@@ -272,6 +272,9 @@ STDMETHODIMP Guest::COMSETTER(MemoryBalloonSize) (ULONG aMemoryBalloonSize)
         mMemoryBalloonSize = aMemoryBalloonSize;
         /* forward the information to the VMM device */
         VMMDev *pVMMDev = mParent->getVMMDev();
+        /* MUST release all locks before calling VMM device as its critsect
+         * has higher lock order than anything in Main. */
+        alock.release();
         if (pVMMDev)
         {
             PPDMIVMMDEVPORT pVMMDevPort = pVMMDev->getVMMDevPort();
@@ -306,6 +309,9 @@ STDMETHODIMP Guest::COMSETTER(StatisticsUpdateInterval)(ULONG aUpdateInterval)
     mStatUpdateInterval = aUpdateInterval;
     /* forward the information to the VMM device */
     VMMDev *pVMMDev = mParent->getVMMDev();
+    /* MUST release all locks before calling VMM device as its critsect
+     * has higher lock order than anything in Main. */
+    alock.release();
     if (pVMMDev)
     {
         PPDMIVMMDEVPORT pVMMDevPort = pVMMDev->getVMMDevPort();
@@ -349,6 +355,10 @@ STDMETHODIMP Guest::InternalGetStatistics(ULONG *aCpuUser, ULONG *aCpuKernel, UL
     *aMemCache   = mCurrentGuestStat[GUESTSTATTYPE_MEMCACHE] * (_4K/_1K);     /* page (4K) -> 1KB units */
     *aPageTotal  = mCurrentGuestStat[GUESTSTATTYPE_PAGETOTAL] * (_4K/_1K);   /* page (4K) -> 1KB units */
 
+    /* MUST release all locks before calling any PGM statistics queries,
+     * as they are executed by EMT and that might deadlock us by VMM device
+     * activity which waits for the Guest object lock. */
+    alock.release();
     Console::SafeVMPtr pVM (mParent);
     if (pVM.isOk())
     {
