@@ -28,8 +28,6 @@
 
 #include <iprt/path.h>
 #include <iprt/dir.h>
-#include <iprt/process.h>
-#include <iprt/ldr.h>
 
 #include <VBox/err.h>
 #include <VBox/param.h>
@@ -116,40 +114,6 @@ HRESULT SystemProperties::init(VirtualBox *aParent)
         }
     }
 
-    /* Driver defaults which are OS specific */
-#if defined(RT_OS_WINDOWS)
-# ifdef VBOX_WITH_WINMM
-    mDefaultAudioDriver = AudioDriverType_WinMM;
-# else /* VBOX_WITH_WINMM */
-    mDefaultAudioDriver = AudioDriverType_DirectSound;
-# endif /* !VBOX_WITH_WINMM */
-#elif defined(RT_OS_SOLARIS)
-    mDefaultAudioDriver = AudioDriverType_SolAudio;
-#elif defined(RT_OS_LINUX)
-# if defined(VBOX_WITH_PULSE)
-    /* Check for the pulse library & that the pulse audio daemon is running. */
-    if (RTProcIsRunningByName("pulseaudio") &&
-        RTLdrIsLoadable("libpulse.so.0"))
-        mDefaultAudioDriver = AudioDriverType_Pulse;
-    else
-# endif /* VBOX_WITH_PULSE */
-# if defined(VBOX_WITH_ALSA)
-        /* Check if we can load the ALSA library */
-        if (RTLdrIsLoadable("libasound.so.2"))
-            mDefaultAudioDriver = AudioDriverType_ALSA;
-        else
-# endif /* VBOX_WITH_ALSA */
-            mDefaultAudioDriver = AudioDriverType_OSS;
-#elif defined(RT_OS_DARWIN)
-    mDefaultAudioDriver = AudioDriverType_CoreAudio;
-#elif defined(RT_OS_OS2)
-    mDefaultAudioDriver = AudioDriverType_MMP;
-#elif defined(RT_OS_FREEBSD)
-    mDefaultAudioDriver = AudioDriverType_OSS;
-#else
-    mDefaultAudioDriver = AudioDriverType_Null;
-#endif
-
     /* Confirm a successful initialization */
     if (SUCCEEDED(rc))
         autoInitSpan.setSucceeded();
@@ -204,7 +168,16 @@ STDMETHODIMP SystemProperties::COMGETTER(MaxGuestRAM)(ULONG *maxRAM)
     ULONG maxRAMArch = maxRAMSys;
 #if HC_ARCH_BITS == 32 && !defined(RT_OS_DARWIN)
 # ifdef RT_OS_WINDOWS
-    maxRAMArch = UINT32_C(1500);
+    SYSTEM_INFO sysInfo;
+    GetSystemInfo(&sysInfo);
+
+    if (sysInfo.lpMaximumApplicationAddress >= (LPVOID)0xC0000000)   /* 3.0 GB */
+        maxRAMArch = UINT32_C(2560);
+    else
+    if (sysInfo.lpMaximumApplicationAddress > (LPVOID)0xA0000000)    /* 2.5 GB */
+        maxRAMArch = UINT32_C(2048);
+    else
+        maxRAMArch = UINT32_C(1500);
 # else
     maxRAMArch = UINT32_C(2560);
 # endif
@@ -810,7 +783,7 @@ STDMETHODIMP SystemProperties::COMGETTER(DefaultAudioDriver)(AudioDriverType_T *
 
     AutoReadLock alock(this COMMA_LOCKVAL_SRC_POS);
 
-    *aAudioDriver = mDefaultAudioDriver;
+    *aAudioDriver = settings::MachineConfigFile::getHostDefaultAudioDriver();
 
     return S_OK;
 }
