@@ -1,5 +1,5 @@
 /*
- * Copyright (C) 2009 Sun Microsystems, Inc.
+ * Copyright (C) 2009-2010 Oracle Corporation
  *
  * This file is part of VirtualBox Open Source Edition (OSE), as
  * available from http://www.virtualbox.org. This file is free software;
@@ -8,10 +8,6 @@
  * Foundation, in version 2 as it comes in the "COPYING" file of the
  * VirtualBox OSE distribution. VirtualBox OSE is distributed in the
  * hope that it will be useful, but WITHOUT ANY WARRANTY of any kind.
- *
- * Please contact Sun Microsystems, Inc., 4150 Network Circle, Santa
- * Clara, CA 95054 USA or visit http://www.sun.com if you need
- * additional information or have any questions.
  */
 
 #ifndef ____H_VIRTUALBOXCALLBACKIMPL
@@ -55,7 +51,7 @@ public:
 
     // public initializers/uninitializers only for internal purposes
     HRESULT init();
-    void uninit (bool aFinalRelease);
+    void uninit(bool aFinalRelease);
 
     // ILocalOwner methods
     STDMETHOD(SetLocalObject)(IUnknown *aLocalObject);
@@ -66,36 +62,37 @@ public:
     STDMETHOD(OnExtraDataCanChange)(IN_BSTR machineId, IN_BSTR key, IN_BSTR value,
                                     BSTR *error, BOOL *changeAllowed);
     STDMETHOD(OnExtraDataChange)(IN_BSTR machineId, IN_BSTR key, IN_BSTR value);
-    STDMETHOD(OnMediumRegistered) (IN_BSTR mediaId, DeviceType_T mediaType,
-                                   BOOL registered);
+    STDMETHOD(OnMediumRegistered)(IN_BSTR mediaId, DeviceType_T mediaType,
+                                  BOOL registered);
     STDMETHOD(OnMachineRegistered)(IN_BSTR machineId, BOOL registered);
     STDMETHOD(OnSessionStateChange)(IN_BSTR machineId, SessionState_T state);
-    STDMETHOD(OnSnapshotTaken) (IN_BSTR aMachineId, IN_BSTR aSnapshotId);
-    STDMETHOD(OnSnapshotDiscarded) (IN_BSTR aMachineId, IN_BSTR aSnapshotId);
-    STDMETHOD(OnSnapshotChange) (IN_BSTR aMachineId, IN_BSTR aSnapshotId);
+    STDMETHOD(OnSnapshotTaken)(IN_BSTR aMachineId, IN_BSTR aSnapshotId);
+    STDMETHOD(OnSnapshotDeleted)(IN_BSTR aMachineId, IN_BSTR aSnapshotId);
+    STDMETHOD(OnSnapshotChange)(IN_BSTR aMachineId, IN_BSTR aSnapshotId);
     STDMETHOD(OnGuestPropertyChange)(IN_BSTR machineId, IN_BSTR key, IN_BSTR value, IN_BSTR flags);
 
     // IConsoleCallback
     STDMETHOD(OnMousePointerShapeChange)(BOOL visible, BOOL alpha, ULONG xHot, ULONG yHot,
-                                         ULONG width, ULONG height, BYTE *shape);
-    STDMETHOD(OnMouseCapabilityChange)(BOOL supportsAbsolute, BOOL needsHostCursor);
+                                         ULONG width, ULONG height, ComSafeArrayIn(BYTE, shape));
+    STDMETHOD(OnMouseCapabilityChange)(BOOL supportsAbsolute, BOOL supportsRelative, BOOL needsHostCursor);
     STDMETHOD(OnKeyboardLedsChange)(BOOL fNumLock, BOOL fCapsLock, BOOL fScrollLock);
     STDMETHOD(OnStateChange)(MachineState_T machineState);
     STDMETHOD(OnAdditionsStateChange)();
-    STDMETHOD(OnNetworkAdapterChange) (INetworkAdapter *aNetworkAdapter);
-    STDMETHOD(OnSerialPortChange) (ISerialPort *aSerialPort);
-    STDMETHOD(OnParallelPortChange) (IParallelPort *aParallelPort);
+    STDMETHOD(OnNetworkAdapterChange)(INetworkAdapter *aNetworkAdapter);
+    STDMETHOD(OnSerialPortChange)(ISerialPort *aSerialPort);
+    STDMETHOD(OnParallelPortChange)(IParallelPort *aParallelPort);
     STDMETHOD(OnVRDPServerChange)();
     STDMETHOD(OnRemoteDisplayInfoChange)();
     STDMETHOD(OnUSBControllerChange)();
-    STDMETHOD(OnUSBDeviceStateChange) (IUSBDevice *aDevice, BOOL aAttached,
+    STDMETHOD(OnUSBDeviceStateChange)(IUSBDevice *aDevice, BOOL aAttached,
                                        IVirtualBoxErrorInfo *aError);
-    STDMETHOD(OnSharedFolderChange) (Scope_T aScope);
-    STDMETHOD(OnStorageControllerChange) ();
+    STDMETHOD(OnSharedFolderChange)(Scope_T aScope);
+    STDMETHOD(OnStorageControllerChange)();
     STDMETHOD(OnMediumChange)(IMediumAttachment *iMediumAttachment);
+    STDMETHOD(OnCPUChange)(ULONG aCPU, BOOL aRemove);
     STDMETHOD(OnRuntimeError)(BOOL fFatal, IN_BSTR id, IN_BSTR message);
     STDMETHOD(OnCanShowWindow)(BOOL *canShow);
-    STDMETHOD(OnShowWindow) (ULONG64 *winId);
+    STDMETHOD(OnShowWindow)(ULONG64 *winId);
 
     // for VirtualBoxSupportErrorInfoImpl
     static const wchar_t *getComponentName() { return L"CallbackWrapper"; }
