@@ -5,7 +5,7 @@
  */
 
 /*
- * Copyright (C) 2006-2007 Sun Microsystems, Inc.
+ * Copyright (C) 2006-2007 Oracle Corporation
  *
  * This file is part of VirtualBox Open Source Edition (OSE), as
  * available from http://www.virtualbox.org. This file is free software;
@@ -14,16 +14,12 @@
  * Foundation, in version 2 as it comes in the "COPYING" file of the
  * VirtualBox OSE distribution. VirtualBox OSE is distributed in the
  * hope that it will be useful, but WITHOUT ANY WARRANTY of any kind.
- *
- * Please contact Sun Microsystems, Inc., 4150 Network Circle, Santa
- * Clara, CA 95054 USA or visit http://www.sun.com if you need
- * additional information or have any questions.
  */
 
 /*******************************************************************************
 *   Header Files                                                               *
 *******************************************************************************/
-#include <iprt/runtime.h>
+#include <iprt/initterm.h>
 #include <VBox/vm.h>
 #include <VBox/err.h>
 #include <signal.h>
@@ -56,7 +52,7 @@ int main(int argc, char **argv)
      * Init runtime.
      */
     rc = RTR3Init();
-    if (!VBOX_SUCCESS(rc))
+    if (!RT_SUCCESS(rc))
     {
         printf("fatal error: failed to initialize runtime. (rc=%d)\n", rc);
         return 1;
@@ -99,13 +95,13 @@ int main(int argc, char **argv)
      */
     PVM pVM = NULL;
     rc = VMR3Create(pszVMConfig, &pVM);
-    if (VBOX_SUCCESS(rc))
+    if (RT_SUCCESS(rc))
     {
         /*
          * Run the VM.
          */
         rc = VMR3PowerOn(pVM);
-        if (VBOX_SUCCESS(rc))
+        if (RT_SUCCESS(rc))
         {
             /*
              * Wait for user to press Control-C,
@@ -132,7 +128,7 @@ int main(int argc, char **argv)
          * Destroy the VM.
          */
         int rc2 = VMR3Destroy(pVM);
-        if (!VBOX_SUCCESS(rc2))
+        if (!RT_SUCCESS(rc2))
         {
             printf("warning: VMR3Destroy() failed with rc=%d\n", rc);
             rc = 1;
