@@ -4,7 +4,7 @@
  */
 
 /*
- * Copyright (C) 2006-2009 Sun Microsystems, Inc.
+ * Copyright (C) 2006-2010 Oracle Corporation
  *
  * This file is part of VirtualBox Open Source Edition (OSE), as
  * available from http://www.virtualbox.org. This file is free software;
@@ -13,10 +13,6 @@
  * Foundation, in version 2 as it comes in the "COPYING" file of the
  * VirtualBox OSE distribution. VirtualBox OSE is distributed in the
  * hope that it will be useful, but WITHOUT ANY WARRANTY of any kind.
- *
- * Please contact Sun Microsystems, Inc., 4150 Network Circle, Santa
- * Clara, CA 95054 USA or visit http://www.sun.com if you need
- * additional information or have any questions.
  */
 
 #include <stdio.h>
@@ -419,7 +415,7 @@ int main(int argc, char *argv[])
                                COMGETTER(Uuids) (ComSafeArrayAsOutParam (uuids)));
 
             for (size_t i = 0; i < uuids.size(); ++ i)
-                RTPrintf ("uuids[%u]=%Vuuid\n", i, &uuids [i]);
+                RTPrintf ("uuids[%u]=%RTuuid\n", i, &uuids [i]);
         }
 
         {
@@ -430,7 +426,7 @@ int main(int argc, char *argv[])
                 Guid id;
                 id.create();
                 uuids [i] = id;
-                RTPrintf ("uuids[%u]=%Vuuid\n", i, &uuids [i]);
+                RTPrintf ("uuids[%u]=%RTuuid\n", i, &uuids [i]);
             }
 
             CHECK_ERROR_BREAK (virtualBox,
@@ -662,7 +658,7 @@ int main(int argc, char *argv[])
                 Bstr location;
                 CHECK_ERROR_BREAK (hd, COMGETTER(Id) (id.asOutParam()));
                 CHECK_ERROR_BREAK (hd, COMGETTER(Location) (location.asOutParam()));
-                RTPrintf ("Found, UUID={%Vuuid}, location='%ls'.\n",
+                RTPrintf ("Found, UUID={%RTuuid}, location='%ls'.\n",
                         id.raw(), location.raw());
 
                 com::SafeArray<BSTR> names;
@@ -741,7 +737,8 @@ int main(int argc, char *argv[])
 
         RTPrintf ("Creating a new machine object (base dir '%ls', name '%ls')...\n",
                 baseDir.raw(), name.raw());
-        CHECK_ERROR_BREAK (virtualBox, CreateMachine (baseDir, name,
+        CHECK_ERROR_BREAK (virtualBox, CreateMachine (name, L"", baseDir, L"",
+                                                      false,
                                                       machine.asOutParam()));
 
         RTPrintf ("Getting name...\n");
@@ -924,7 +921,7 @@ int main(int argc, char *argv[])
                 CHECK_ERROR_BREAK (disks [i], COMGETTER(Format) (format.asOutParam()));
 
                 RTPrintf (" disks[%u]: '%ls'\n"
-                        "  UUID:         {%Vuuid}\n"
+                        "  UUID:         {%RTuuid}\n"
                         "  State:        %s\n"
                         "  Format:       %ls\n",
                         i, loc.raw(), id.raw(),
@@ -959,7 +956,7 @@ int main(int argc, char *argv[])
                 {
                     for (size_t j = 0; j < ids.size(); ++ j)
                     {
-                        RTPrintf ("   {%Vuuid}\n", &ids [j]);
+                        RTPrintf ("   {%RTuuid}\n", &ids [j]);
                     }
                 }
             }
@@ -982,7 +979,7 @@ int main(int argc, char *argv[])
                 CHECK_ERROR_BREAK (images [i], COMGETTER(State) (&state));
 
                 RTPrintf (" images[%u]: '%ls'\n"
-                        "  UUID:         {%Vuuid}\n"
+                        "  UUID:         {%RTuuid}\n"
                         "  State:        %s\n",
                         i, loc.raw(), id.raw(),
                         state == MediaState_NotCreated ? "Not Created" :
@@ -1015,7 +1012,7 @@ int main(int argc, char *argv[])
                 {
                     for (size_t j = 0; j < ids.size(); ++ j)
                     {
-                        RTPrintf ("   {%Vuuid}\n", &ids [j]);
+                        RTPrintf ("   {%RTuuid}\n", &ids [j]);
                     }
                 }
             }
