@@ -208,8 +208,9 @@ enum
 
 static bool gAutoShutdown = false;
 
-static nsIEventQueue  *gEventQ      = nsnull;
-static PRBool volatile gKeepRunning = PR_TRUE;
+static nsIEventQueue  *gEventQ          = nsnull;
+static PRBool volatile gKeepRunning     = PR_TRUE;
+static PRBool volatile gAllowSigUsrQuit = PR_TRUE;
 
 /////////////////////////////////////////////////////////////////////////////
 
@@ -334,6 +335,8 @@ public:
                 Assert(onMainThread);
             }
 
+            gAllowSigUsrQuit = PR_TRUE;
+
             if (!timerStarted)
             {
                 if (!onMainThread)
@@ -531,6 +534,7 @@ public:
                 {
                     /* On success, make sure the previous timer is stopped to
                      * cancel a scheduled server termination (if any). */
+                    gAllowSigUsrQuit = PR_FALSE;
                     RTTimerLRStop(sTimer);
                 }
             }
@@ -550,6 +554,7 @@ public:
                 LogFlowFunc(("Another client has requested a reference to VirtualBox, canceling detruction...\n"));
 
                 /* make sure the previous timer is stopped */
+                gAllowSigUsrQuit = PR_FALSE;
                 RTTimerLRStop(sTimer);
             }
         }
@@ -727,13 +732,25 @@ class ForceQuitEvent : public MyEvent
     }
 };
 
-static void signal_handler(int /* sig */)
+static void signal_handler(int sig)
 {
     if (gEventQ && gKeepRunning)
     {
-        /* post a quit event to the queue */
-        ForceQuitEvent *ev = new ForceQuitEvent();
-        ev->postTo(gEventQ);
+        if (sig == SIGUSR1)
+        {
+            if (gAllowSigUsrQuit)
+            {
+                VirtualBoxClassFactory::MaybeQuitEvent *ev = new VirtualBoxClassFactory::MaybeQuitEvent();
+                ev->postTo(gEventQ);
+            }
+            /* else do nothing */
+        }
+        else
+        {
+            /* post a force quit event to the queue */
+            ForceQuitEvent *ev = new ForceQuitEvent();
+            ev->postTo(gEventQ);
+        }
     }
 }
 
@@ -1028,6 +1045,7 @@ int main(int argc, char **argv)
             sigaction(SIGQUIT, &sa, NULL);
             sigaction(SIGTERM, &sa, NULL);
             sigaction(SIGTRAP, &sa, NULL);
+            sigaction(SIGUSR1, &sa, NULL);
         }
 
         {
