@@ -17,6 +17,15 @@
    along with this program; if not, write to the Free Software
    Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
 */
+
+/*
+ * Sun GPL Disclaimer: For the avoidance of doubt, except that if any license choice
+ * other than GPL or LGPL is available it will apply instead, Sun elects to use only
+ * the General Public License version 2 (GPLv2) at this time for any software where
+ * a choice of GPL license versions is made available with the language indicating
+ * that GPLv2 or any later version may be used, or where a choice of which version
+ * of the GPL is applied is otherwise unspecified.
+ */
 #include <unistd.h>
 #include <fcntl.h>
 #include <termios.h>
@@ -38,7 +47,7 @@
 #ifdef WITH_DEBUG_SERIAL
 #define DEBUG_SERIAL(args) printf args;
 #else
-#define DEBUG_SERIAL(args)
+#define DEBUG_SERIAL(args) do {} while (0)
 #endif
 
 #define FILE_DEVICE_SERIAL_PORT		0x1b
