@@ -49,7 +49,7 @@ MALLOC_DEFINE(M_IPRTHEAP, "iprtheap", "IPRT - heap");
 MALLOC_DEFINE(M_IPRTCONT, "iprtcont", "IPRT - contiguous");
 
 
-PRTMEMHDR rtMemAlloc(size_t cb, uint32_t fFlags)
+PRTMEMHDR rtR0MemAlloc(size_t cb, uint32_t fFlags)
 {
     size_t cbAllocated = cb;
     PRTMEMHDR pHdr = NULL;
@@ -61,54 +61,26 @@ PRTMEMHDR rtMemAlloc(size_t cb, uint32_t fFlags)
 #ifdef RT_ARCH_AMD64
     if (fFlags & RTMEMHDR_FLAG_EXEC)
     {
+# ifdef USE_KMEM_ALLOC_PROT
+        pHdr = (PRTMEMHDR)kmem_alloc_prot(kernel_map, cb + sizeof(*pHdr),
+                                          VM_PROT_ALL, VM_PROT_ALL, KERNBASE);
+# else
+        vm_object_t pVmObject = NULL;
         vm_offset_t Addr = KERNBASE;
         cbAllocated = RT_ALIGN_Z(cb + sizeof(*pHdr), PAGE_SIZE);
 
+        pVmObject = vm_object_allocate(OBJT_DEFAULT, cbAllocated >> PAGE_SHIFT);
+        if (!pVmObject)
+            return NULL;
+
         /* Addr contains a start address vm_map_find will start searching for suitable space at. */
-        int rc = vm_map_find(kernel_map, NULL, 0, &Addr,
-                             cbAllocated,
-                             TRUE, VM_PROT_ALL, VM_PROT_ALL, 0);
+        int rc = vm_map_find(kernel_map, pVmObject, 0, &Addr,
+                             cbAllocated, TRUE, VM_PROT_ALL, VM_PROT_ALL, 0);
         if (rc == KERN_SUCCESS)
         {
-            /* Add the pages. */
-            vm_offset_t AddressDst = Addr;
-            bool fSuccess = true;
-
-            do
-            {
-                vm_pindex_t PageIndex = OFF_TO_IDX(AddressDst);
-                vm_page_t   pPage;
-
-                pPage = vm_page_alloc(NULL, PageIndex,
-                                      VM_ALLOC_NOBUSY | VM_ALLOC_SYSTEM |
-                                      VM_ALLOC_WIRED  | VM_ALLOC_NOOBJ);
-                if (pPage)
-                {
-                    vm_page_lock_queues();
-                    vm_page_wire(pPage);
-                    vm_page_unlock_queues();
-                    /* Put the page into the page table now. */
-#if __FreeBSD_version >= 701105
-                    pmap_enter(kernel_map->pmap, AddressDst, VM_PROT_NONE, pPage,
-                               VM_PROT_ALL, TRUE);
-#else
-                    pmap_enter(kernel_map->pmap, AddressDst, pPage,
-                               VM_PROT_ALL, TRUE);
-#endif
-                }
-                else
-                {
-                    /*
-                     * Allocation failed. vm_map_remove will remove any
-                     * page already allocated.
-                     */
-                    fSuccess = false;
-                    break;
-                }
-                AddressDst += PAGE_SIZE;
-            } while(AddressDst < (Addr + cbAllocated));
-
-            if (fSuccess)
+            rc = vm_map_wire(kernel_map, Addr, Addr + cbAllocated,
+                             VM_MAP_WIRE_SYSTEM | VM_MAP_WIRE_NOHOLES);
+            if (rc == KERN_SUCCESS)
             {
                 pHdr = (PRTMEMHDR)Addr;
 
@@ -118,8 +90,11 @@ PRTMEMHDR rtMemAlloc(size_t cb, uint32_t fFlags)
             else
                 vm_map_remove(kernel_map,
                               Addr,
-                              Addr + cb);
+                              Addr + cbAllocated);
         }
+        else
+            vm_object_deallocate(pVmObject);
+# endif
     }
     else
 #endif
@@ -140,13 +115,17 @@ PRTMEMHDR rtMemAlloc(size_t cb, uint32_t fFlags)
 }
 
 
-void rtMemFree(PRTMEMHDR pHdr)
+void rtR0MemFree(PRTMEMHDR pHdr)
 {
     pHdr->u32Magic += 1;
 
 #ifdef RT_ARCH_AMD64
     if (pHdr->fFlags & RTMEMHDR_FLAG_EXEC)
+# ifdef USE_KMEM_ALLOC_PROT
+        kmem_free(kernel_map, (vm_offset_t)pHdr, pHdr->cb);
+# else
         vm_map_remove(kernel_map, (vm_offset_t)pHdr, ((vm_offset_t)pHdr) + pHdr->cb);
+# endif
     else
 #endif
         free(pHdr, M_IPRTHEAP);
