@@ -4,7 +4,7 @@
  */
 
 /*
- * Copyright (C) 2006-2007 Sun Microsystems, Inc.
+ * Copyright (C) 2006-2007 Oracle Corporation
  *
  * This file is part of VirtualBox Open Source Edition (OSE), as
  * available from http://www.virtualbox.org. This file is free software;
@@ -22,10 +22,6 @@
  *
  * You may elect to license modified versions of this file under the
  * terms and conditions of either the GPL or the CDDL or both.
- *
- * Please contact Sun Microsystems, Inc., 4150 Network Circle, Santa
- * Clara, CA 95054 USA or visit http://www.sun.com if you need
- * additional information or have any questions.
  */
 
 
@@ -97,7 +93,8 @@ RTR3DECL(RTPROCPRIORITY) RTProcGetPriority(void)
 
 RTR3DECL(char *) RTProcGetExecutableName(char *pszExecName, size_t cchExecName)
 {
-    AssertReturn(g_szrtProcExePath[0] != '\0', NULL);
+    if (RT_UNLIKELY(g_szrtProcExePath[0] == '\0'))
+        return NULL;
 
     /*
      * Calc the length and check if there is space before copying.
