@@ -44,12 +44,14 @@
 # include <netdb.h>
 # include <unistd.h>
 # include <fcntl.h>
+# include <sys/uio.h>
 #endif /* !RT_OS_WINDOWS */
 #include <limits.h>
 
 #include "internal/iprt.h"
 #include <iprt/socket.h>
 
+#include <iprt/alloca.h>
 #include <iprt/asm.h>
 #include <iprt/assert.h>
 #include <iprt/err.h>
@@ -58,6 +60,8 @@
 #include <iprt/string.h>
 #include <iprt/thread.h>
 #include <iprt/time.h>
+#include <iprt/mem.h>
+#include <iprt/sg.h>
 
 #include "internal/magics.h"
 #include "internal/socket.h"
@@ -573,6 +577,112 @@ RTDECL(int) RTSocketWrite(RTSOCKET hSocket, const void *pvBuffer, size_t cbBuffe
 }
 
 
+RTDECL(int) RTSocketSgWrite(RTSOCKET hSocket, PCRTSGBUF pSgBuf)
+{
+    /*
+     * Validate input.
+     */
+    RTSOCKETINT *pThis = hSocket;
+    AssertPtrReturn(pThis, VERR_INVALID_HANDLE);
+    AssertReturn(pThis->u32Magic == RTSOCKET_MAGIC, VERR_INVALID_HANDLE);
+    AssertPtrReturn(pSgBuf, VERR_INVALID_PARAMETER);
+    AssertReturn(pSgBuf->cSegs > 0, VERR_INVALID_PARAMETER);
+    AssertReturn(rtSocketTryLock(pThis), VERR_CONCURRENT_ACCESS);
+
+    /*
+     * Construct message descriptor (translate pSgBuf) and send it.
+     */
+    int rc = VERR_NO_TMP_MEMORY;
+#ifdef RT_OS_WINDOWS
+    AssertCompileSize(WSABUF, sizeof(RTSGSEG));
+    AssertCompileMemberSize(WSABUF, buf, RT_SIZEOFMEMB(RTSGSEG, pvSeg));
+
+    LPWSABUF paMsg = (LPWSABUF)RTMemTmpAllocZ(pSgBuf->cSegs * sizeof(WSABUF));
+    if (paMsg)
+    {
+        for (unsigned i = 0; i < pSgBuf->cSegs; i++)
+        {
+            paMsg[i].buf = (char *)pSgBuf->paSegs[i].pvSeg;
+            paMsg[i].len = (u_long)pSgBuf->paSegs[i].cbSeg;
+        }
+
+        DWORD dwSent;
+        int hrc = WSASend(pThis->hNative, paMsg, pSgBuf->cSegs, &dwSent,
+                          MSG_NOSIGNAL, NULL, NULL);
+        if (!hrc)
+            rc = VINF_SUCCESS;
+/** @todo check for incomplete writes */
+        else
+            rc = rtSocketError();
+
+        RTMemTmpFree(paMsg);
+    }
+
+#else  /* !RT_OS_WINDOWS */
+    AssertCompileSize(struct iovec, sizeof(RTSGSEG));
+    AssertCompileMemberSize(struct iovec, iov_base, RT_SIZEOFMEMB(RTSGSEG, pvSeg));
+    AssertCompileMemberSize(struct iovec, iov_len,  RT_SIZEOFMEMB(RTSGSEG, cbSeg));
+
+    struct iovec *paMsg = (struct iovec *)RTMemTmpAllocZ(pSgBuf->cSegs * sizeof(struct iovec));
+    if (paMsg)
+    {
+        for (unsigned i = 0; i < pSgBuf->cSegs; i++)
+        {
+            paMsg[i].iov_base = pSgBuf->paSegs[i].pvSeg;
+            paMsg[i].iov_len  = pSgBuf->paSegs[i].cbSeg;
+        }
+
+        struct msghdr msgHdr;
+        RT_ZERO(msgHdr);
+        msgHdr.msg_iov    = paMsg;
+        msgHdr.msg_iovlen = pSgBuf->cSegs;
+        ssize_t cbWritten = sendmsg(pThis->hNative, &msgHdr, MSG_NOSIGNAL);
+        if (RT_LIKELY(cbWritten >= 0))
+            rc = VINF_SUCCESS;
+/** @todo check for incomplete writes */
+        else
+            rc = rtSocketError();
+
+        RTMemTmpFree(paMsg);
+    }
+#endif /* !RT_OS_WINDOWS */
+
+    rtSocketUnlock(pThis);
+    return rc;
+}
+
+
+RTDECL(int) RTSocketSgWriteL(RTSOCKET hSocket, size_t cSegs, ...)
+{
+    va_list va;
+    va_start(va, cSegs);
+    int rc = RTSocketSgWriteLV(hSocket, cSegs, va);
+    va_end(va);
+    return rc;
+}
+
+
+RTDECL(int) RTSocketSgWriteLV(RTSOCKET hSocket, size_t cSegs, va_list va)
+{
+    /*
+     * Set up a S/G segment array + buffer on the stack and pass it
+     * on to RTSocketSgWrite.
+     */
+    Assert(cSegs <= 16);
+    PRTSGSEG paSegs = (PRTSGSEG)alloca(cSegs * sizeof(RTSGSEG));
+    AssertReturn(paSegs, VERR_NO_TMP_MEMORY);
+    for (size_t i = 0; i < cSegs; i++)
+    {
+        paSegs[i].pvSeg = va_arg(va, void *);
+        paSegs[i].cbSeg = va_arg(va, size_t);
+    }
+
+    RTSGBUF SgBuf;
+    RTSgBufInit(&SgBuf, paSegs, cSegs);
+    return RTSocketSgWrite(hSocket, &SgBuf);
+}
+
+
 RTDECL(int) RTSocketSelectOne(RTSOCKET hSocket, RTMSINTERVAL cMillies)
 {
     /*
