@@ -4,7 +4,7 @@
  */
 
 /*
- * Copyright (C) 2006-2007 Sun Microsystems, Inc.
+ * Copyright (C) 2006-2007 Oracle Corporation
  *
  * This file is part of VirtualBox Open Source Edition (OSE), as
  * available from http://www.virtualbox.org. This file is free software;
@@ -22,10 +22,6 @@
  *
  * You may elect to license modified versions of this file under the
  * terms and conditions of either the GPL or the CDDL or both.
- *
- * Please contact Sun Microsystems, Inc., 4150 Network Circle, Santa
- * Clara, CA 95054 USA or visit http://www.sun.com if you need
- * additional information or have any questions.
  */
 
 
@@ -127,7 +123,13 @@ RTDECL(int) RTDirRemove(const char *pszPath)
 }
 
 
-int rtOpenDirNative(PRTDIR pDir, char *pszPathBuf)
+RTDECL(int) RTDirFlush(const char *pszPath)
+{
+    return VERR_NOT_SUPPORTED;
+}
+
+
+int rtDirNativeOpen(PRTDIR pDir, char *pszPathBuf)
 {
     /*
      * Setup the search expression.
@@ -157,15 +159,11 @@ int rtOpenDirNative(PRTDIR pDir, char *pszPathBuf)
      * Attempt opening the search.
      */
     int rc = VINF_SUCCESS;
-#ifndef RT_DONT_CONVERT_FILENAMES
     PRTUTF16 pwszName;
     rc = RTStrToUtf16(pszPathBuf, &pwszName);
     if (RT_SUCCESS(rc))
     {
         pDir->hDir    = FindFirstFileW((LPCWSTR)pwszName, &pDir->Data);
-#else
-        pDir->hDir    = FindFirstFileA(pszPathBuf, &pDir->Data);
-#endif
         if (pDir->hDir != INVALID_HANDLE_VALUE)
             pDir->fDataUnread = true;
         /* theoretical case of an empty directory. */
@@ -173,10 +171,8 @@ int rtOpenDirNative(PRTDIR pDir, char *pszPathBuf)
             pDir->fDataUnread = false;
         else
             rc = RTErrConvertFromWin32(GetLastError());
-#ifndef RT_DONT_CONVERT_FILENAMES
         RTUtf16Free(pwszName);
     }
-#endif
 
     return rc;
 }
@@ -244,15 +240,10 @@ RTDECL(int) RTDirRead(PRTDIR pDir, PRTDIRENTRY pDirEntry, size_t *pcbDirEntry)
      */
     if (!pDir->fDataUnread)
     {
-#ifdef RT_DONT_CONVERT_FILENAMES
-        BOOL fRc = FindNextFileA(pDir->hDir, &pDir->Data);
-
-#else
         RTStrFree(pDir->pszName);
         pDir->pszName = NULL;
 
         BOOL fRc = FindNextFileW(pDir->hDir, &pDir->Data);
-#endif
         if (!fRc)
         {
             int iErr = GetLastError();
@@ -262,7 +253,6 @@ RTDECL(int) RTDirRead(PRTDIR pDir, PRTDIRENTRY pDirEntry, size_t *pcbDirEntry)
         }
     }
 
-#ifndef RT_DONT_CONVERT_FILENAMES
     /*
      * Convert the filename to UTF-8.
      */
@@ -276,18 +266,12 @@ RTDECL(int) RTDirRead(PRTDIR pDir, PRTDIRENTRY pDirEntry, size_t *pcbDirEntry)
         }
         pDir->cchName = strlen(pDir->pszName);
     }
-#endif
 
     /*
      * Check if we've got enough space to return the data.
      */
-#ifdef RT_DONT_CONVERT_FILENAMES
-    const char  *pszName    = pDir->Data.cName;
-    const size_t cchName    = strlen(pszName);
-#else
     const char  *pszName    = pDir->pszName;
     const size_t cchName    = pDir->cchName;
-#endif
     const size_t cbRequired = RT_OFFSETOF(RTDIRENTRY, szName[1]) + cchName;
     if (pcbDirEntry)
         *pcbDirEntry = cbRequired;
@@ -349,15 +333,10 @@ RTDECL(int) RTDirReadEx(PRTDIR pDir, PRTDIRENTRYEX pDirEntry, size_t *pcbDirEntr
      */
     if (!pDir->fDataUnread)
     {
-#ifdef RT_DONT_CONVERT_FILENAMES
-        BOOL fRc = FindNextFileA(pDir->hDir, &pDir->Data);
-
-#else
         RTStrFree(pDir->pszName);
         pDir->pszName = NULL;
 
         BOOL fRc = FindNextFileW(pDir->hDir, &pDir->Data);
-#endif
         if (!fRc)
         {
             int iErr = GetLastError();
@@ -367,7 +346,6 @@ RTDECL(int) RTDirReadEx(PRTDIR pDir, PRTDIRENTRYEX pDirEntry, size_t *pcbDirEntr
         }
     }
 
-#ifndef RT_DONT_CONVERT_FILENAMES
     /*
      * Convert the filename to UTF-8.
      */
@@ -381,18 +359,12 @@ RTDECL(int) RTDirReadEx(PRTDIR pDir, PRTDIRENTRYEX pDirEntry, size_t *pcbDirEntr
         }
         pDir->cchName = strlen(pDir->pszName);
     }
-#endif
 
     /*
      * Check if we've got enough space to return the data.
      */
-#ifdef RT_DONT_CONVERT_FILENAMES
-    const char  *pszName    = pDir->Data.cName;
-    const size_t cchName    = strlen(pszName);
-#else
     const char  *pszName    = pDir->pszName;
     const size_t cchName    = pDir->cchName;
-#endif
     const size_t cbRequired = RT_OFFSETOF(RTDIRENTRYEX, szName[1]) + cchName;
     if (pcbDirEntry)
         *pcbDirEntry = cbRequired;
@@ -406,7 +378,6 @@ RTDECL(int) RTDirReadEx(PRTDIR pDir, PRTDIRENTRYEX pDirEntry, size_t *pcbDirEntr
     pDirEntry->cbName  = (uint16_t)cchName;
     Assert(pDirEntry->cbName == cchName);
     memcpy(pDirEntry->szName, pszName, cchName + 1);
-#ifndef RT_DONT_CONVERT_FILENAMES /* this ain't nice since the whole point of this define is not to drag in conversion... */
     if (pDir->Data.cAlternateFileName[0])
     {
         /* copy and calc length */
@@ -421,7 +392,6 @@ RTDECL(int) RTDirReadEx(PRTDIR pDir, PRTDIRENTRYEX pDirEntry, size_t *pcbDirEntr
             *pwszDst++ = '\0';
     }
     else
-#endif
     {
         memset(pDirEntry->wszShortName, 0, sizeof(pDirEntry->wszShortName));
         pDirEntry->cwcShortName = 0;
