@@ -4,7 +4,7 @@
  */
 
 /*
- * Copyright (C) 2006-2007 Sun Microsystems, Inc.
+ * Copyright (C) 2006-2007 Oracle Corporation
  *
  * This file is part of VirtualBox Open Source Edition (OSE), as
  * available from http://www.virtualbox.org. This file is free software;
@@ -13,10 +13,6 @@
  * Foundation, in version 2 as it comes in the "COPYING" file of the
  * VirtualBox OSE distribution. VirtualBox OSE is distributed in the
  * hope that it will be useful, but WITHOUT ANY WARRANTY of any kind.
- *
- * Please contact Sun Microsystems, Inc., 4150 Network Circle, Santa
- * Clara, CA 95054 USA or visit http://www.sun.com if you need
- * additional information or have any questions.
  */
 
 
@@ -118,13 +114,14 @@ VMMDECL(RTGCUINTREG) DBGFBpGetDR3(PVM pVM)
 
 
 /**
- * Returns single stepping state
+ * Returns the single stepping state for a virtual CPU.
  *
- * @returns stepping or not
- * @param   pVM         The VM handle.
+ * @returns stepping (true) or not (false).
+ *
+ * @param   pVCpu       The virtual CPU handle.
  */
-VMMDECL(bool) DBGFIsStepping(PVM pVM)
+VMMDECL(bool) DBGFIsStepping(PVMCPU pVCpu)
 {
-    return pVM->dbgf.s.fSingleSteppingRaw;
+    return pVCpu->dbgf.s.fSingleSteppingRaw;
 }
 
